package de.uni_goettingen.med.mi.bdtparser.core;

import java.util.LinkedList;
import java.util.concurrent.LinkedTransferQueue;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datacarrier.Datacarrier;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datapackage.Datapackage;
import de.uni_goettingen.med.mi.bdtparser.core.io.reader.BdtFileReader;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;

/**
 * This class is used to take the elements from a queue and store them in a bdt-like data structures
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTMarshaller implements Runnable {

    public Datapackage datapackage; // The reference to the datacarriers after the marshalling process
    private final static Logger log = LogManager.getLogger(BDTMarshaller.class);
    private LinkedTransferQueue<String> queue;
    private Thread readerThread;
    private static final LinkedList<Datacarrier> datacarriers = new LinkedList<>();
    private BdtFileReader reader;
    public DoubleProperty processProperty;
    private long numberOfLines;
    private boolean linesStartWithQuotes = false;

    /**
     * Constructor for the Marshaller which requests the queue holding the bdt lines from the reader
     *
     * @param readerThread the thread which holds the reader, may not be null
     * @param reader the reader which is reading in the bdt-file, may not be null
     */
    public BDTMarshaller(Thread readerThread, BdtFileReader reader) {
        this();
        processProperty = new SimpleDoubleProperty();
        this.queue = reader.getQueue();
        this.readerThread = readerThread;
        this.reader = reader;
    }

    private BDTMarshaller() {
        datapackage = new Datapackage();
    }

    /**
     * read the lines from the given queue, and pack them into the data carrier
     * and make the openly available
     */
    public void run() {
        int progress = 0;
        while (true) {
            String tmp = null;
            try {
                if (readerThread.getState() == Thread.State.TERMINATED && queue.peek() == null)
                    break;
                tmp = queue.take();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            if (tmp.charAt(0) == '"') { // remove "" at the begin and the end, add log entry
                String warnMsg;
                if (tmp.charAt(tmp.length() - 1) == '"') {
                    tmp = tmp.substring(1, tmp.length() - 1);
                    warnMsg = "Found \" at the start and end of the line. Removing it.";
                } else {
                    tmp = tmp.substring(1);
                    warnMsg = "Found \" at the start of the line. Removing it.";
                }
                log.warn(warnMsg);
                linesStartWithQuotes = true;
            }
            if (datacarriers.size() == 0) // add to data package, create a new one if it's needed
                datacarriers.add(new Datacarrier(tmp));
            else if (datacarriers.get(datacarriers.size() - 1).add(tmp) < 1) // if <1 --> create new package and add the field again. >=1 field successfully added
                datacarriers.add(new Datacarrier(tmp));
            progress++; // update progressbar
            processProperty.set((double) progress / (double) reader.getNumberOfLines());
        }
        datapackage.setDatacarriers(datacarriers);
        numberOfLines = reader.getNumberOfLines();
    }

    /**
     * Method to get the number of lines that were read in while reading the bdt-files
     * @return the number of lines
     */
    public long getNumberOfLines() {
        return numberOfLines;
    }

    /**
     * Method to read the flag if quotes at the beginning of the lines were removed.
     * @return the flag
     */
    public boolean isLinesStartWithQuotes() {
        return linesStartWithQuotes;
    }
}
