package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datacarrier;

import java.util.ArrayList;
import java.util.concurrent.atomic.AtomicInteger;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.BdtLine;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.Sentence;

/**
 * This class is used to store a data carrier of a bdt file
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class Datacarrier {

    public boolean used;
    public final int ID; // The unique ID from this Datacarrier
    private static AtomicInteger idCounter = new AtomicInteger();
    private long size;// the size of the Datacarrier in bytes
    private ArrayList<Sentence> sentences;
    private boolean EOC = false;// end of carrier

    /**
     * Default constructor creates a new ID and creates a sentence list
     */
    public Datacarrier() {
        this(idCounter.getAndIncrement());

    }

    /**
     * sets the id of the data carrier and created an empty list for the sentences
     * @param id the id of the data carrier
     */
    private Datacarrier(int id) {
        ID = id;
        sentences = new ArrayList<Sentence>();
        size = 0;
    }

    /**
     * Calls the default constructor and adds given Bdtline
     *
     * @param bdt_line the line to add
     */
    public Datacarrier(String bdt_line) {
        this();
        add(bdt_line);

    }

    // ---- public Methods

    /**
     * The size of the datacarrier stored in the bdt file
     *
     * @return the size of the datacarrier
     */
    public long getSize() {
        return size;
    }

    /**
     * Returns the Sentences belonging to this sentence
     *
     * @return
     */
    public ArrayList<Sentence> getSentences() {
        return sentences;
    }

    /**
     * adds the given line/field to the Datacarrier, returns -1 if a new Datacarrier one should be created
     *
     * @param bdt_line the line that should be added
     * @return the size of the added line or -1 if it failed and a new Datacarrier should be created
     */
    public int add(String bdt_line) {
        return add(new BdtLine(bdt_line));
    }

    /**
     * adds the given line/field to the Datacarrier, returns -1 if a new Datacarrier one should be created
     *
     * @param bdt_Line the line that should be added
     * @return the size of the added line or -1 if it failed and a new Datacarrier should be created
     */
    public int add(BdtLine bdt_Line) {
        if (Sentence.isSentence(bdt_Line)) {
            if (bdt_Line.getData().equals("0021") && !EOC)
                EOC = true;
            else if (EOC && bdt_Line.getData().equals("0020")) {
                EOC = false;
                return -1;
            }
        }
        return addField(bdt_Line);

    }

    /**
     * sets the size of the Datacarrier
     *
     * @param size the new size
     */
    public void setSize(String size) {
        setSize(Integer.parseInt(size));
    }

    /**
     * sets the size of the Datacarrier
     *
     * @param size the new size
     */
    private void setSize(long size) {
        this.size = size;
    }

    /// private methods

    /**
     * adds a Field the senctence and creates a new sentence if needed
     *
     * @param bdtLine the line/field that should be added
     * @return the size of the field
     */
    private int addField(BdtLine bdtLine) {
        if (Sentence.isSentence(bdtLine) || sentences.size() == 0) { // new Sentence or
            // sentences.add(new Sentence(bdtLine,this));
            sentences.add(new Sentence());
        }
        return sentences.get(sentences.size() - 1).addField(bdtLine, this);
    }

}
