package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.BdtLine;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datacarrier.Datacarrier;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic;

import java.util.LinkedList;
import java.util.concurrent.atomic.AtomicLong;

/**
 * This class is used to store a sentence of a bdt file
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class Sentence {

    public final long ID; // unique ID of the sentence
    public boolean used;
    private static final AtomicLong idCounter = new AtomicLong();
    private LinkedList<Field> fields;
    private long size;
    private Datacarrier datacarrier;

    /**
     * Default constructor
     * generates a unique id and  creates a new list to store the fields
     */
    public Sentence() {
        ID = idCounter.getAndIncrement();
        fields = new LinkedList<>();
    }

    /**
     * Constructor that uses the given id and datacarrier to create the sentence
     * @param id the id that should be used for the sentence
     * @param datacarrier the data carrier to which the sentence belongs
     */
    public Sentence(long id, Datacarrier datacarrier) {
        fields = new LinkedList<>();
        ID = id;
        this.datacarrier = datacarrier;
    }

    /**
     * Constructor which also accept a field to add to the sentence
     *
     * @param line the Field that should be added
     * @param datacarrier the corresponding datacarrier
     */
    public Sentence(BdtLine line, Datacarrier datacarrier) {
        this();
        this.datacarrier = datacarrier;
        addField(line, datacarrier);
    }

    /**
     * adds a Field to this sentence
     *
     * @param bdtLineLine the bdtline that should be added
     * @param datacarrier the corresponding datacarrier
     * @return the the length of the added field
     */
    public int addField(BdtLine bdtLineLine, Datacarrier datacarrier) {
        this.datacarrier = datacarrier;
        Field field = new Field(bdtLineLine, this);
        fields.add(field);
        if (bdtLineLine.getFieldNumber().equals(FieldCharacteristic.Satzlaenge.getNumber()))
            setSize(bdtLineLine.getData());
        if (bdtLineLine.getFieldNumber().equals(FieldCharacteristic.GesamtlaengeDesDatenpaketesInByte.getNumber()))
            datacarrier.setSize(bdtLineLine.getData());
        return Integer.parseInt(field.getLength());
    }

    /**
     * Adds a Field to the sentence ,doesnt updated
     *
     * @param field
     */
    public void addField(Field field) {
        fields.add(field);

    }

    /**
     * checks if the given line is the beginning of a new sentence
     *
     * @param bdtLine the line to check
     * @return the result of the check
     */
    public static boolean isSentence(BdtLine bdtLine) {
        return bdtLine.getFieldNumber().equals(FieldCharacteristic.Satzidentifikation.getNumber());
    }

    /**
     * Returns a list to the fields in the sentence
     * returns the real references and not a copy
     *
     * @return the list of fields
     */
    public LinkedList<Field> getFields() {
        return fields;
    }

    /**
     * Method to set the list of the fields in this sentence
     *
     * @param fields the list of Fields
     */
    public void setFields(LinkedList<Field> fields) {
        this.fields = fields;
    }

    /**
     * The size of the sentence that is stored in the bdt file
     *
     * @return the size
     */
    public long getSize() {
        return size;
    }

    /**
     * Setter for the Size
     *
     * @param size the new size
     */
    public void setSize(String size) {
        setSize(Integer.parseInt(size));
    }

    /**
     * Setter for the size
     *
     * @param size the new size
     */
    public void setSize(long size) {
        this.size = size;
    }

    /**
     * Returns the id that identifys the type of the sentence
     *
     * @return the ID of the Sentence
     */
    public String getSentenceType() throws IllegalSentenceStructureException {
        if (fields.isEmpty())
            throw new IllegalSentenceStructureException("Cant found sentence identifier");
        return fields.getFirst().getData();
    }

    /**
     * checks if the sentence contains a field with the given id
     *
     * @param fieldId the id of the field to search for
     * @return true, if the sentence contains the field, false otherwise
     */
    public boolean contains(String fieldId) {
        for (Field field : fields) {
            if (field.getNumber().equals(fieldId))
                return true;
        }
        return false;
    }

    /**
     * returns the first field found witch matching ID
     *
     * @param fieldId the id of the field to search for
     * @return the first field found or null if no field was found
     */
    public Field getField(String fieldId) {
        for (Field field : fields) {
            if (field.getNumber().equals(fieldId))
                return field;
        }
        return null;
    }

    /**
     * Getter for the datacarrier
     *
     * @return the datacarrier of this sentence
     */
    public Datacarrier getDatacarrier() {
        return datacarrier;
    }
}
