package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Enum for all the known sentence types, including the SubSentences structure to hold all accepted fields
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public enum SentenceCharacteristic {
    Datentraeger_Header("0020"), 
    Datentraeger_Abschluss("0021"), 
    Datenpaket_Header("0022"), 
    Datenpaket_Abschluss("0023"), 
    Praxisdaten("0010"),
    AerztlicheBehandlung("0101"), 
    Ueberweisungsfall("0102"), 
    BelegeaerztlicheBehandlung("0103"), 
    Notfalldienst_Vertretung_Notfalll("0104"), 
    Privatabrechnung("0190"), 
    BG_Abrechnung("0191"), 
    UnstrukturierteFaelle("0199"), 
    Patientenstamm("6100"), 
    Behandlungsdaten("6200");

    public final String id;
    private ArrayList<SubSentence> subSentence;

    /**
     * delivers a deep copie from the list of subsentences
     *
     * @return the list of subsentences
     */
    public ArrayList<SubSentence> getSubSenteces() {
        // return subSentence;
        ArrayList<SubSentence> clonedList = new ArrayList<>(subSentence.size());
        clonedList.addAll(subSentence);
        // for (SubSentence subSentence : this.subSentence) {
        // clonedList.add((SubSentence) subSentence.clone());
        // }
        return clonedList;
    }

    /**
     * Returns an Interator over this SentenceCharacteristic.<br/>
     * Check {@link SentenceIterator} for more information about the iterator.
     *
     * @return the SentenceIterator
     */
    public SentenceIterator getSentenceInterator() {
        return new SentenceIterator(this);
    }

    /**
     * Returns an Interator over the toplevel SubSentences
     *
     * @return the Iterator over list of SubSentences
     */
    protected Iterator<SubSentence> getIterator() {
        return subSentence.listIterator();
    }

    SentenceCharacteristic(String id) {
        this.id = id;
        this.subSentence = new ArrayList<>();
    }

    /**
     * private methode used in the static block to add the subsenteces (wich are Fields which could be followed by more fields on the next level)
     * adds a reference from the subsentence, which is one level higher, to each subsentece
     *
     * @param level the level of the field
     * @param subSentence the field that should be added
     */
    private void addField(int level, SubSentence subSentence) {
        switch (level) {
        case 1:
            this.subSentence.add(subSentence);
            subSentence.setSentenceCharacteristic(this);
            break;
        case 2:
            subSentence.setHigherLevelSubSentence(this.subSentence.get(this.subSentence.size() - 1));
            this.subSentence.get(this.subSentence.size() - 1).addCharacteristic(subSentence);
            break;
        case 3:
            subSentence.setHigherLevelSubSentence(this.subSentence.get(this.subSentence.size() - 1).getLastSubSentence());
            this.subSentence.get(this.subSentence.size() - 1).getLastSubSentence().addCharacteristic(subSentence);
            break;
        case 4:
            subSentence.setHigherLevelSubSentence(this.subSentence.get(this.subSentence.size() - 1).getLastSubSentence().getLastSubSentence());
            this.subSentence.get(this.subSentence.size() - 1).getLastSubSentence().getLastSubSentence().addCharacteristic(subSentence);
            break;
        default:
            throw new IllegalArgumentException("level must be between 1 and 4, was " + level);
        }
    }

    /*
     * static creation of the subsentences, change only if the set of subsentences has been modified,
     * codeblock was created from the the sentencetable from the specifiaction
     */
    static {
        /*
        Datentraeger Header 0020;
         */
        Datentraeger_Header.addField(1, new SubSentence(8000, false, true));
        Datentraeger_Header.addField(1, new SubSentence(8100, false, true));
        Datentraeger_Header.addField(1, new SubSentence(9100, false, true));
        Datentraeger_Header.addField(1, new SubSentence(9103, false, true));
        Datentraeger_Header.addField(1, new SubSentence(9105, false, true));
        Datentraeger_Header.addField(1, new SubSentence(9106, false, true));

        /*
        Datentraeger Abschluss 0021
         */
        Datentraeger_Abschluss.addField(1, new SubSentence(8000, false, true));
        Datentraeger_Abschluss.addField(1, new SubSentence(8100, false, true));

        /*
        Datenpaket Header 0022
         */
        Datenpaket_Header.addField(1, new SubSentence(8000, false, true));
        Datenpaket_Header.addField(1, new SubSentence(8100, false, true));
        Datenpaket_Header.addField(1, new SubSentence(9210, false, true));
        Datenpaket_Header.addField(1, new SubSentence(9213, false, true));
        Datenpaket_Header.addField(1, new SubSentence(9600, false, true));
        Datenpaket_Header.addField(1, new SubSentence(9601, false, true));
        Datenpaket_Header.addField(1, new SubSentence(9602, false, true));

        /*
        Datenpaket Abschluss  0023
         */
        Datenpaket_Abschluss.addField(1, new SubSentence(8000, false, true));
        Datenpaket_Abschluss.addField(1, new SubSentence(8100, false, true));
        Datenpaket_Abschluss.addField(1, new SubSentence(9202, false, true));
        Datenpaket_Abschluss.addField(1, new SubSentence(9203, false, true));

        /*
        Praxisdaten 0010
         */
        Praxisdaten.addField(1, new SubSentence(8000, false, true));
        Praxisdaten.addField(1, new SubSentence(8100, false, true));
        Praxisdaten.addField(1, new SubSentence(101, false, true));
        Praxisdaten.addField(1, new SubSentence(102, false, true));
        Praxisdaten.addField(1, new SubSentence(103, false, true));
        Praxisdaten.addField(1, new SubSentence(104, false, true));
        Praxisdaten.addField(1, new SubSentence(201, false, true));
        Praxisdaten.addField(1, new SubSentence(202, false, true));
        Praxisdaten.addField(1, new SubSentence(203, true, true));
        Praxisdaten.addField(2, new SubSentence(204, false, true));
        Praxisdaten.addField(1, new SubSentence(205, false, true));
        Praxisdaten.addField(1, new SubSentence(206, false, true));
        Praxisdaten.addField(1, new SubSentence(207, true, false));//bedingtes muss
        Praxisdaten.addField(1, new SubSentence(208, false, true));
        Praxisdaten.addField(1, new SubSentence(209, false, false));
        Praxisdaten.addField(1, new SubSentence(210, false, false));
        Praxisdaten.addField(1, new SubSentence(225, false, false));//bedingtes muss
        Praxisdaten.addField(1, new SubSentence(250, false, false));
        Praxisdaten.addField(2, new SubSentence(251, true, false));
        Praxisdaten.addField(1, new SubSentence(252, false, false));
        Praxisdaten.addField(2, new SubSentence(253, true, false));
        Praxisdaten.addField(1, new SubSentence(254, false, false));
        Praxisdaten.addField(2, new SubSentence(255, true, false));
        Praxisdaten.addField(1, new SubSentence(256, false, false));
        Praxisdaten.addField(2, new SubSentence(257, true, false));
        Praxisdaten.addField(1, new SubSentence(258, false, false));
        Praxisdaten.addField(2, new SubSentence(259, true, false));
        Praxisdaten.addField(1, new SubSentence(260, false, false));
        Praxisdaten.addField(2, new SubSentence(261, true, false));
        Praxisdaten.addField(1, new SubSentence(262, false, false));
        Praxisdaten.addField(2, new SubSentence(263, true, false));
        Praxisdaten.addField(1, new SubSentence(264, false, false));
        Praxisdaten.addField(2, new SubSentence(265, true, false));
        Praxisdaten.addField(1, new SubSentence(266, false, false));
        Praxisdaten.addField(2, new SubSentence(267, true, false));
        Praxisdaten.addField(1, new SubSentence(268, false, false));
        Praxisdaten.addField(2, new SubSentence(269, true, false));
        Praxisdaten.addField(1, new SubSentence(270, false, false));
        Praxisdaten.addField(2, new SubSentence(271, true, false));
        Praxisdaten.addField(1, new SubSentence(272, false, false));
        Praxisdaten.addField(2, new SubSentence(273, true, false));
        Praxisdaten.addField(1, new SubSentence(274, false, false));
        Praxisdaten.addField(2, new SubSentence(275, true, false));
        Praxisdaten.addField(1, new SubSentence(276, false, false));
        Praxisdaten.addField(2, new SubSentence(277, true, false));
        Praxisdaten.addField(1, new SubSentence(278, false, false));
        Praxisdaten.addField(2, new SubSentence(279, true, false));
        Praxisdaten.addField(1, new SubSentence(280, false, false));
        Praxisdaten.addField(2, new SubSentence(281, true, false));
        Praxisdaten.addField(1, new SubSentence(282, false, false));
        Praxisdaten.addField(2, new SubSentence(283, true, false));
        Praxisdaten.addField(1, new SubSentence(284, false, false));
        Praxisdaten.addField(2, new SubSentence(285, true, false));
        Praxisdaten.addField(1, new SubSentence(286, false, false));
        Praxisdaten.addField(2, new SubSentence(287, true, false));
        Praxisdaten.addField(1, new SubSentence(288, false, false));
        Praxisdaten.addField(2, new SubSentence(289, true, false));
        Praxisdaten.addField(1, new SubSentence(290, false, false));
        Praxisdaten.addField(2, new SubSentence(291, true, false));
        Praxisdaten.addField(1, new SubSentence(292, false, false));
        Praxisdaten.addField(2, new SubSentence(293, true, false));
        Praxisdaten.addField(1, new SubSentence(294, false, false));
        Praxisdaten.addField(2, new SubSentence(295, true, false));
        Praxisdaten.addField(1, new SubSentence(296, false, false));
        Praxisdaten.addField(2, new SubSentence(297, true, false));
        Praxisdaten.addField(1, new SubSentence(298, false, false));
        Praxisdaten.addField(2, new SubSentence(299, true, false));

        /*
        Aerztliche Behandlung 0101
         */
        AerztlicheBehandlung.addField(1, new SubSentence(8000, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(8100, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(3000, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(3100, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3101, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(3102, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(3103, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(3104, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3105, false, false));//bedingtes muss
        AerztlicheBehandlung.addField(1, new SubSentence(3106, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3107, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3108, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(3110, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3201, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3202, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3203, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3204, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(3205, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(4101, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(4102, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(4103, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(4104, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(4105, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(4106, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(4107, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(4109, false, false));//bedingtes muss
        AerztlicheBehandlung.addField(1, new SubSentence(4110, false, false));//bedingtes muss
        AerztlicheBehandlung.addField(1, new SubSentence(4111, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(4112, false, false));//bedingtes muss
        AerztlicheBehandlung.addField(1, new SubSentence(4113, false, false));//bedingtes muss
        AerztlicheBehandlung.addField(1, new SubSentence(4121, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(4122, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(4201, true, false));
        AerztlicheBehandlung.addField(1, new SubSentence(4206, false, false));
        AerztlicheBehandlung.addField(1, new SubSentence(4239, false, true));
        AerztlicheBehandlung.addField(1, new SubSentence(5000, true, false));//bedingtes muss
        AerztlicheBehandlung.addField(2, new SubSentence(5001, true, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5002, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5003, true, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5004, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5005, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5006, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5007, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5008, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5009, true, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5011, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5012, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5013, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5015, true, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5017, false, false));
        AerztlicheBehandlung.addField(3, new SubSentence(5018, false, false));
        AerztlicheBehandlung.addField(2, new SubSentence(6000, true, false));
        AerztlicheBehandlung.addField(2, new SubSentence(6001, true, false));

        /*
        Patientensatmm 6100
         */
        Patientenstamm.addField(1, new SubSentence(8000, false, true));
        Patientenstamm.addField(1, new SubSentence(8100, false, true));
        Patientenstamm.addField(1, new SubSentence(3000, false, true));
        Patientenstamm.addField(1, new SubSentence(3100, false, false));
        Patientenstamm.addField(1, new SubSentence(3101, false, true));
        Patientenstamm.addField(1, new SubSentence(3102, false, true));
        Patientenstamm.addField(1, new SubSentence(3103, false, true));
        Patientenstamm.addField(1, new SubSentence(3104, false, false));
        Patientenstamm.addField(1, new SubSentence(3105, false, false));
        Patientenstamm.addField(1, new SubSentence(3106, false, false));
        Patientenstamm.addField(1, new SubSentence(3107, false, false));
        Patientenstamm.addField(1, new SubSentence(3108, false, false));
        Patientenstamm.addField(1, new SubSentence(3110, false, false));
        Patientenstamm.addField(1, new SubSentence(3201, false, false));
        Patientenstamm.addField(1, new SubSentence(3202, false, false));
        Patientenstamm.addField(1, new SubSentence(3203, false, false));
        Patientenstamm.addField(1, new SubSentence(3204, false, false));
        Patientenstamm.addField(1, new SubSentence(3205, false, false));
        Patientenstamm.addField(1, new SubSentence(3210, false, false));
        Patientenstamm.addField(1, new SubSentence(3601, false, false));
        Patientenstamm.addField(1, new SubSentence(3602, false, false));
        Patientenstamm.addField(1, new SubSentence(3603, false, false));
        Patientenstamm.addField(1, new SubSentence(3610, false, false));
        Patientenstamm.addField(1, new SubSentence(3612, false, false));
        Patientenstamm.addField(1, new SubSentence(3620, false, false));
        Patientenstamm.addField(1, new SubSentence(3625, false, false));
        Patientenstamm.addField(1, new SubSentence(3626, true, false));
        Patientenstamm.addField(1, new SubSentence(3627, false, false));
        Patientenstamm.addField(1, new SubSentence(3628, false, false));
        Patientenstamm.addField(1, new SubSentence(3630, false, false));
        Patientenstamm.addField(1, new SubSentence(3631, false, false));
        Patientenstamm.addField(1, new SubSentence(3635, true, false));
        Patientenstamm.addField(1, new SubSentence(3637, false, false));
        Patientenstamm.addField(1, new SubSentence(3649, true, false));
        Patientenstamm.addField(2, new SubSentence(3650, true, false));
        Patientenstamm.addField(1, new SubSentence(3651, true, false));
        Patientenstamm.addField(2, new SubSentence(3652, true, false));
        Patientenstamm.addField(1, new SubSentence(3654, true, false));
        Patientenstamm.addField(1, new SubSentence(3656, true, false));
        Patientenstamm.addField(1, new SubSentence(3658, true, false));
        Patientenstamm.addField(1, new SubSentence(3660, true, false));
        Patientenstamm.addField(1, new SubSentence(3662, true, false));
        Patientenstamm.addField(1, new SubSentence(3664, false, false));
        Patientenstamm.addField(1, new SubSentence(3666, false, false));
        Patientenstamm.addField(1, new SubSentence(3668, false, false));
        Patientenstamm.addField(1, new SubSentence(3670, true, false));
        Patientenstamm.addField(1, new SubSentence(3672, true, false));
        Patientenstamm.addField(1, new SubSentence(3700, false, false));
        Patientenstamm.addField(2, new SubSentence(3701, true, false));
        Patientenstamm.addField(1, new SubSentence(3702, false, false));
        Patientenstamm.addField(2, new SubSentence(3703, true, false));
        Patientenstamm.addField(1, new SubSentence(3704, false, false));
        Patientenstamm.addField(2, new SubSentence(3705, true, false));
        Patientenstamm.addField(1, new SubSentence(3706, false, false));
        Patientenstamm.addField(2, new SubSentence(3707, true, false));
        Patientenstamm.addField(1, new SubSentence(3708, false, false));
        Patientenstamm.addField(2, new SubSentence(3709, true, false));
        Patientenstamm.addField(1, new SubSentence(3710, false, false));
        Patientenstamm.addField(2, new SubSentence(3711, true, false));
        Patientenstamm.addField(1, new SubSentence(3712, false, false));
        Patientenstamm.addField(2, new SubSentence(3713, true, false));
        Patientenstamm.addField(1, new SubSentence(3714, false, false));
        Patientenstamm.addField(2, new SubSentence(3715, true, false));
        Patientenstamm.addField(1, new SubSentence(3716, false, false));
        Patientenstamm.addField(2, new SubSentence(3717, true, false));
        Patientenstamm.addField(1, new SubSentence(3718, false, false));
        Patientenstamm.addField(2, new SubSentence(3719, true, false));
        Patientenstamm.addField(1, new SubSentence(4104, false, false));
        Patientenstamm.addField(1, new SubSentence(4111, false, false));
        Patientenstamm.addField(1, new SubSentence(4112, false, false));
        Patientenstamm.addField(1, new SubSentence(4113, false, false));

        /*
        Behandlungsdaten 6200
         */
        Behandlungsdaten.addField(1, new SubSentence(8000, false, true));
        Behandlungsdaten.addField(1, new SubSentence(8100, false, true));
        Behandlungsdaten.addField(1, new SubSentence(3000, false, true));
        Behandlungsdaten.addField(1, new SubSentence(3622, false, false));
        Behandlungsdaten.addField(1, new SubSentence(3623, false, false));
        Behandlungsdaten.addField(1, new SubSentence(6200, true, true));
        Behandlungsdaten.addField(2, new SubSentence(6205, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6210, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6211, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6215, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6220, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6221, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6222, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6225, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6230, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6240, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6260, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6265, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6280, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6285, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6286, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6290, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6291, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6300, true, false));
        Behandlungsdaten.addField(3, new SubSentence(6301, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6306, true, false));
        Behandlungsdaten.addField(3, new SubSentence(6307, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6310, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6311, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6312, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6313, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6314, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6315, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6316, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6317, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6319, false, false));
        Behandlungsdaten.addField(2, new SubSentence(6320, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6325, true, false));
        Behandlungsdaten.addField(3, new SubSentence(6326, false, false));//bedingtes muss
        Behandlungsdaten.addField(3, new SubSentence(6327, false, false));//bedingtes muss
        Behandlungsdaten.addField(2, new SubSentence(6330, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6331, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6332, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6333, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6334, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6335, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6336, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6337, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6338, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6339, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6340, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6341, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6342, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6343, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6344, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6345, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6346, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6347, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6348, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6349, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6350, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6351, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6352, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6353, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6354, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6355, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6356, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6357, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6358, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6359, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6360, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6361, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6362, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6363, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6364, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6365, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6366, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6367, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6368, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6369, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6370, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6371, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6372, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6373, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6374, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6375, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6376, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6377, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6378, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6379, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6380, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6381, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6382, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6383, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6384, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6385, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6386, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6387, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6388, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6389, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6390, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6391, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6392, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6393, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6394, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6395, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6396, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6397, true, false));
        Behandlungsdaten.addField(2, new SubSentence(6398, false, false));
        Behandlungsdaten.addField(3, new SubSentence(6399, true, false));
        Behandlungsdaten.addField(2, new SubSentence(8401, false, false));
        Behandlungsdaten.addField(3, new SubSentence(8402, true, false));
        Behandlungsdaten.addField(3, new SubSentence(8410, true, false));
        Behandlungsdaten.addField(4, new SubSentence(8411, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8418, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8420, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8421, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8422, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8429, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8430, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8431, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8432, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8433, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8440, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8441, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8442, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8443, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8444, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8445, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8446, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8447, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8460, false, false));
        Behandlungsdaten.addField(4, new SubSentence(8470, true, false));
        Behandlungsdaten.addField(4, new SubSentence(8480, true, false));
        Behandlungsdaten.addField(4, new SubSentence(8490, true, false));
        Behandlungsdaten.addField(2, new SubSentence(8990, false, false));

    }
}
