package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.NoSuchElementException;

/**
 * A class used to iterate over the Sentence characteristic and computes the next possible Fields based on the last used Field stored as SubSentences in the sentence<br/>
 * CAUTION: this class does not implement the Iterator Interface
 * #hasNext must always called before #next()
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class SentenceIterator {
    private SentenceCharacteristic sentence;
    private boolean calculatedNext;
    private LinkedList<SubSentence> lastFields;
    private LinkedList<SubSentence> nextFields;

    /**
     * constructor for the SentenceIterator
     *
     * @param characteristic the SentenceCharacteristic over which one should be iterated
     */
    public SentenceIterator(SentenceCharacteristic characteristic) {
        sentence = characteristic;
        calculatedNext = false;
        lastFields = new LinkedList<>();
        nextFields = new LinkedList<>();
    }

    /**
     * Methode to check if there are next possible fields in the sentences, based on the given last subsentence
     * This method has to be called before next()
     *
     * @param lastUsed the last used subsentence must be part in
     * @return true if there is at least on next subsentence, false otherwise
     * @throws IllegalSentenceStructureException if {@link SubSentence#getNextFromSameLevel()} throws an IllegalSentenceStructureException
     */
    public boolean hasNext(SubSentence lastUsed) throws IllegalSentenceStructureException {
        try {
            if (!calculatedNext)
                calculatedNextFields(lastUsed);
            return !nextFields.isEmpty();
        } catch (IllegalArgumentException | NullPointerException iae) {
            calculatedNext = false;
            return false;
        }
    }

    /**
     * calculates the next possible SubSntences based on the last one used
     *
     * @param lastUsed the {@link SubSentence} that should be used to calculate the next Fields, must be contained in nextFields, must be not null, unless teh method is called for the first time
     * @throws IllegalSentenceStructureException if {@link SubSentence#getNextFromSameLevel()} throws an IllegalSentenceStructureException
     * @throws IllegalArgumentException if the last computed nextFields does not contains lastUsed or lastUsed is null and the method isn`t called for the first time
     */
    private void calculatedNextFields(SubSentence lastUsed) throws IllegalSentenceStructureException {
        if (lastUsed == null && lastFields.isEmpty() && !calculatedNext) {
            nextFields.add(sentence.getSubSenteces().get(0));
            calculatedNext = true;
            return;
        }
        if (lastUsed == null)
            throw new IllegalArgumentException("lastUsed must be not null, except for the first call");
        if (!nextFields.contains(lastUsed))
            throw new IllegalArgumentException("last used field isn`t element from the last possible fields");
        lastFields = nextFields;
        nextFields = new LinkedList<>();
        SubSentence level = lastUsed;
        addAllPossibleFieldsFromSameLevel(level, false);
        while (level.higherLevelSubSentence != null && !calculatedNext) {
            level = level.higherLevelSubSentence;
            addAllPossibleFieldsFromSameLevel(level, true);
        }
        calculatedNext = true;
    }

    /**
     * adds all the next possible sentences from the given sentence on the same and lower levels
     *
     * @param sentence the sentence to start
     * @param allFromSublevelAdded true if all SubSentences from a LowerLevel are already added to the list
     * @throws IllegalSentenceStructureException if {@link SubSentence#getNextFromSameLevel()} throws an IllegalSentenceStructureException
     */
    private void addAllPossibleFieldsFromSameLevel(SubSentence sentence, boolean allFromSublevelAdded) throws IllegalSentenceStructureException {
        if (calculatedNext)
            return;
        if ((sentence.isField() || allFromSublevelAdded)) {
            if (sentence.multiple) { // add itself when it's a multiple field
                nextFields.add(sentence);
            }

            SubSentence next = sentence.getNextFromSameLevel(); // the direct next field on the same level
            while (next != null) { // add all next possible fields from the same level, stop as soon as one field is required
                nextFields.add(next);
                if (next.required) {
                    calculatedNext = true;
                    return;
                }
                if (!next.isField()) { // if the next sentence has a subsentence check it
                    addAllPossibleFieldsFromSameLevel(next.getSubSentences().getFirst(), false);
                    if (calculatedNext) {
                        return;
                    }
                }
                next = next.getNextFromSameLevel();
            }
        } else { // add all from the subsentencen until one is required
            Iterator<SubSentence> iterator = sentence.getSubSentences().listIterator();
            SubSentence next;
            while (iterator.hasNext()) {
                next = iterator.next();
                nextFields.add(next);
                if (next.required) {
                    calculatedNext = true;
                    return;
                }
            }
            addAllPossibleFieldsFromSameLevel(sentence, true);// proceed on the same level
        }
    }

    /**
     * Returns the list of the next possible subsentece in the iteration.
     * next has to be called befor this method is
     *
     * @return the next element in the iteration
     * @throws java.util.NoSuchElementException if the iteration has no more elements
     * @throws IllegalStateException if the next is called without hasNext() before it
     */
    public ArrayList<SubSentence> next() {
        if (!calculatedNext)
            throw new IllegalStateException("has next has to been called before");
        if (nextFields.isEmpty())
            throw new NoSuchElementException("no possible next fields");
        ArrayList<SubSentence> clone = new ArrayList<>();
        clone.addAll(nextFields);
        calculatedNext = false;
        return clone;
    }

}
