package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.concurrent.atomic.AtomicLong;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldTable;

/**
 * A class used to dispalay a stored Field in the SentenceCharacteristic, which could contains a list of fields stored as SubSentences
 * most methods are package private and shouldn`t be called unless you know what you are doing
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public final class SubSentence implements Cloneable {

    /**
     * the id from the Subsentence if 2 subSentence are equal the id is same and if the id is the same 2 SubSentences have to be equal
     */
    public final long id;
    /**
     * used to generate a unique id for every subsentence
     * only if a subsentence is cloned the id remains the same
     */
    private static AtomicLong counter = new AtomicLong();
    private final FieldCharacteristic field;
    public final boolean multiple;
    public final boolean required;
    private final LinkedList<SubSentence> subSentences;
    SubSentence higherLevelSubSentence;
    private SubSentence nextSubsentenceFromSameLevel = null;
    private SentenceCharacteristic sentenceCharacteristic;

    /**
     * Creates a new SubSentence from the given fieldnumber, multiple and required
     * creates a new id
     *
     * @param fieldnumber the fieldnumber of the stored field
     * @param multiple if the field is multiple
     * @param required if the field is required
     */
    SubSentence(int fieldnumber, boolean multiple, boolean required) {
        this.field = FieldTable.getCharateristic(fieldnumber);
        this.multiple = multiple;
        this.required = required;
        this.subSentences = new LinkedList<>();
        id = counter.getAndIncrement();
    }

    /**
     * Copy constructor <br/>
     * doesn't create deep copys from the stored params
     * the id from both SubSentences is equal
     *
     * @param subSentence the subsentece that should be cloned
     */
    private SubSentence(SubSentence subSentence) {
        field = FieldCharacteristic.valueOf(subSentence.field.getName());
        multiple = subSentence.multiple;
        required = subSentence.required;
        subSentences = new LinkedList<>();
        sentenceCharacteristic = subSentence.sentenceCharacteristic;
        higherLevelSubSentence = subSentence.higherLevelSubSentence;
        id = subSentence.id;
    }

    /**
     * the list of stored SubSentences
     *
     * @return the list containing the Subsentences from this Subsentence
     */
    LinkedList<SubSentence> getSubSentences() {
        return subSentences;
    }

    void setSentenceCharacteristic(SentenceCharacteristic sentenceCharacteristic) {
        this.sentenceCharacteristic = sentenceCharacteristic;
    }

    /**
     * Delivers the next Subsentence that is stored on the same level
     *
     * @return the next Seubsentence on the same level
     * @throws IllegalSentenceStructureException if this SubSentence isn`t stored correctly in the SubSentence on a higher level
     */
    SubSentence getNextFromSameLevel() throws IllegalSentenceStructureException {
        if (nextSubsentenceFromSameLevel == null) {
            Iterator<SubSentence> iterator;
            if (higherLevelSubSentence == null) {
                iterator = sentenceCharacteristic.getIterator();
            } else {
                iterator = higherLevelSubSentence.subSentences.iterator();
            }
            while (iterator.hasNext()) {
                SubSentence subSentence = iterator.next();
                if (subSentence.equals(this)) {
                    if (iterator.hasNext()) {
                        nextSubsentenceFromSameLevel = iterator.next();
                        return nextSubsentenceFromSameLevel;
                    }
                    return null;
                }
            }
            throw new IllegalSentenceStructureException("something went terrible wrong; cound`t find the father from the child subsentence");
        }
        return nextSubsentenceFromSameLevel;
    }

    /**
     * Returns the stored Field
     *
     * @return the Characteristtic of the stored Field
     */
    public FieldCharacteristic getFieldCharactersitic() {
        return FieldCharacteristic.valueOf(field.name());
    }

    /**
     * returns if the SubSentence is a Field or contains a lower Level of Subsentences
     *
     * @return true if it is a Field, false otherwise
     */
    public boolean isField() { // umbenennen
        return subSentences.isEmpty();
    }

    /**
     * Creates an deep copy of this Subsentence with the same id as the this
     *
     * @return a deep copy
     */
    @Override
    public Object clone() {
        SubSentence cloneSubSentence = new SubSentence(this);
        for (SubSentence ss : subSentences) {
            cloneSubSentence.subSentences.add((SubSentence) ss.clone());
        }
        return cloneSubSentence;
    }

    /**
     * checks if two subsentence are equals through comparing the ids
     *
     * @param o the subsentence that should be compared
     * @return true if both subsentence are equals
     */
    public boolean equals(Object o) {
        return o == this || o != null && o instanceof SubSentence && id == ((SubSentence) o).id;
    }

    /**
     * generates a hashcode for the subsentence.
     * @return the hash code
     */
    @Override
    public int hashCode() {
        int result = (int) (id ^ (id >>> 32));
        result = 31 * result + field.hashCode();
        result = 31 * result + (multiple ? 1 : 0);
        result = 31 * result + (required ? 1 : 0);
        result = 31 * result + subSentences.hashCode();
        result = 31 * result + higherLevelSubSentence.hashCode();
        result = 31 * result + nextSubsentenceFromSameLevel.hashCode();
        result = 31 * result + sentenceCharacteristic.hashCode();
        return result;
    }

    /**
     * ONLY {@link SentenceCharacteristic} SHOULD USE THIS METHOD in the static part to creates the SubSentences!
     * <br/>
     * adds the given SubSentence to the list of SubSentences
     *
     * @param subSentence the SubSentence to add
     */
    void addCharacteristic(SubSentence subSentence) {
        subSentences.add(subSentence);
    }

    /**
     * ONLY {@link SentenceCharacteristic} SHOULD USE THIS METHOD in the static part to creates the SubSentences!
     * <br/>
     * returns the last SubSentence from the list of SubSentences
     *
     * @return
     */
    SubSentence getLastSubSentence() {
        return this.subSentences.get(subSentences.size() - 1);
    }

    /**
     * ONLY {@link SentenceCharacteristic} SHOULD USE THIS METHOD in the static part to creates the SubSentences!
     * <br/>
     * set the Higher Level SubSentence (the SubSentece from a Level that contains this subsentence!
     *
     * @param higherLevelSubSentence the SubSentence that contains this SubSentence
     */
    void setHigherLevelSubSentence(SubSentence higherLevelSubSentence) {
        this.higherLevelSubSentence = higherLevelSubSentence;
    }
}
