package de.uni_goettingen.med.mi.bdtparser.core.io.reader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
import java.util.concurrent.LinkedTransferQueue;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;

/**
 * A reader to read in *.bdt or *.BDT files used in the german health care system.
 * It read in all Bdt-files line by line in a given directory or file list.
 * Afterwards the lines a stored in a LinkedTransferQueue to be processed in another thread.
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BdtFileReader implements Runnable {

    private static final Logger log = LogManager.getLogger(BdtFileReader.class);
    private LinkedTransferQueue<String> queue;
    private File[] fileList;
    private long numberOfLines;
    public DoubleProperty processProperty;

    /**
     * Constructor for the file reader
     * @param queue the queue to store the lines
     * @param pathName the path to the directory where the files are stored
     */
    public BdtFileReader(LinkedTransferQueue<String> queue, String pathName) {
        numberOfLines = 0;
        processProperty = new SimpleDoubleProperty();
        validatePath(pathName);
        fileList = new File(pathName).listFiles(new FileFilter() {
            public boolean accept(File pathname) {
                return pathname.toString().toLowerCase().endsWith(".bdt");
            }
        });
        if (fileList == null || fileList.length == 0) // An Error occured or there are no BDT files
            throw new IllegalArgumentException("No BDT-files in this directory");
        Arrays.sort(fileList);
        setQueue(queue);
    }

    /**
     * The run Method where the main job is done. It will open a reader for all files in the List using the CP437 encoding.
     * With this encoding the reader is able to read the 7 bit ASCII and the 8 bit
     * encoded files. The files are read in line by line and stored in the queue to be processed in another thread.
     */
    public void run() {
        BufferedReader reader = null; // create the reader with Cp437 encoding to read ASCII and CP437 encoded files
        String encoding = "Cp437";
        int numberOfFiles = fileList.length;
        int filecounter = 0;
        for (File f : fileList) {
            filecounter++;
            processProperty.set((double) filecounter / (double) numberOfFiles);
            try {
                reader = new BufferedReader(new InputStreamReader(new FileInputStream(f), encoding));
                // read the complete File line by line and queue the strings
                String in = reader.readLine();
                while (in != null) {
                    numberOfLines++;
                    queue.put(in);
                    in = reader.readLine();
                }
                reader.close();
            } catch (FileNotFoundException fnfe) {
                log.error("File Not Found! ", fnfe);
            } catch (UnsupportedEncodingException uee) {
                log.error("Ecoding " + encoding + " not supported");
            } catch (IOException ioe) {
                log.error("IOException");
            } finally {
                try {
                    reader.close();
                } catch (IOException ioe) {
                    log.error("could not close the reader ");
                } catch (NullPointerException npe) {
                    log.error("couldn't close the the reader, null reference");
                }
            }
        }
    }

    public LinkedTransferQueue<String> getQueue() {
        return queue;
    }

    /**
     * This function returns the number of lines that already read in and stored in the queue.
     * This could be used for a progressbar to get the number of lines read in after the thread has finished.
     *
     * @return the number of lines allready stored in the queue
     */
    public synchronized long getNumberOfLines() {
        return numberOfLines;
    }

    /**
     * Validates the give path by checking if it exists and it's readable and a directory
     *
     * @param path the path to validate as a string
     */
    private void validatePath(String path) {
        File dir = new File(path);
        if (!(dir.exists() && dir.canRead() && dir.isDirectory()))
            throw new IllegalArgumentException("Path is not valid " + path);
    }

    /**
     * Setter for the queue. Checks if its not null, if not an IllegalArgumentException is thrown.
     *
     * @param queue the queue to used store the Strings
     */
    private void setQueue(LinkedTransferQueue<String> queue) {
        if (queue != null)
            this.queue = queue;
        else
            throw new IllegalArgumentException("queue must not be null and an instance from LinkedTransferQueue");
    }

}