package de.uni_goettingen.med.mi.bdtparser.core.io.writer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.LinkedList;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datacarrier.Datacarrier;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datapackage.Datapackage;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.Sentence;

/**
 * The Unmarshaller wich could be used to recreate the bdt files based on the datacarriers
 * To work properly the data has to be stored using the classes from the bdt_structur package
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class Unmarshaller implements Runnable {
    private LinkedList<Datacarrier> datacarriers;
    private BufferedWriter writer;
    private String path;
    private long counter;
    private final static String endl = "\r\n";
    private final static String Datetraegerheader = "0020";
    public final Logger log = LogManager.getLogger(Unmarshaller.class);

    /**
     * Constructor for the Unmarshaller, which needs a datacarrier to write it out
     *
     * @param datacarriers the datacarrier that should be unmarshalled and written to files
     */
    public Unmarshaller(LinkedList<Datacarrier> datacarriers) {
        counter = 1; // for naming the files
        this.datacarriers = datacarriers;
    }

    public Unmarshaller(LinkedList<Datacarrier> datacarriers, String path) {
        this(datacarriers);
        this.path = path;
    }

    public Unmarshaller(Datapackage datapackage, String path) {
        this(datapackage.getDatacarriers(), path);
    }

    /**
     * write the data back to new bdt files, while maintaining the original structure of the files
     * prompt the user to enter a path to store the files, creates the directory if it has to
     */
    @Override
    public void run() {
        if (path == null)
            path = getDirectory();
        try {
            if (new File(path).mkdirs())
                log.debug("created directories");
            else
                log.debug("directory already existed");
            // create a new file for each old one
            for (Datacarrier dp : datacarriers) {
                ArrayList<Sentence> sentences = dp.getSentences();
                for (Sentence s : sentences) {
                    if (s.getFields().getFirst().getData().equals(Datetraegerheader) ){
                        createNewFile();
                        log.debug("created new BDT-File");
                    }
                    for (Field field : s.getFields()) {
                        writer.write(buildField(field));
                    }
                }
            }
            writer.flush();
            writer.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
            try {
                writer.close();
            } catch (IOException ioe1) {
                System.out.println("Test");
            }
        }
    }

    /**
     * Flushes and closes the current writer and create a new file and writer
     *
     * @throws IOException if something goes wrong
     */
    private void createNewFile() throws IOException {
        if (writer != null) {
            writer.flush();
            writer.close();
        }
        File f = new File(buildPath());
        if (!f.exists()) {
            if (f.createNewFile())
                log.info("created new File");
        }
        FileWriter fw = new FileWriter(f);
        writer = new BufferedWriter(fw);

    }

    /**
     * form a field to a line, which could be written to a bdt file
     *
     * @param field the field that should be written to the file
     * @return the line as String
     */
    private String buildField(Field field) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(field.getLength());
        stringBuilder.append(field.getNumber());
        stringBuilder.append(field.getData());
        stringBuilder.append(endl);
        return stringBuilder.toString();
    }

    /**
     * creates the path for each new bdt-file, each time the function is called it's incrementing the file count by one
     *
     * @return the path of the file
     */
    private String buildPath() {
        StringBuilder stringBuilder;
        stringBuilder = new StringBuilder(path);
        stringBuilder.append('/');
        if (System.getProperty("os.name").contains("linux"))
            stringBuilder.append("/");
        stringBuilder.append("X0");
        stringBuilder.append(counter++);
        stringBuilder.append(".bdt");
        return stringBuilder.toString();
    }

    /**
     * Asks the the user to enter the path to the directory where the new bdt files should be stored
     *
     * @return the Path to the directory as String
     */
    public String getDirectory() {
        BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("Enter path to write new files:\tDefaultpath: Output\\Testdata");
        try {
            return reader.readLine();
        } catch (IOException e) {
            System.out.println("Something went wrong: " + e.getMessage());
            try {
                reader.close();
            } catch (IOException e1) {
                System.out.println("Cant Close the Reader." + e.getMessage());
            }
        }
        return null;
    }
}
