package de.uni_goettingen.med.mi.bdtparser.core.validator;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.time.DateTimeException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datacarrier.Datacarrier;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Datatyp;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldTable;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.IllegalSentenceStructureException;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.Sentence;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.SentenceCharacteristic;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.SentenceIterator;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.SentenceTable;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.SubSentence;
import javafx.scene.control.Label;

/**
 * Class used to Validate the bdt files.
 * <p>
 * See validator.properties which bdt-sentences were validated
 * Exports a method to validate a Data package
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class Validator {
    // =======final public static Variable=======
    public final static String NULLIFIED_STRING = "NULLIFIED DATA!";
    public final static int BASIC_FIELD_LENGTH = 9; // basic field length 3 for the length, 4 for field number, 2 for crlf
    private final static Logger logger = LogManager.getLogger(Validator.class);
    // =====errorcodes=====
    public final static int OK = 0;
    public final static int DATACARRIER_STRUCTURE_VALIDATION_ERROR = 1;
    public final static int SENTENCES_STRUCTURE_VALIDATION_ERROR = 2;
    public final static int DATACARRIER_AND_SENTENCE_STRUCTURE_VALIDATION_ERROR = 3;
    public final static int FIELD_VALIDATION_ERROR = 4;
    public final static int DATACARRIER_STRCUTURE_AND_FIELD_VALIDATION_ERROR = 5;
    public final static int SENTENCE_STRUCTURE_AND_FIELD_VALIDATION_ERROR = 6;
    public final static int DATACARRIER_AND_SENTENCE_STRUCTURE_AND_FIELD_VALIDATION_ERROR = 7;

    private boolean criticalDiscrepancyFound;
    private Datacarrier datacarrier;
    private Label progressBarLabel;
    private String validationPart;
    private Properties properties;
    private boolean invalidDateFormat = false;
    List<String> lstCities;
    List<String> lstKnownIdentifierPatterns;

    public Validator() {
        init();
    }

    /**
     * loads the config file with the sentence which should be validated
     */
    private void init() {
        
        properties = new Properties();
        String fileNameProperties = "/validator/validator.properties";
        InputStream inputStreamValidatorProperties = Validator.class.getResourceAsStream(fileNameProperties);
        try {
            properties.load(inputStreamValidatorProperties);
        } catch (IOException ioe) {
            logger.error("Error while loading config file from " + fileNameProperties + " exception message: " + ioe.getMessage());
        }
       
        String fileNameCities = "/validator/cities.txt";
        InputStream inputStreamFileNameCities = Validator.class.getResourceAsStream(fileNameCities);
        BufferedReader bufferedReaderFileNameCities = new BufferedReader(new InputStreamReader(inputStreamFileNameCities));
        lstCities = new ArrayList<String>();
        try {
            String line;
            while ((line = bufferedReaderFileNameCities.readLine()) != null) {
                lstCities.add(line);
            }
        } catch (IOException ioe) {
            logger.error("Error while loading file from " + fileNameCities + " exception message: " + ioe.getMessage());
        }
        
        String fileNameKnownIdentifierPatterns = "/validator/knownIdentifierPatterns.txt";
        InputStream inputStreamKnownIdentifierPatterns = Validator.class.getResourceAsStream(fileNameKnownIdentifierPatterns);
        BufferedReader bufferedReaderKnownIdentifierPatterns = new BufferedReader(new InputStreamReader(inputStreamKnownIdentifierPatterns));
        lstKnownIdentifierPatterns = new ArrayList<String>();
        try {
            String line;
            while ((line = bufferedReaderKnownIdentifierPatterns.readLine()) != null) {
                lstKnownIdentifierPatterns.add(line);
            }
        } catch (IOException ioe) {
            logger.error("Error while loading file from " + fileNameKnownIdentifierPatterns + " exception message: " + ioe.getMessage());
        }
        
    }

    /**
     * constructor for the validator
     *
     * @param progressBarLabel the label that should be updated while validating the datacarrier
     */
    public Validator(Label progressBarLabel) {
        this();
        criticalDiscrepancyFound = false;
        if (progressBarLabel == null)
            throw new IllegalArgumentException("label must not be null");
        this.progressBarLabel = progressBarLabel;
    }

    /**
     * Validates the given data carrier. Checks the structure of data carrier, structure of the sentences
     * and the if the content of the fields match the rules stored in the field characteristic
     * If a discrepancy occurred a log entry is created.
     * If the discrepancy is critical a ValidationException is thrown, otherwise the data is either nullified or corrected!
     *
     * @param datacarrier the datacarrier to validate, must not be null
     * @param firstDatacarrier true if the data carrier is the first data carrier in the data package
     * @param lastDatacarrier true if the data carrier is the last data carrier in the data package
     * @throws ValidationException the exception thrown if a (critical) discrepancy occurred
     * @throws IllegalArgumentException if the given datacarrier is null
     */
    public void validate(Datacarrier datacarrier, boolean firstDatacarrier, boolean lastDatacarrier) throws ValidationException {
        int errorCode = 0;
        if (datacarrier == null)
            throw new IllegalArgumentException("datacarrier must not be null");
        this.datacarrier = datacarrier;
        logger.debug("starting validation");
        if (!validateDataCarrierStructure(firstDatacarrier, lastDatacarrier))
            errorCode += DATACARRIER_STRUCTURE_VALIDATION_ERROR;
        logger.debug("validated Datacarrier Structure");
        if (!validateSentenceStructur())
            errorCode += SENTENCES_STRUCTURE_VALIDATION_ERROR;
        logger.debug("validated SentenceStructure");
        // progressBarLabel.setText("validating field content");
        if (!validateFields())
            errorCode += FIELD_VALIDATION_ERROR;
        logger.debug("validated Fields");
        // if a discrepancy is found create the error message and throw a Exception
        String errorMessage = "";
        switch (errorCode) {
        case OK:
            break;
        case DATACARRIER_STRUCTURE_VALIDATION_ERROR:
            errorMessage = "Found at least one discrepancy in the datacarrier structure.";
            break;
        case SENTENCES_STRUCTURE_VALIDATION_ERROR:
            errorMessage = "Found at least one discrepancy in the sentence structure.";
            break;
        case DATACARRIER_AND_SENTENCE_STRUCTURE_VALIDATION_ERROR:
            errorMessage = "Found at least one discrepancy in the sentence structure and in the datacarrier structure.";
            break;
        case FIELD_VALIDATION_ERROR:
            errorMessage = "Found at least one discrepancy while the validation of the field content.";
            break;
        case DATACARRIER_STRCUTURE_AND_FIELD_VALIDATION_ERROR:
            errorMessage = "Found at least one discrepancy in the datacarrier structure and in the field content.";
            break;
        case SENTENCE_STRUCTURE_AND_FIELD_VALIDATION_ERROR:
            errorMessage = "Found at least one discrepancy in the sentence structure and in the field content.";
            break;
        case DATACARRIER_AND_SENTENCE_STRUCTURE_AND_FIELD_VALIDATION_ERROR:
            errorMessage = "Found at least one discrepancy in the sentence structure, in the datacarrier structure and in the field content.";
            break;
        default:
            throw new RuntimeException("IllegalErrorCode");
        }
        if (errorCode > 0)
            throw new ValidationException(errorMessage + " Check the log for further information!", criticalDiscrepancyFound, errorCode);
    }

    /**
     * Validates the data carrier structure, if a discrepancy occurred a log entry is created
     *
     * @param first first carrier in datapackage
     * @param last last carrier in datapackage
     * @return true if the sentence structure is valid, false otherwise
     */
    private boolean validateDataCarrierStructure(boolean first, boolean last) {
        boolean valid = true;
        validationPart = "Datacarrier structure";
        ArrayList<Sentence> sentences = datacarrier.getSentences();
        // check if the second sentence holds the data carrier header
        String sentenceIdFirstSetence = sentences.get(1).getFields().getFirst().getData();
        if (first && !sentenceIdFirstSetence.equals(SentenceCharacteristic.Datenpaket_Header.id)) {
            String fieldNumber = sentences.get(1).getFields().getFirst().getNumber();
            createLogEntry(fieldNumber, true, "datapackage header missing");
            valid = false;
            criticalDiscrepancyFound = true;
        }
        // check if the next to last sentence holds data carrier end sentence
        if (last && !sentences.get(sentences.size() - 2).getFields().getFirst().getData().equals(SentenceCharacteristic.Datenpaket_Abschluss.id)) {
            String fieldNumber = sentences.get(sentences.size() - 2).getFields().getFirst().getNumber();
            createLogEntry(fieldNumber, true, "datacarrier end sentence missing");
            valid = false;
            criticalDiscrepancyFound = true;
        }
        // Be tolerant to sentence Betriebsstaettendaten("besa") which actually does not belong in BDT
        if (first && sentences.get(2).getFields().getFirst().getData().equals("besa")) {
            String fieldNumber = sentences.get(2).getFields().getFirst().getNumber();
            sentences.get(2).getFields().getFirst().setData(SentenceCharacteristic.Praxisdaten.id);
            createLogEntry(fieldNumber, false, "found besa sentence instead of praxis data. Changed it to 0010.");
        }
        if (first && !sentences.get(2).getFields().getFirst().getData().equals(SentenceCharacteristic.Praxisdaten.id)) {
            String fieldNumber = sentences.get(2).getFields().getFirst().getNumber();
            createLogEntry(fieldNumber, true, "praxis data sentence missing");
            valid = false;
            criticalDiscrepancyFound = true;
        }
        // check if the datacarrier header or end sentence is at a wrong position
        for (int i = 0; i < sentences.size(); i++) {
            for (Field field : sentences.get(i).getFields()) {
                if (field.getNumber().equals(FieldCharacteristic.Satzidentifikation.getNumber())) {
                    if (field.getData().equals(SentenceCharacteristic.Datenpaket_Header.id) && (!first || i != 1)) {
                        createLogEntry(field.getNumber(), true, " found a data packet header at a wrong position");
                        valid = false;
                        criticalDiscrepancyFound = true;
                    }
                    if (field.getData().equals(SentenceCharacteristic.Datenpaket_Abschluss.id) && (!last || i != sentences.size() - 2)) {
                        createLogEntry(field.getNumber(), true, " found a data packet end sentence at a wrong position");
                        valid = false;
                        criticalDiscrepancyFound = true;
                    }
                    if (field.getData().equals(SentenceCharacteristic.Datentraeger_Header.id) && i != 0) {
                        createLogEntry(field.getNumber(), true, " found a datacarrier header at a wrong position");
                        valid = false;
                        criticalDiscrepancyFound = true;
                    }
                    if (field.getData().equals(SentenceCharacteristic.Datentraeger_Abschluss.id) && i != sentences.size() - 1) {
                        createLogEntry(field.getNumber(), true, " found a datacarrier end sentence at a wrong position");
                        valid = false;
                        criticalDiscrepancyFound = true;
                    }
                    if (field.getData().equals(SentenceCharacteristic.Praxisdaten.id) && (!first || i != 2)) {
                        createLogEntry(field.getNumber(), true, " found a praxis data sentence at a wrong position");
                        valid = false;
                        criticalDiscrepancyFound = true;
                    }
                }
            }
        }
        return valid;
    }


    /**
     * Validates the structure of all sentences in the datacarrier. Returns false and creates a log entry if a discrepancy is found.
     *
     * @return true if the sentence structure is valid, false otherwise
     */
    private boolean validateSentenceStructur() {
        validationPart = "Sentence structure";
        boolean valid = true;
        String id;
        for (Sentence sentence : datacarrier.getSentences()) {
            SubSentence lastFoundField = null;
            try {
                id = sentence.getSentenceType();
                int id_int = Integer.parseInt(id);
                SentenceCharacteristic sentenceDescription = SentenceTable.getCharateristic(id_int);
                if (sentenceShouldBeValidated(sentenceDescription)) {
                    ArrayList<SubSentence> lstPossibleNextForLastFoundField = null;
                    SentenceIterator iterator = sentenceDescription.getSentenceInterator();
                    boolean lastValid = true;
                    List<Field> invalidFields = new ArrayList<>();
                    for (Field field : sentence.getFields()) {
                        String fieldNumber = field.getNumber();
                        boolean fieldValid = false;
                        if (iterator.hasNext(lastFoundField)) {
                            lstPossibleNextForLastFoundField = iterator.next();
                            for (SubSentence possibleNextSubSentence : lstPossibleNextForLastFoundField) {
                                String possibleNextSubSentenceNumber = possibleNextSubSentence.getFieldCharactersitic().getNumber();
                                if (possibleNextSubSentenceNumber.equals(fieldNumber)) {
                                    fieldValid = true;
                                    lastFoundField = possibleNextSubSentence;
                                    break;
                                }
                            }
                        }
                        if (fieldValid) {
                            if (!lastValid) {
                                createInvalidsentenceLog(invalidFields, lstPossibleNextForLastFoundField, id);
                                valid = false;
                                lastValid = true;
                            }
                        } else {
                            lastValid = false;
                            invalidFields.add(field);
                        }
                    }
                    if (!invalidFields.isEmpty()) {
                        createInvalidsentenceLog(invalidFields, lstPossibleNextForLastFoundField, id);
                    }
                } else {
                    for (Field field : sentence.getFields()) {
                        field.setValid(false);
                    }
                }
            } catch (IllegalSentenceStructureException isse) {// found a sentence class which contains no fields. if that happens a critical error occurred while unmarshalling the files
                createNoSenteceTypFoundLogEntry(isse);
                valid = false;
                criticalDiscrepancyFound = true;
            }
        }
        return valid;
    }

    /**
     * Method to create an log entry if an invalid sentence was found
     * @param invalidFields the list of invalid fields found
     * @param lstPossibleNextForLastFoundField the list of possible next fields that were expected
     * @param id the senctence id
     */
    private void createInvalidsentenceLog(List<Field> invalidFields, ArrayList<SubSentence> lstPossibleNextForLastFoundField, String id) {
        StringBuilder errorMessage = new StringBuilder();
        errorMessage.append(id).append(": expected one from the following fields: ");
        if (lstPossibleNextForLastFoundField != null) {
            String prefix = "";
            for (SubSentence subSentence : lstPossibleNextForLastFoundField) {
                errorMessage.append(prefix);
                prefix = ", ";
                errorMessage.append(subSentence.getFieldCharactersitic().getNumber());
            }
        }
        StringBuilder invalidFieldsString = new StringBuilder();
        for (Field iField : invalidFields) {
            invalidFieldsString.append(iField.getNumber()).append(", ");
        }
        invalidFields.clear();
        createLogEntry(invalidFieldsString.toString().substring(0, invalidFieldsString.length() - 2), true, errorMessage.toString());
        criticalDiscrepancyFound = true;

    }

    /**
     * checks if the given sentence characteristic is part of the list of sentences that should be validated
     *
     * @param sentenceDescription the sentence characteristic
     * @return true if it should be validated
     */
    private boolean sentenceShouldBeValidated(SentenceCharacteristic sentenceDescription) {
        String propertie = "";
        if (properties.containsKey(sentenceDescription.name())) {
            propertie = properties.getProperty(sentenceDescription.name());
        }
        return propertie.equals("true");
    }

    /**
     * checks if at least one subSentence in the given list is required
     *
     * @param next the list containing the subSentences
     * @return true if at least one is required, false otherwise
     */
    private boolean required(ArrayList<SubSentence> next) {
        for (SubSentence subSentence : next) {
            if (subSentence.required)
                return true;
        }
        return false;
    }

    /**
     * validates the fields against their characteristics and their rule according to the specification
     *
     * @return true, if all fields are valid, false otherwise
     */
    private boolean validateFields() {
        validationPart = "Field content";
        boolean valid = true;
        for (Sentence sentence : datacarrier.getSentences()) {
            for (Field field : sentence.getFields()) {
                boolean validField = true;
                FieldCharacteristic fieldCharacteristic = FieldTable.getCharateristic(Integer.parseInt(field.getNumber()));
                if (fieldCharacteristic == null) {// check if field is known
                    createLogEntry(field.getNumber(), false, "Found a field with unknown number! Field Data nullified.");
                    nullifieFieldData(field);
                    valid = false;
                } else {
                    try {// check data typ
                        if (!fieldDatatypMatch(field, fieldCharacteristic.getType())) {
                            valid = false;
                            validField = false;
                            createLogEntry(field.getNumber(), false, "data typ doesn`t match. Should be " + fieldCharacteristic.getType().name() + "! Datapart nullified.");
                            nullifieFieldData(field);
                            continue;
                        }
                    } catch (IllegalDatatypException | NumberFormatException ide) {
                        valid = false;
                    }
                    // check length of datapart
                    if ((field.getData().length() < fieldCharacteristic.getMinLength()
                            || field.getData().length() > fieldCharacteristic.getMaxLength()) && validField) {
                        StringBuilder errorMessage = new StringBuilder();
                        errorMessage.append("Field length doesn`t fit, was ").append(field.getData().length())
                                .append(" should be between ").append(fieldCharacteristic.getMinLength());
                        errorMessage.append(" and ").append(fieldCharacteristic.getMaxLength());
                        createLogEntry(field.getNumber(), false, errorMessage.toString());
                        valid = false;
                        continue;
                    }
                    if (fieldCharacteristic.getRules() != null) {// check all rules from the field
                        boolean ruleViolated = false;
                        for (int rule : fieldCharacteristic.getRules()) {
                            try {
                                if (rule > 200 && rule < 300)
                                    validationPart = "sentence structure";
                                else
                                    validationPart = "field content";
                                if (validField && !checkRule(rule, field)) {
                                    createLogEntry(field.getNumber(), false, "Rule " + rule + " violated. Field content nullified.");
                                    ruleViolated = true;
                                }
                            } catch (IllegalSentenceStructureException isse) {
                                createNoSenteceTypFoundLogEntry(isse);
                            } catch (Exception e) {
                                createLogEntry(field.getNumber(), false, "An Exception was thrown while validating the rule " + rule + ". Exception type: " + e.getClass());
                                ruleViolated = true;
                            } finally {
                                validationPart = "field content";
                            }
                        }

                        if (ruleViolated) {
                            nullifieFieldData(field);
                            valid = false;
                            continue;
                        }
                    }
                    int fieldLength = Integer.parseInt(field.getLength()) - BASIC_FIELD_LENGTH;
                    int realLength = field.getData().length();
                    if (fieldLength != realLength) { // check the length of the field match the stored length
                        valid = false;
                        realLength += BASIC_FIELD_LENGTH;
                        createLogEntry(field.getNumber(), false, "BDT line length '" + field.getLength() + "' does not match the actual length of " + realLength);
                    }
                }
            }
        }
        return valid;
    }

    /**
     * checks if the content of a field met the given rule
     *
     * @param rule the rule that should be checked
     * @param field the field which should be validated
     * @return true if the rule is met
     */
    private boolean checkRule(int rule, Field field) throws IllegalSentenceStructureException {
        int year, month, day, quartal, number;
        String toUpper;
        LocalDate date;
        LocalDate endDate, startDate;
        Field otherField;
        switch (rule) {
        // ======= Format rules =======/
        case 5:// check if valid time format
            if (!validTime(field.getData()))
                return false;
            break;
        case 6:// check if valid from to time format
            if (!(validTime(field.getData().substring(0, 4)) && validTime(field.getData().substring(4))))
                return false;
            break;
        case 7: // check if YYMMnnn with YY=year, mm = month, nnn=ifd.nr nnn is numeral, already checked through field type
            year = Integer.parseInt(field.getData().substring(0, 2));
            month = Integer.parseInt(field.getData().substring(2, 4));
            if (year < 0 || year > 99 || month < 1 || month > 12)
                return false;
            break;
        case 8:// check if valid date ddmmyyyy
            year = Integer.parseInt(field.getData().substring(4));
            month = Integer.parseInt(field.getData().substring(2, 4));
            day = Integer.parseInt(field.getData().substring(0, 2));
            if (!validDate(year, month, day))
                return false;
            break;
        case 14:
            month = Integer.parseInt(field.getData().substring(0, 2));
            year = Integer.parseInt(field.getData().substring(2));
            if (year < 0 || year > 99 || month < 0 || month > 12)
                return false;
            break;
        case 15:// check if the fields only contains 0-9,a-z,A-Z,#,$<,> and * (a-z not included in the specification
            if (!field.getData().matches("[0-9a-zA-Z#<>\\\\*$]{" + field.getData().length() + "}"))
                return false;
            break;
        case 16:// quartal in year
            quartal = field.getData().charAt(0) - 48; // 48 ascii value for 0
            // year=Integer.parseInt(field.getData().substring(1));
            if (quartal > 4 || quartal < 0)
                return false;
            break;
        case 17:// VKNR
            number = Integer.parseInt(field.getData());
            if (number < 0 || number > 99999)
                return false;
            break;
        case 18:// Krankenkassennummer
            number = Integer.parseInt(field.getData());
            if (number < 0 || number > 9999999)
                return false;
            break;
        case 19:// from date to date
            year = Integer.parseInt(field.getData().substring(4, 8));
            month = Integer.parseInt(field.getData().substring(2, 4));
            day = Integer.parseInt(field.getData().substring(0, 2));
            if (!validDate(year, month, day))
                return false;
            year = Integer.parseInt(field.getData().substring(12));
            month = Integer.parseInt(field.getData().substring(10, 12));
            day = Integer.parseInt(field.getData().substring(8, 10));
            if (!validDate(year, month, day))
                return false;
            break;
        case 20:// date
            year = Integer.parseInt(field.getData().substring(4));
            month = Integer.parseInt(field.getData().substring(2, 4));
            day = Integer.parseInt(field.getData().substring(0, 2));
            if (day < 0 || day > 31 || month < 0 || month > 12 || year < 0 || year > 9999)
                return false;
            break;
        case 101: // field exists/checked
            if (!field.getData().equals("1"))
                return false;
            break;
        case 102:
            number = Integer.parseInt(field.getData());
            if (!(number == 1 || number == 2 || number == 3 || number == 4 || number == 5))
                return false;
            break;
        case 106:
            number = Integer.parseInt(field.getData());
            if (!(number == 1 || number == 2 || number == 3))
                return false;
            break;
        case 111:// Weggeldzonen
            toUpper = field.getData().toUpperCase();
            if (!(toUpper.equals("Z1") || toUpper.equals("Z2") || toUpper.equals("Z3") || toUpper.equals("Z4")))
                return false;
            break;
        case 112:
            number = Integer.parseInt(field.getData());
            if (!(number == 2 || number == 1))
                return false;
            break;
        case 116: // Versichertenart MFR
            number = Integer.parseInt(field.getData());
            if (!(number == 1 || number == 3 || number == 5))
                return false;
            break;

        case 117:// Ost/West Status
            number = Integer.parseInt(field.getData());
            if (!(number == 1 || number == 9))
                return false;
            break;
        case 118:// Version ADT-Satzbeschreibung should be a check if field data equals "10/93", not useful, because it is outdated
            break;//
        case 119:// change to getdata.equals("00").....equals("06")
            number = Integer.parseInt(field.getData());
            if (number < 0 || number > 6)
                return false;
            break;
        case 120:
            number = Integer.parseInt(field.getData());
            if (!(number == 2 || number == 3))
                return false;
            break;
        case 126:// change to getdata.equals("00").....equals("07")
            number = Integer.parseInt(field.getData());
            if (number < 0 || number > 7)
                return false;
            break;
        case 167:// already checked in sentence structure
            break;
        case 172:// 02/94 Version BDTSatzbeschreibung, see case 118
            break;

        // ======existence check======//
        case 201:// not needed, already checked
        case 202:// not used
        case 203:// not used
            break;
        case 298:
            if (!field.getSentence().contains(FieldCharacteristic.Bild_Archivierungssnummer.getNumber()))
                return false;
            break;
        case 299:// self defined rule for fields between 5001 and 6001, if theses fields exist field 5000 must exists
            if (!field.getSentence().contains(FieldCharacteristic.Leistungstag.getNumber()))
                return false;
            break;

        // ======Context rules ======//
        case 304:// date < acutal date
            try {
                date = createDateFromField(field);
                if (date == null || date.compareTo(LocalDate.now()) > 0) {
                    return false;
                }
            } catch (DateTimeException dte) {
                return false;
            }
            break;
        case 314:// check if all dates in field 5000 are smaller than this! //better would be to check it at field 5000
            date = createDateFromField(field);
            for (Field field1 : field.getSentence().getFields()) {
                if (field1.getNumber().equals(FieldCharacteristic.Leistungstag.getNumber())) {
                    year = Integer.parseInt(field1.getData().substring(4, 8));
                    month = Integer.parseInt(field1.getData().substring(2, 4));
                    day = Integer.parseInt(field1.getData().substring(0, 2));
                    if (date.compareTo(LocalDate.of(year, month, day)) < 1)
                        return false;
                }
            }
            break;
        case 321:
            if ((field.getSentence().getSentenceType().equals(SentenceCharacteristic.Ueberweisungsfall.id) && field.getData().equals("21")
                    && !(field.getSentence().getSentenceType().contains(FieldCharacteristic.ErlaeuternderTextZurUeberweisung.getNumber())
                            || field.getSentence().getSentenceType().contains(FieldCharacteristic.AnkreuzfeldMuvoLSR.getNumber())
                            || field.getSentence().getSentenceType().contains(FieldCharacteristic.AnkreuzfeldMuvoHAH.getNumber())
                            || field.getSentence().getSentenceType().contains(FieldCharacteristic.AnkreuzfeldAK.getNumber()))))
                return false;
            break;
        case 322:
            if ((field.getSentence().getSentenceType().equals(SentenceCharacteristic.Ueberweisungsfall.id) && field.getData().equals("22")
                    && field.getSentence().getSentenceType().contains(FieldCharacteristic.ErlaeuternderTextZurUeberweisung.getNumber())))
                return false;
            break;
        case 323:
            break;
        case 324:// checks if the given date matches the given quartal
            date = createDateFromField(field);
            otherField = field.getSentence().getField(FieldCharacteristic.QuartalDerAbrechnung.getNumber());
            if (otherField == null)
                return true;
            year = Integer.parseInt(field.getData().substring(4));
            switch (otherField.getData().charAt(0)) {
            case '1':
                startDate = LocalDate.of(year, 1, 1);
                endDate = LocalDate.of(year, 4, 1);
                break;
            case '2':
                startDate = LocalDate.of(year, 4, 1);
                endDate = LocalDate.of(year, 7, 1);
                break;
            case '3':
                startDate = LocalDate.of(year, 7, 1);
                endDate = LocalDate.of(year, 10, 1);
                break;
            case '4':
                startDate = LocalDate.of(year, 10, 1);
                endDate = LocalDate.of(year + 1, 1, 1);
                break;
            default:
                endDate = startDate = null;
            }
            if (startDate == null || date.isBefore(startDate) || date.isEqual(endDate) || date.isAfter(endDate))
                return false;
            break;
        case 325:
            if ((field.getNumber().equals(FieldCharacteristic.Gebuehrenordnung.getNumber()) && (field.getData().contains("1") || field.getData().contains("2"))
                    && field.getSentence().contains(FieldCharacteristic.KilometerNurBeiGOAe.getNumber())))
                return false;
            break;
        case 329:// checks if the Arztnumner is equal to the Artznummer des Absenders
            if (field.getNumber().equals(FieldCharacteristic.Arztnummer.getNumber()) && field.getSentence().getSentenceType().equals(SentenceCharacteristic.Datentraeger_Header.id)
                    && !field.getSentence().getDatacarrier().getSentences().get(0).getField(FieldCharacteristic.ArztnummerDesAbsenders.getNumber()).getData().equals(field.getData()))
                return false;
            break;
        case 330:
            if (field.getNumber().equals(FieldCharacteristic.Praxistyp.getNumber()) && field.getData().equals("3")
                    && !field.getSentence().contains(FieldCharacteristic.ArztMitLeistungskennzeichen.getNumber()))
                return false;
            break;
        case 331:
            if (field.getSentence().getSentenceType().equals(SentenceCharacteristic.AerztlicheBehandlung.id) && field.getNumber().equals(FieldCharacteristic.Scheinuntergruppe.getNumber())
                    && !field.getData().equals("00"))
                return false;
            break;
        case 332:
            if (field.getSentence().getSentenceType().equals(SentenceCharacteristic.Ueberweisungsfall.id) && field.getNumber().equals(FieldCharacteristic.Scheinuntergruppe.getNumber())
                    && !(field.getData().equals("20") || field.getData().equals("21") || field.getData().equals("22") || field.getData().equals("23") || field.getData().equals("24")
                            || field.getData().equals("25") || field.getData().equals("20")))
                return false;
            break;
        case 333:
            if (field.getSentence().getSentenceType().equals(SentenceCharacteristic.BelegeaerztlicheBehandlung.id) && field.getNumber().equals(FieldCharacteristic.Scheinuntergruppe.getNumber())
                    && !(field.getData().equals("30") || field.getData().equals("31")))
                return false;
            break;
        case 334:
            if (field.getSentence().getSentenceType().equals(SentenceCharacteristic.Notfalldienst_Vertretung_Notfalll.id) && field.getNumber().equals(FieldCharacteristic.Scheinuntergruppe.getNumber())
                    && !(field.getData().equals("41") || field.getData().equals("42") || field.getData().equals("43") || field.getData().equals("44") || field.getData().equals("45")))
                return false;
            break;
        case 335:
            if (field.getNumber().equals(FieldCharacteristic.LetzterEinlesetagDerVK_KarteImQuartal.getNumber())
                    && !(field.getSentence().contains(FieldCharacteristic.VersichertennummerDesPatienten.getNumber())
                            && field.getSentence().contains(FieldCharacteristic.Bis_DatumDerGueltigkeit.getNumber())
                            && field.getSentence().contains(FieldCharacteristic.KrankenkassennummerIK.getNumber()) && field.getSentence().contains(FieldCharacteristic.VersichertenstatusVK.getNumber())
                            && field.getSentence().contains(FieldCharacteristic.Ost_West_StatusVK.getNumber())))
                return false;
            break;
        case 338:
            if (field.getNumber().equals(FieldCharacteristic.LetzterEinlesetagDerVK_KarteImQuartal.getNumber())
                    && (field.getSentence().contains(FieldCharacteristic.NameDesHauptversicherten.getNumber())
                            || field.getSentence().contains(FieldCharacteristic.GeburtsdatumDesHauptversicherten.getNumber())
                            || field.getSentence().contains(FieldCharacteristic.VornameDesHauptversicherten.getNumber())
                            || field.getSentence().contains(FieldCharacteristic.WohnortDesHauptversicherten.getNumber())
                            || field.getSentence().contains(FieldCharacteristic.StrasseDesHauptversicherten.getNumber())
                            || field.getSentence().contains(FieldCharacteristic.Gueltigkeitsdatum.getNumber()) || field.getSentence().contains(FieldCharacteristic.Geschaeftsstelle.getNumber())))
                return false;
            break;
        case 350:
            if (field.getNumber().equals(FieldCharacteristic.Praxistyp.getNumber()) && (field.getData().equals("2") || field.getData().equals("3"))
                    && !field.getSentence().contains(FieldCharacteristic.AnzahlAerzte.getNumber()))
                return false;
            break;
        case 351:// erste Stelle von feld 4112 muss identisch zu inhalt 3108
            if (field.getNumber().equals(FieldCharacteristic.VersichertenstatusVK.getNumber()) && (!field.getSentence().contains(FieldCharacteristic.VersichertenartMFR.getNumber())
                    && !field.getData().substring(0, 1).equals(field.getSentence().getField(FieldCharacteristic.VersichertenartMFR.getNumber()).getData())))
                break;
        case 352:// not needed
            break;
        case 353:
            if (field.getNumber().equals(FieldCharacteristic.VersichertennummerDesPatienten.getNumber()) && (field.getNumber().length() < 6 || field.getNumber().length() > 12)
                    && (!(field.getSentence().contains(FieldCharacteristic.AbrechnungsartSchein.getNumber())
                            && (field.getSentence().getField(FieldCharacteristic.AbrechnungsartSchein.getNumber()).getData().equals("1"))
                            || field.getSentence().getField(FieldCharacteristic.AbrechnungsartSchein.getNumber()).getData().equals("2"))))
                return false;
            break;
        // self defined rule to check if the zip code city field contains the zip code and a address string or just one if them
        case -1:
            Pattern pattern = Pattern.compile("[0-9]{5}");
            Matcher matcher = pattern.matcher(field.getData());
            if (!matcher.find() || matcher.group().length() == field.getData().length()) {
                createLogEntry(field.getNumber(), false, "Rule -1 was violated: zip code or city name is missing for city field");
            }
            // don´t return false, would result in a nullyfied field content, but content is still usable
            break;
        /**
         * This rule was created on 11.12.2020. Knowing the poor performance and little anonymization power of this function, it was requested by UMGa.
         * It checks the free text fields for keywords from a city- and a patternkeyword-file and tags the fields with a rule violation warning.
         */
//        case -2:
//            if (field.getData() != null && !field.getData().isEmpty()) {
//                if (lstKnownIdentifierPatterns != null) {
//                    for (String patter1 : lstKnownIdentifierPatterns) {
//                        Pattern pattern1 = Pattern.compile(patter1);
//                        Matcher matcher1 = pattern1.matcher(field.getData());
//                        if (matcher1.find()) {
//                            return false;
//                        }
//                    }
//                }
//            }
//            break;
        /**
         * This rule was created on 11.12.2020. Knowing the poor performance and little anonymization power of this function, it was requested by UMGa.
         * It checks the free text fields for keywords from a city- and a patternkeyword-file and tags the fields with a rule violation warning.
         */
//        case -3:
//            if (field.getData() != null && !field.getData().isEmpty()) {
//                if (lstCities != null) {
//                    for (String patter2 : lstCities) {
//                        Pattern pattern2 = Pattern.compile(patter2);
//                        Matcher matcher2 = pattern2.matcher(field.getData());
//                        if (matcher2.find()) {
//                            return false;
//                        }
//                    }
//                }
//            }
//            break;
        // default
        default:
            throw new IllegalArgumentException("Rule " + rule + " unknown!");
        }
        return true;
    }

    /**
     * checks if the given day,year, month is valid Date
     *
     * @param year the year of the date
     * @param month the month of the date
     * @param day the day of the date
     * @return true if it`s a valid date, false otherwise
     */
    private boolean validDate(int year, int month, int day) {
        try {
            LocalDate.of(year, month, day);
        } catch (DateTimeException dte) {
            return false;
        }
        return true;
    }

    /**
     * creates a date from the data part of a given field
     *
     * @param field the field which data part should be converted to date
     * @return the date as LocalDate
     */
    public static LocalDate createDateFromField(Field field) {
        int year = Integer.parseInt(field.getData().substring(4, 8));
        int month = Integer.parseInt(field.getData().substring(2, 4));
        int day = Integer.parseInt(field.getData().substring(0, 2));
        return LocalDate.of(year, month, day);
    }

    /**
     * checks if the given string is a valid time format according to the specification
     *
     * @param dataString the string that should be checked
     * @return true if its a valid time, false otherwise
     */
    private boolean validTime(String dataString) {
        int data = Integer.parseInt(dataString);
        return (data > 0 && data < 2359); // abfrage entsprechende regeltabelle, sinvoller: stunden zwischen 0 und 24: min zwischen 0 und 60
    }

    /**
     * checks if the data of the field matches the given datatyp
     *
     * @param field the field which content should be checked
     * @param datatyp the data typ the field should match
     * @return true if the given dataty pmatches, false otherwise
     * @throws IllegalDatatypException thrown if the data typ doesn`t match the given field, but the error could be corrected
     */
    private boolean fieldDatatypMatch(Field field, Datatyp datatyp) throws IllegalDatatypException {
        if(field.getData()==null || field.getData().isEmpty()){
            createLogEntry(field.getNumber(),false,"WARNING: EMPTY FIELD DATA");
            field.setValid(false);
            return false;
        }
        switch (datatyp) {
        case NUM:
            // if(!field.getData().matches("\\d+")) {
            if (!field.getData().matches("-?[0-9]+")) {
                String fieldData = field.getData().toUpperCase();
                if (fieldData.equals("M")) {
                    field.setData("1");
                    createLogEntry(field.getNumber(), false, "sex was wrongly formatted");
                } else if (fieldData.equals("W") || fieldData.equals("F")) {
                    field.setData("2");
                    createLogEntry(field.getNumber(), false, "sex was wrongly formatted");
                } else
                    return false;
            }
            break;
        case DATE:
            if (!field.getData().matches("\\d{8}")) {
                if (field.getData().matches("\\d{2}[-]+\\d{2}[-]+\\d{4}")) {
                    field.setData(field.getData().replaceAll("-", ""));
                    createLogEntry(field.getNumber(), false, "Field of data typ date was wrongly formatted. Found a minus as delimiter, removed it." + field.getData());
                    setNewSize(field);
                    throw new IllegalDatatypException("");
                }
                return false;
            }
            break;
        case ALNUM:
        		if (!field.getData().matches("\\p{Alnum}{" + (field.getData().length() - 1) + "}"))
    				break;
        	return false;

        case FLOAT:
            if (!field.getData().matches("-?\\d+(\\.\\d+)?")) {
                if (field.getData().matches("-?\\d+(,\\d+)?")) {
                    createLogEntry(field.getNumber(), false, "field datatyp is float, found a comma instead a point as seperator. Corrected it to right format");
                    field.setData(field.getData().replace(',', '.'));
                    throw new IllegalDatatypException("");
                }
                return false;
            }
            break;
        default:
            return false;

        }
        return true;
    }

    /**
     * Updates the size of the field and the corresponding sentence and datacarrier
     *
     * @param field which size should be updated
     */
    private void setNewSize(Field field) {
        int length = Integer.parseInt(field.getLength());
        int newLength = BASIC_FIELD_LENGTH + field.getData().length();
        field.setLength(String.format("%03d", newLength));
        // update sentence size
        field.getSentence().setSize(field.getSentence().getSize() - length + newLength);
        // update datacarrier size
        datacarrier.setSize(String.valueOf(datacarrier.getSize() - length + newLength));
    }

    /**
     * set NULLIFIED_STRING as the Datapart of the field and updates the size afterwards using the setNewSizeMethode
     *
     * @param field the field whichs datapart should be nullified
     */
    private void nullifieFieldData(Field field) {
        // update field size
        field.setData(NULLIFIED_STRING);
        field.setValid(false);
        setNewSize(field);
    }

    /**
     * creates a Log entry using the given params
     *
     * @param fieldNumber the field where the validation error occurred
     * @param critical it is a critical error ?
     * @param errorMessage what is wrong
     */
    private void createLogEntry(String fieldNumber, boolean critical, String errorMessage) {
        StringBuilder stringBuilder = new StringBuilder();
        if (critical)
            stringBuilder.append("Critical ");
        stringBuilder.append("FieldValidationError - Fieldnumber ");
        stringBuilder.append(fieldNumber);
        stringBuilder.append(": ");
        stringBuilder.append(validationPart);
        stringBuilder.append(" ");
        stringBuilder.append(errorMessage);
        logger.log(Level.ERROR, stringBuilder.toString());
    }

    /**
     * Method to create a log entry, when it was not possible to access the sentence typ.
     * @param e The exception that was thrown while trying to access the sentence typ.
     */
    private void createNoSenteceTypFoundLogEntry(Exception e) {
        logger.error("Tried to access sentence typ. Wasnt`t possible because the list of fields in that sentence is empty."
                + " It looks like an error occurred while unmarshalling the datafiles! ExceptionMessage: " + e.getMessage());
    }

}
