package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.LinkedList;

/**
 * Class used to store a case of a patient
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTCase extends Referencer {
    /**
     * Enum for the case types
     */
    public enum CaseTypes {
        aerztlicheBehandlung, ueberweisungsFall, belegAerztlicheBehandlung, notfallDienst, privatAbrechnung, bgAbrechnung, unstrukturierteFaelle;
    }

    private BDTValue<String> geschaeftsstelle;
    private BDTValue<String> quartal;
    private BDTValue<String> abrechnungsArt;
    private BDTValue<String> gebuehrenOrdnung;
    private LinkedList<BDTTreatment> treatments;
    private LinkedList<BDTDiagnose> abrechnungsDiagnosen;
    private CaseTypes type;

    /**
     * public constructor creating a case and adds a treatment
     * 
     * @param treatmentDay the date the treatment took place
     * @param bdtValueDate the date as BDTValue
     */
    public BDTCase(LocalDate treatmentDay, BDTValue<String> bdtValueDate) {
        this();
        addTreatment(treatmentDay, bdtValueDate);
    }

    /**
     * private default constructor
     * initialise all lists
     */
    private BDTCase() {
        treatments = new LinkedList<>();
        abrechnungsDiagnosen = new LinkedList<>();
    }

    /**
     * adds a treatment to the list of treatments in this case
     * 
     * @param treatmentDay the day the treatment took place
     * @param bdtValueDate the day as BDTValue
     */
    public void addTreatment(LocalDate treatmentDay, BDTValue<String> bdtValueDate) {
        treatments.add(new BDTTreatment(treatmentDay, bdtValueDate));
    }

    /**
     * gets a BDTTreatment by the date it took place
     * 
     * @param treatmentDay the date the treatment took place
     * @return the treatment, null if no treatment found
     */
    public BDTTreatment getTreatmentsByDate(LocalDate treatmentDay) {
        for (BDTTreatment treatment : treatments) {
            LocalDate treatmentDate = treatment.getDate();
            if (treatmentDate.isEqual(treatmentDay)) {
                return treatment;
            }
        }
        return null;
    }

    /**
     * Returns all attritbutes as list of Referenceables.
     * 
     * @return the list of Referenceables
     * @throws IllegalAccessException if the super method throws it
     */
    @Override
    public LinkedList<Referenceable> getAsReferenceables() throws IllegalAccessException {
        LinkedList<Referenceable> list = super.getAsReferenceables();
        for (BDTTreatment treatment : treatments)
            list.addAll(treatment.getAsReferenceables());
        for (BDTDiagnose diagnose : abrechnungsDiagnosen) {
            list.addAll(diagnose.getAsReferenceables());
        }
        return list;
    }

    /**
     * Return a list of days at which a treatment took place.
     * 
     * @return the list of dates
     */
    public ArrayList<LocalDate> getTreamentDates() {
        ArrayList<LocalDate> dates = new ArrayList<>();
        for (BDTTreatment treatment : treatments) {
            dates.add(treatment.getDate());
        }
        return dates;
    }

    // getter setter
    public BDTValue<String> getGeschaeftsstelle() {
        return geschaeftsstelle;
    }

    public void setGeschaeftsstelle(BDTValue<String> geschaeftsstelle) {
        this.geschaeftsstelle = geschaeftsstelle;
    }

    public BDTValue<String> getQuartal() {
        return quartal;
    }

    public void setQuartal(BDTValue<String> quartal) {
        this.quartal = quartal;
    }

    public BDTValue<String> getAbrechnungsArt() {
        return abrechnungsArt;
    }

    public void setAbrechnungsArt(BDTValue<String> abrechnungsArt) {
        this.abrechnungsArt = abrechnungsArt;
    }

    public BDTValue<String> getGebuehrenOrdnung() {
        return gebuehrenOrdnung;
    }

    public void setGebuehrenOrdnung(BDTValue<String> gebuehrenOrdnung) {
        this.gebuehrenOrdnung = gebuehrenOrdnung;
    }

    public CaseTypes getType() {
        return type;
    }

    public void setType(CaseTypes type) {
        this.type = type;
    }

    public LinkedList<BDTTreatment> getTreatments() {
        return treatments;
    }

    public void setTreatments(LinkedList<BDTTreatment> treatments) {
        this.treatments = treatments;
    }

    public LinkedList<BDTDiagnose> getAbrechnungsDiagnosen() {
        return abrechnungsDiagnosen;
    }

    public void setAbrechnungsDiagnosen(LinkedList<BDTDiagnose> abrechnungsDiagnosen) {
        this.abrechnungsDiagnosen = abrechnungsDiagnosen;
    }

}
