package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Properties;


import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datacarrier.Datacarrier;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datapackage.Datapackage;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.IllegalSentenceStructureException;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.Sentence;
import de.uni_goettingen.med.mi.bdtparser.core.io.writer.Unmarshaller;
import de.uni_goettingen.med.mi.bdtparser.core.validator.Validator;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * class used to save to the data from the vpr back to the bdt files using the old structure
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTFileSaver {
    Logger log = LogManager.getLogger(getClass());
    private Datapackage datapackage;
    private LinkedList<Datacarrier> datacarriers;
    private Properties saveProperties = new Properties();
    private InputStream inputStream = null;
	private String path = "";

    /**
     * Constructor
     * 
     * @param datapackage the datapackage used to reconstruct the bdt files
     */
	public BDTFileSaver(Datapackage datapackage, String path) {
		this.datapackage = datapackage;
		datacarriers = datapackage.getDatacarriers();
		log.debug(path);
		if (path == null)
			throw new IllegalArgumentException("path must no be null");
		this.path = path;
		inputStream = this.getClass().getResourceAsStream("/saver/saver.properties");
		if (inputStream != null) {
			try {
				saveProperties.load(inputStream);
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}
	}

    /**
     * Saves the data from the vpr back to bdt files using the old datapackage structure and removes invalid and not in the vpr stored fields
     * 
     * @param arztpraxis the arxtpraxis from which the data should be written back
     */
    public void save(BDTArztpraxis arztpraxis) {
        HashMap<Long, BDTPatient> patients = arztpraxis.getPatients();
        LinkedList<Referenceable> mergedLists = arztpraxis.getAsReferenceables();
        for (BDTPatient patient : patients.values()) {
            LinkedList<Referenceable> patientAsReferenceable = patient.getAsReferenceables();
            mergedLists.addAll(patientAsReferenceable);
        }
        for (Referenceable referencable : mergedLists) {
            if (referencable != null && referencable.getValue() != null) {
                for (Object referenceObject : referencable.getReferences()) {
                    Field reference = ((Field) referenceObject);
                    setField(referencable.getValue().toString(), reference);
                }
            }
        }
        markDefaultSentences();
        reduceToSpecifiedFields();
        reduceDatapackage();
        updateSize();
        Unmarshaller unmarshaller = new Unmarshaller(datapackage, path);
        unmarshaller.run();
    }

    protected void reduceToSpecifiedFields() {
        if (saveProperties != null) {
            StringBuilder sbAllowedFields = new StringBuilder();
            sbAllowedFields.append(saveProperties.getProperty("AllowedPatientFields"));
            sbAllowedFields.append(saveProperties.getProperty("AllowedGPFields"));
            sbAllowedFields.append(saveProperties.getProperty("AllowedBDTStructureFields"));
            ArrayList<String> lstAllowedFields = new ArrayList<String>(Arrays.asList(sbAllowedFields.toString().split(",")));
            Iterator<Datacarrier> datacarrierIterator = datacarriers.listIterator();
            while (datacarrierIterator.hasNext()) {
                Datacarrier datacarrier = datacarrierIterator.next();
                Iterator<Sentence> sentenceIterator = datacarrier.getSentences().listIterator();
                while (sentenceIterator.hasNext()) {
                    Sentence sentence = sentenceIterator.next();
                    LinkedList<Field> fields = sentence.getFields();
                    for (Field field : fields) {
                        if (!lstAllowedFields.contains(field.getNumber())) {
                            field.used = false;
                        }
                    }
                }
            }
        }
    }

    private void removeEmptySentences(){

    }

    /**
     * updates the size (amount of fields, should be bytes by specification ) in the datapackage end sentence
     */
    void updateSize() {
        long datapackageLength = 0;
        for (Datacarrier datacarrier : datacarriers) {
            long datacarrierLength = 0;
            for (Sentence sentence : datacarrier.getSentences()) {
                long sentenceLength = 0;
                for (Field field : sentence.getFields()) {
                    sentenceLength += Long.parseLong(field.getLength());
                }
                datacarrierLength += setSetentenceLength(sentenceLength, sentence.getFields().get(1));
            }
            datapackageLength += datacarrierLength;
        }
        updateDatapackageEndSentence(datapackageLength);
    }

    /**
     * updates the length of the package in the datapackage end sentence
     * 
     * @param datapackageLength the length of all field together in the datapackage
     */
    private void updateDatapackageEndSentence(long datapackageLength) {
        Datacarrier lastCarrier = datacarriers.getLast();
        Sentence lastSentence = lastCarrier.getSentences().get(lastCarrier.getSentences().size() - 2);
        Field field = lastSentence.getField(FieldCharacteristic.AnzahlDatentraegerDesDP.getNumber());
        if (field != null) {
            field.setData(String.valueOf(datacarriers.size()));
            field.setLength(String.format("%1$03d", Validator.BASIC_FIELD_LENGTH + datacarriers.size()));
            field.used = true;
            setSetentenceLength(datapackageLength, lastCarrier.getSentences().get(lastCarrier.getSentences().size() - 2).getFields().get(2));
        } else {
            System.out.println("ERROR UPDATING FIELD");
        }
    }

    /**
     * Removes unused fields, sentences and datacarriers from the datapackage
     */
    protected void reduceDatapackage() {
        Iterator<Datacarrier> datacarrierIterator = datacarriers.listIterator();
        while (datacarrierIterator.hasNext()) {
            Datacarrier datacarrier = datacarrierIterator.next();
            if (!datacarrier.used) {
                datacarrierIterator.remove();
            } else {
                Iterator<Sentence> sentenceIterator = datacarrier.getSentences().listIterator();
                while (sentenceIterator.hasNext()) {
                    Sentence sentence = sentenceIterator.next();
                    if (!sentence.used || isEmpty(sentence)) {
                        sentenceIterator.remove();
                    } else {
                        sentence.getFields().removeIf(field -> !field.used);
                    }
                }
            }
        }
    }

    private boolean isEmpty(Sentence sentence) {
        try {
            String sentenceType = sentence.getSentenceType();
            return sentence.getFields().size()==3 &&( sentenceType.equals("0101") || sentenceType.equals("6200"));
        }catch (IllegalSentenceStructureException isse){
            log.error("Could not access Sentencetyp from Sentence");
            return false;
        }


    }

    /**
     * Marks the sentences which are not stored in the vpr, but are needed for the bdt-files
     */
    protected void markDefaultSentences() {
        // first carrier
        Datacarrier datacarrier = datacarriers.getFirst();
        markAllFieldsInSentence(datacarrier.getSentences().get(0));
        markAllFieldsInSentence(datacarrier.getSentences().get(1));
        int carriersize = datacarrier.getSentences().size();
        markAllFieldsInSentence(datacarrier.getSentences().get(carriersize - 1));

        // last carrier
        datacarrier = datacarriers.getLast();
        markAllFieldsInSentence(datacarrier.getSentences().get(1));
        carriersize = datacarrier.getSentences().size();
        markAllFieldsInSentence(datacarrier.getSentences().get(carriersize - 1));
        markAllFieldsInSentence(datacarrier.getSentences().get(carriersize - 2));
    }

    /**
     * marks all fields in the given sentence , the sentence and the datacarrier as used.
     * 
     * @param sentence the sentence in that all fields should be marked
     */
    private void markAllFieldsInSentence(Sentence sentence) {
        sentence.getDatacarrier().used = true;
        sentence.used = true;
        for (Field field : sentence.getFields())
            field.used = true;
    }

    /**
     * sets the data part of a given field, and marked it as used
     * 
     * @param fieldData the data for the field
     * @param reference the reference to the field
     */
    protected void setField(String fieldData, Field reference) {
        reference.setData(fieldData);
        reference.setLength(String.format("%1$03d", Validator.BASIC_FIELD_LENGTH + fieldData.length()));
        reference.used = true;
        reference.getSentence().used = true;
        reference.getSentence().getFields().getFirst().used = true;
        reference.getSentence().getField(FieldCharacteristic.Satzlaenge.getNumber()).used = true;
        markCarrier(reference.getSentence().getDatacarrier());
    }

    /**
     * marks the datacarrier as used ( including the header and end sentence)
     * 
     * @param datacarrier the datacarrier that should be marked as used
     */
    private void markCarrier(Datacarrier datacarrier) {
        if (datacarrier.used)
            return;
        datacarrier.used = true;
        markAllFieldsInSentence(datacarrier.getSentences().get(0));
        markAllFieldsInSentence(datacarrier.getSentences().get(datacarrier.getSentences().size() - 1));
    }

    /**
     * sets the length of the sentence ands returns the final length
     * 
     * @param sentenceLength the length of the sentence
     * @param sentenceLengthField the field which stored the length of the field
     * @return
     */
    private long setSetentenceLength(long sentenceLength, Field sentenceLengthField) {
        sentenceLength = String.valueOf(sentenceLength).length() + sentenceLength;
        String sentencelength = String.format("%1$05d", sentenceLength);
        String fieldLength = String.format("%1$03d", Validator.BASIC_FIELD_LENGTH + sentencelength.length());
        sentenceLengthField.setData(sentencelength);
        sentenceLengthField.setLength(fieldLength);
        return sentenceLength;
    }

    protected Datapackage getDatapackage() {
        return datapackage;
    }

    protected String getPath() {
        return path;
    }

}
