package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.lang.reflect.Field;
import java.time.DateTimeException;
import java.time.LocalDate;
import java.util.LinkedList;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Only implements the fields that a requested for the RADAR-Project.
 * Main Class of the VPR. Stores all the information to one patient. With special getter and settern to get the values in a visualistion you can work with.
 * Uses Class BDTValue as a wrapper to store the value and a reference to the original field or sentence. The reference can be used to rebuild the bdt-files.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTPatient extends Referencer {
    private static final Logger logger = LogManager.getLogger(BDTPatient.class);
    private LocalDate DefaultStartDate, DefaultEndDate;
    private BDTValue<String> vorName;
    private BDTValue<String> nachName;
    private BDTValue<String> patientNummer;
    private Long patientNummerKey;
    private BDTValue<String> geburtsDatum;
    private LocalDate birthday;
    private BDTValue<String> geschlecht;
    private LinkedList<BDTValue<String>> allergien;
    private BDTAdresse adresse;
    private LinkedList<BDTDauerdiagnose> dauerDiagnosen;
    private LinkedList<BDTDauermedikamentation> dauerMedikamentation;
    private LinkedList<BDTCase> BDTCases;
    private int age;
    private int birthYear;


    public BDTPatient() {
        adresse = new BDTAdresse();
        dauerMedikamentation = new LinkedList<>();
        dauerDiagnosen = new LinkedList<>();
        BDTCases = new LinkedList<>();
        allergien = new LinkedList<>();
    }

    /**
     * getter for a case by a date with a treatment at this date
     * 
     * @param date the date of the treatment
     * @return the case, null if no case witch at treatment at the day exist
     */
    public BDTCase getCaseByDate(LocalDate date) {
        if (date == null)
            throw new IllegalArgumentException("Date must be not null!");
        int quartal = (date.getMonth().getValue() - 1) / 3 + 1;
        for (BDTCase singleBDTCase : BDTCases) {
            LocalDate treatmentDate = singleBDTCase.getTreatments().getFirst().getDate();
            if (date.getYear() == treatmentDate.getYear() && ((treatmentDate.getMonthValue() - 1) / 3 + 1) == quartal)
                return singleBDTCase;
        }
        return null;
    }

    /**
     * adds a case for this patient
     * 
     * @param bdtCase
     */
    public void addCase(BDTCase bdtCase) {
        if (bdtCase == null)
            throw new IllegalArgumentException("BDTCase must be not null");
        BDTCases.add(bdtCase);
    }

    /**
     * The methode delivers the age of the patient as integer. It may throw an exception
     * 
     * @return the age as integer
     */
    public int getAge() {
        return age;
    }

    public int getBirthYear() {
        return birthYear;
    }

    public String getSex() {
        if (geschlecht != null && geschlecht.getValue() != null)
            return geschlecht.getValue();
        return null;

    }

    /**
     * Returns all attributes of the patient as list of referencebales
     * 
     * @return the list of referenceable
     */
    public LinkedList<Referenceable> getAsReferenceables() {
        LinkedList<Referenceable> list = new LinkedList<>();
        try {
            Field[] fields = this.getClass().getDeclaredFields();
            for (Field field : fields) {
                field.setAccessible(true);
                if (field.getType() == BDTValue.class)
                    list.add((Referenceable) field.get(this));
                else if (field.getType() == LinkedList.class) {
                    LinkedList BDTValueList = (LinkedList) field.get(this);
                    if (BDTValueList != null && !BDTValueList.isEmpty()) {
                        if (BDTValueList.getFirst().getClass() == BDTValue.class) {
                            list.addAll(BDTValueList);
                        } else {
                            LinkedList<Referencer> referencers = BDTValueList;
                            for (Referencer tmp : referencers) {
                                list.addAll(tmp.getAsReferenceables());
                            }
                        }
                    }
                    field.setAccessible(false);
                } else if (field.getType() == BDTAdresse.class) {
                    list.addAll(((BDTAdresse) field.get(this)).getAsReferenceables());
                }
            }
        } catch (IllegalAccessException iae) {
            logger.error("An error occured while collecting the references to write the data from vpr to file");
        }
        return list;
    }

    /**
     * Delivers all BDTDiagnose as String by calling #getAllDiagnosen(LocalDate.Min, LocalDate.Max)
     * The list also contains all the dauerDiagnosen
     * 
     * @return a LinkedList containing all BDTDiagnose including the dauerDiagnosen
     */
    public LinkedList<String> getAllDiagnosen() {
        LinkedList list = getAllDiagnosen(LocalDate.MIN, LocalDate.MAX);
        for (BDTDauerdiagnose diagnoseAb : dauerDiagnosen) {
            for (BDTValue<String> diagnose : diagnoseAb.diagnosen)
                list.add(diagnose.toString());
        }
        return list;
    }

    public LinkedList<String> getAllDauerDiagnosen() {
        LinkedList list = new LinkedList();
        for (BDTDauerdiagnose diagnoseAb : dauerDiagnosen) {
            for (BDTValue<String> diagnose : diagnoseAb.diagnosen)
                list.add(diagnose.toString());
        }
        return list;
    }

    public LinkedList<String> getAllDauerMedikamente() {
        LinkedList list = new LinkedList();
        for (BDTDauermedikamentation dauerMedik : dauerMedikamentation) {
            for (BDTValue<String> med : dauerMedik.medikamente)
                list.add(med.toString());
        }
        return list;
    }

    /**
     * adds a reference to a field to the patientnumber
     * 
     * @param reference the field that holds the patientnumber
     * @throws IllegalStateException if the patientnumber wasn't set bevor calling this methode
     */
    public void addPatientNumberReference(de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field reference) throws IllegalStateException {
        if (patientNummer == null)
            throw new IllegalStateException("patientnumber was't set, bevor trying to add a reference");
        patientNummer.references.add(reference);
    }

    /**
     * Delivers all BDTDiagnose from the given time interval, if no interval is given the default dates are used.<br/>
     * This method does not include the dauerDiagnosen.
     * 
     * @param startDate the start of the interval, if null the default startDate is used
     * @param endDate the end of the interval, if null the default endDate is used
     * @throws IllegalArgumentException if startDate is after endDate
     * @return a list from all BDTDiagnose as Strings between startDate and endDate
     */
    public LinkedList<String> getAllDiagnosen(LocalDate startDate, LocalDate endDate) {
        if (startDate == null)
            startDate = DefaultStartDate;
        if (endDate == null)
            endDate = DefaultEndDate;
        if (startDate.isAfter(endDate))
            throw new IllegalArgumentException(String.format("Startdate %1$s is after Enddate %2$s", startDate.toString(), endDate.toString()));
        LinkedList<String> diagnosen = new LinkedList<>();

        return diagnosen;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;

        BDTPatient that = (BDTPatient) o;

        return patientNummer.equals(that.patientNummer);
    }

    @Override
    public int hashCode() {
        return Integer.parseInt(patientNummer.getValue());
    }

    public BDTAdresse getAdresse() {
        return adresse;
    }

    public void setAdresse(BDTAdresse adresse) {
        this.adresse = adresse;
    }

    public LinkedList<BDTCase> getBDTCases() {
        return BDTCases;
    }

    public void setBDTCases(LinkedList<BDTCase> bDTCases) {
        BDTCases = bDTCases;
    }

    public BDTPatient(BDTValue<String> patientNummer) {
        this();
        this.patientNummer = patientNummer;
        try {
            this.patientNummerKey = Long.parseLong(patientNummer.getValue());
        } catch (NumberFormatException nfe) {
            logger.error("could not parse patient number" + nfe.getMessage());
        }
    }

    public void setVorName(BDTValue<String> vorName) {
        this.vorName = vorName;
    }

    public void setNachName(BDTValue<String> nachName) {
        this.nachName = nachName;
    }

    public void setPatientNummer(BDTValue<String> patientNummer) {
        this.patientNummer = patientNummer;
        try {
            this.patientNummerKey = Long.parseLong(patientNummer.getValue());
        } catch (NumberFormatException nfe) {
            // nothing
        }
    }

    public Long getPatientNummerKey() {
        return patientNummerKey;
    }

    public void setGeburtsDatum(BDTValue<String> geburtsDatum) {
        this.geburtsDatum = geburtsDatum;
        try {
            if (geburtsDatum != null) {
                String strGeburtsDatum = geburtsDatum.toString();
                if (strGeburtsDatum.length() == 8) {
                    int day = Integer.parseInt(strGeburtsDatum.substring(0, 2));
                    int month = Integer.parseInt(strGeburtsDatum.substring(2, 4));
                    int year = Integer.parseInt(strGeburtsDatum.substring(4));
                    birthYear = year;
                    LocalDate now = LocalDate.of(2013, 8, 22);// LocalDate.now();
                    this.birthday = LocalDate.of(year, month, day);
                    if (now.isBefore(birthday.plusYears(now.getYear() - birthday.getYear())))
                        age = now.getYear() - birthday.getYear() - 1;
                    else
                        age = now.getYear() - birthday.getYear();
                } else if (strGeburtsDatum.length() == 4) { // walkaround to enable bdt-viewer to parse pseudonymised (to year) birthdates
                    int year = Integer.parseInt(strGeburtsDatum.substring(0, 4));
                    birthYear = year;
                    geburtsDatum.setValue(String.valueOf(year));
                }
            } else {
                logger.error("birthdate is null");
            }
        } catch (DateTimeException dte) {
            logger.error(dte.getLocalizedMessage());
        }
    }

    public void setGeschlecht(BDTValue<String> geschlecht) {
        this.geschlecht = geschlecht;
    }

    public BDTValue<String> getVorName() {
        return vorName;
    }

    public BDTValue<String> getNachName() {
        return nachName;
    }

    public BDTValue<String> getPatientNummer() {
        return patientNummer;
    }

    public BDTValue<String> getGeburtsDatum() {
        return geburtsDatum;
    }

    public LocalDate getGeburtsDatumDate() {
        return this.birthday;
    }

    public BDTValue<String> getGeschlecht() {
        return geschlecht;
    }

    public LinkedList<BDTValue<String>> getAllergien() {
        return allergien;
    }

    public void addAllergie(BDTValue bdtValue) {
        allergien.add(bdtValue);
    }

    public LinkedList<BDTDauerdiagnose> getDauerDiagnosen() {
        return dauerDiagnosen;
    }

    public LinkedList<BDTDauermedikamentation> getDauerMedikamentation() {
        return dauerMedikamentation;
    }

    public void setDauerMedikamentation(LinkedList<BDTDauermedikamentation> dauerMedikamentation) {
        this.dauerMedikamentation = dauerMedikamentation;
    }

    public int getVisitCount(LocalDate from, LocalDate to) {
        int count = 0;
        for (BDTCase bdtCase : BDTCases) {
            for (BDTTreatment treatment : bdtCase.getTreatments()) {
                if (treatment.getDate().isAfter(from) && treatment.getDate().isBefore(to))
                    count++;
            }
        }
        return count;
    }

    public void setDauerDiagnosen(LinkedList<BDTDauerdiagnose> dauerDiagnosen) {
        this.dauerDiagnosen = dauerDiagnosen;
    }

    public void setAge(String age) {
        if (geburtsDatum != null)
            geburtsDatum.setValue(age);
    }

    public void setSex(String sex) {
        if (geschlecht != null)
            geschlecht.setValue(sex);
    }

    public void setZipCode(String zipCode) {
        if (adresse != null)
            adresse.setZipCode(zipCode);
    }

}
