package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.lang.reflect.Field;
import java.time.LocalDate;
import java.util.LinkedList;

/**
 * Class used to store a BDTTreatment of a patient from the bdt files.
 * Doesn't include all values from the bdtfiles at the moment
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTTreatment extends Referencer {
    private BDTValue<String> behandlungsTag;
    private LocalDate date;
    private LinkedList<BDTGNR> gebuehrennummern;
    private LinkedList<BDTDiagnose> diagnosen;
    private LinkedList<BDTValue<String>> medikamenteOhneRezept;
    private LinkedList<BDTValue<String>> medikamenteAufRezept;
    private LinkedList<BDTValue<String>> medikamentAerztemuster;
    private LinkedList<BDTValue<String>> therapie;
    private LinkedList<BDTValue<String>> physikTherapie;
    private LinkedList<BDTValue<String>> ueberweisungInhalt;
    private BDTArbeitsunfaehigkeit arbeitsunfaehigkeit;
    private LinkedList<BDTBefundArt> befundArten;
    private BDTKrankenhausEinweisung krankenhausEinweisungen;
    private LinkedList<BDTBefund> befunde;

    /**
     * Public default constructor. Needs a date to reference the treatment and the bdtvalue where it's string representation and field reference is stored
     * 
     * @param treatmentDate the date of the treatment
     * @param bdtValueTreatmentDay the BDTValue of the date
     */
    public BDTTreatment(LocalDate treatmentDate, BDTValue bdtValueTreatmentDay) {
        this();
        behandlungsTag = bdtValueTreatmentDay;
        this.date = treatmentDate;
    }

    /**
     * private default constructor initialised all the list in the class
     */
    private BDTTreatment() {
        gebuehrennummern = new LinkedList<>();
        diagnosen = new LinkedList<>();
        medikamenteAufRezept = new LinkedList<>();
        medikamenteOhneRezept = new LinkedList<>();
        medikamentAerztemuster = new LinkedList<>();
        befunde = new LinkedList<>();
        therapie = new LinkedList<>();
        physikTherapie = new LinkedList<>();
        ueberweisungInhalt = new LinkedList<>();
        befundArten = new LinkedList<>();
    }

    /**
     * Returns the date of the treatment as Local Date to allow to work with it.
     * 
     * @return the date of the treatment
     */
    public LocalDate getDate() {
        return date;
    }

    /**
     * Overwrites the the getAsReferenceables methode. Returns a list of all stored values as Referenceables. The list includes all the values stored in the list
     * 
     * @return the list of all stored values
     * @throws IllegalAccessException if a field is not accessible
     */
    @Override
    public LinkedList<Referenceable> getAsReferenceables() throws IllegalAccessException {
        LinkedList<Referenceable> list = new LinkedList<>();
        Field[] fields = this.getClass().getDeclaredFields();
        for (Field field : fields) {
            field.setAccessible(true);
            // simple bdtvalue just add
            if (field.getType() == BDTValue.class)
                list.add((Referenceable) field.get(this));
            // class implements getAsReference, call and add it
            else if (field.getType() == BDTArbeitsunfaehigkeit.class || field.getType() == BDTKrankenhausEinweisung.class) {
                Referencer referencer = (Referencer) (field.get(this));
                if (referencer != null)
                    list.addAll(referencer.getAsReferenceables());
            }
            // is a linked list
            else if (field.getType() == LinkedList.class) {
                LinkedList BDTValueList = (LinkedList) field.get(this);
                if (!BDTValueList.isEmpty()) {
                    // of simple bdtvalues, just add
                    if (BDTValueList.getFirst().getClass() == BDTValue.class || BDTValueList.getFirst().getClass() == BDTBefund.class) {
                        list.addAll(BDTValueList);
                        // of classes which export getAsReferenceable, just call and add it
                    } else {
                        LinkedList<Referencer> referencers = BDTValueList;
                        for (Referencer tmp : referencers) {
                            list.addAll(tmp.getAsReferenceables());
                        }
                    }
                }
                field.setAccessible(false);
            }
        }
        return list;
    }

    // Setter and Getter for all the stored values

    public void setBehandlungsTag(BDTValue<String> behandlungsTag) {
        this.behandlungsTag = behandlungsTag;
    }

    public BDTValue<String> getBehandlungsTag() {
        return behandlungsTag;
    }

    public LinkedList<BDTGNR> getGebuehrennummern() {
        return gebuehrennummern;
    }

    public LinkedList<BDTDiagnose> getDiagnosen() {
        return diagnosen;
    }

    public LinkedList<BDTValue<String>> getMedikamenteOhneRezept() {
        return medikamenteOhneRezept;
    }

    public LinkedList<BDTValue<String>> getMedikamenteAufRezept() {
        return medikamenteAufRezept;
    }

    public LinkedList<BDTValue<String>> getMedikamentAerztemuster() {
        return medikamentAerztemuster;
    }

    public LinkedList<BDTBefund> getBefunde() {
        return befunde;
    }

    public LinkedList<BDTValue<String>> getTherapie() {
        return therapie;
    }

    public LinkedList<BDTValue<String>> getPhysikTherapie() {
        return physikTherapie;
    }

    public LinkedList<BDTValue<String>> getUeberweisungInhalt() {
        return ueberweisungInhalt;
    }

    public BDTArbeitsunfaehigkeit getArbeitsunfaehigkeit() {
        return arbeitsunfaehigkeit;
    }

    public LinkedList<BDTBefundArt> getBefundArten() {
        return befundArten;
    }

    public BDTKrankenhausEinweisung getKrankenhausEinweisungen() {
        return krankenhausEinweisungen;
    }

    public void addDiagnose(BDTDiagnose diagnose) {
        diagnosen.add(diagnose);
    }

    public void addDrugsOnRecipe(BDTValue<String> bdtValue) {
        medikamenteAufRezept.add(bdtValue);
    }

    public void addDrugsWithoutRecipe(BDTValue<String> bdtValue) {
        medikamenteOhneRezept.add(bdtValue);
    }

    public void addMedDocSample(BDTValue<String> bdtValue) {
        medikamentAerztemuster.add(bdtValue);
    }

    public void addBefund(BDTBefund bdtBefund) {
        befunde.add(bdtBefund);
    }


    public void addTherapie(BDTValue<String> bdtValue) {
        therapie.add(bdtValue);
    }

    public void addPhysikTherapie(BDTValue<String> bdtValue) {
        physikTherapie.add(bdtValue);
    }

    public void addUeberweisungs(BDTValue<String> bdtValue) {
        ueberweisungInhalt.add(bdtValue);
    }

    public void addArbeitsUnfaehigkeit(BDTArbeitsunfaehigkeit arbeitsunfaehigkeit) {
        this.arbeitsunfaehigkeit = arbeitsunfaehigkeit;
    }

    public void addKrankenhausEinweisung(BDTKrankenhausEinweisung krankenhausEinweisung) {
        this.krankenhausEinweisungen = krankenhausEinweisung;
    }

    public void addBDTBefund(BDTBefundArt befund) {
        this.befundArten.add(befund);
    }

    public void addGNR(BDTGNR bdtgnr) {
        gebuehrennummern.add(bdtgnr);
    }

    public BDTValue<String> getDateAsReferenceable() {
        return behandlungsTag;
    }
}
