package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.io.IOException;
import java.io.InputStream;
import java.time.DateTimeException;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datacarrier.Datacarrier;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datapackage.Datapackage;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldTable;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.IllegalSentenceStructureException;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.Sentence;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.SentenceCharacteristic;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.SentenceTable;
import de.uni_goettingen.med.mi.bdtparser.core.validator.Validator;

/**
 * A Class used to create a VPR for all patients in a given datacarrier. 
 * The VPRs are stored in a HashMap and are referenceable by the patient ID.
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTVPRCreator {
    private List<Datacarrier> datacarriers;
    public final static Logger logger = LogManager.getLogger(BDTVPRCreator.class);
    private LocalDate filterStartDate = null;
    private LocalDate filterEndDate = null;
    private BDTArztpraxis arztpraxis;
    private HashMap<Long, BDTPatient> patientHashMap;
    private Properties radarProperties;
    BDTMetadata metadata;
    private boolean viewerMode = false;

    public BDTVPRCreator() {
        loadRadarProperties();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd.MM.yyyy");
        LocalDate dateFrom = LocalDate.parse(radarProperties.getProperty("radar_filter_from"), formatter);
        LocalDate dateTo = LocalDate.parse(radarProperties.getProperty("radar_filter_to"), formatter);
        filterStartDate = dateFrom;
        filterEndDate = dateTo;
        metadata = new BDTMetadata();
    }

    private void loadRadarProperties() {
        radarProperties = new Properties();
        InputStream radar = this.getClass().getResourceAsStream("/RADAR.properties");
        try {
            radarProperties.load(radar);
            radar.close();
        } catch (IOException ioe) {
            logger.error(ioe.getMessage());
            try {
                radar.close();
            } catch (IOException ioeClose) {
                logger.error("could not close the property reader");
            }

        }
    }

    /**
     * Creates a HashMap with the VPR for all Patienten from the given List of Datacarrier. <br/>
     * Also creates the Arzpraxis. <br/>
     * the datacarriers should only contain one sentence Praxisdaten {@link SentenceCharacteristic#Praxisdaten}.
     * 
     * @param datapackage the datapackage that should be used to create the patientens
     * @return a Hashmap containing the Patients, referenceable by their ID
     */
    public BDTArztpraxis createPatients(Datapackage datapackage) {
        if (datapackage == null || datapackage.getDatacarriers() == null)
            throw new IllegalArgumentException("datacarrier and datapackage must not be null");
        this.datacarriers = datapackage.getDatacarriers();
        patientHashMap = new HashMap<>();
        for (Datacarrier datacarrier : datacarriers) {
            for (Sentence sentence : datacarrier.getSentences()) {
                try {
                    SentenceCharacteristic sentenceCharacteristic = SentenceTable.getCharateristic(Integer.parseInt(sentence.getSentenceType()));
                    if (sentenceCharacteristic == null)
                        continue;
                    switch (sentenceCharacteristic) {
                    case Praxisdaten:
                        addPraxisdaten(sentence);
                        break;
                    case Patientenstamm:
                        addPatientenstanmm(sentence);
                        break;
                    case Behandlungsdaten:
                        addBehandlungsdaten(sentence);
                        break;
                    case AerztlicheBehandlung:
                        addAerztlicheBehandlung(sentence);
                        break;
                    case Datenpaket_Header:
                        addDatenpaketHeader(sentence);
                        break;
                    case Datentraeger_Header:
                        addDatenTraegerHeader(sentence);
                        break;
                    default:
                        break;
                    }
                } catch (IllegalSentenceStructureException ilsse) {
                    logger.error("something went wrong: Exception caught: " + ilsse.getClass());
                }
            }
        }
        if(!viewerMode) {
            filterPatientHashMap(patientHashMap);
        }
        arztpraxis.setPatients(patientHashMap);
        return arztpraxis;
    }
    
    private void filterPatientHashMap(HashMap<Long, BDTPatient> patientHashmap) {
        removePatientsWhichAreNotInTimePeriod(patientHashMap);
        removeTreatmentsWhichAreNotInTimePeriod(patientHashMap);
    }

    /**
     * Methode to remove all patients from the HashMap, which didn't visit the doctor in the interval defined by startDateFilter and endDateFilter
     * 
     * @param patientHashMap HashMap which stores the Patients
     */
    private void removePatientsWhichAreNotInTimePeriod(HashMap<Long, BDTPatient> patientHashMap) {
        Iterator<Long> iterator = patientHashMap.keySet().iterator();
        while (iterator.hasNext()) {
            BDTPatient patient = patientHashMap.get(iterator.next());
            boolean treatmentInTimePeriod = false;
            for (BDTCase bdtCase : patient.getBDTCases()) {
                if (treatmentInTimePeriod) {
                    break;
                }
                for (LocalDate treatmentDay : bdtCase.getTreamentDates()) {
                    if (treatmentDay.isAfter(filterStartDate) && treatmentDay.isBefore(filterEndDate)) {
                        treatmentInTimePeriod = true;
                        break;
                    }
                }
            }
            if (!treatmentInTimePeriod) {
                iterator.remove();
                String patienNumber = "";
                if (patient.getPatientNummer() != null) {
                    patienNumber = patient.getPatientNummer().getValue();
                }
                logger.info("Removed patient " + patienNumber + " which didn't visit the doctor in the interval defined by startDateFilter and endDateFilter");
            }
        }
    }

    /**
     * Methode to remove all treatment from the HashMap, which weren't in the interval defined by startDateFilter and endDateFilter
     * 
     * @param patientHashMap HashMap which stores the Patients
     */
    private void removeTreatmentsWhichAreNotInTimePeriod(HashMap<Long, BDTPatient> patientHashMap) {
        Iterator<Long> iterator = patientHashMap.keySet().iterator();
        while (iterator.hasNext()) {
            BDTPatient patient = patientHashMap.get(iterator.next());
            if (patient.getPatientNummerKey() == 88597) {
                System.out.println("found");
            }
            List<BDTCase> lstCases = patient.getBDTCases();
            Iterator<BDTCase> iteratorBDTCase = lstCases.iterator();
            while (iteratorBDTCase.hasNext()) {
                BDTCase bdtCase = iteratorBDTCase.next();
                List<BDTTreatment> lstTreatments = bdtCase.getTreatments();
                Iterator<BDTTreatment> iteratorBDTTeatment = lstTreatments.iterator();
                while (iteratorBDTTeatment.hasNext()) {
                    BDTTreatment treatment = iteratorBDTTeatment.next();
                    LocalDate treatmentDay = treatment.getDate();
                    if (treatmentDay.isBefore(filterStartDate) || treatmentDay.isAfter(filterEndDate)) {
                        iteratorBDTTeatment.remove();
                    }
                }
                if (lstTreatments.size() == 0) {
                    iteratorBDTCase.remove();
                }
            }
            if (lstCases.size() == 0) {
                iterator.remove();
            }
        }
    }

    private void addDatenTraegerHeader(Sentence sentence) {
        for (Field field : sentence.getFields()) {
            FieldCharacteristic fieldCharacteristic = FieldTable.getCharateristic(Integer.parseInt(field.getNumber()));
            BDTValue<String> value = new BDTValue<>(field.getData(), field);
            switch (fieldCharacteristic) {
            case DatumDerErstellung:
                if (metadata.getDatumDerErstellung() == null)
                    metadata.setDatumDerErstellung(value);
                break;
            case Zeichencode:
                if (metadata.getZeichencode() == null)
                    metadata.setZeichencode(value);
                break;
            default:
                break;
            }
        }
    }

    private void addDatenpaketHeader(Sentence sentence) {
        for (Field field : sentence.getFields()) {
            FieldCharacteristic fieldCharacteristic = FieldTable.getCharateristic(Integer.parseInt(field.getNumber()));
            BDTValue<String> value = new BDTValue<>(field.getData(), field);

            switch (fieldCharacteristic) {
            case VersionADT_Satzbeschreibung:
                metadata.setVersionADT(value);
                break;
            case VersionBDT:
                metadata.setVersionBDT(value);
                break;
            case Archivierungsart:
                metadata.setArchivierungsart(value);
                break;
            case ZeitraumDerSpeicherung:
                metadata.setZeitraumDerSpeicherung(value);
                break;
            case BeginnDerUebertragung:
                metadata.setBeginnDerÜbertragung(value);
                break;
            default:
                break;

            }
        }
    }

    /**
     * reads each field from an AerztlicheBehandlung sentence and adds it to a case from patient
     * 
     * @param sentence the sentence to read from
     */
    private void  addAerztlicheBehandlung(Sentence sentence) {
        BDTPatient patient;
        Field treatmentDayField;
        LocalDate treatmentDay;
        BDTCase bdtCase;
        BDTTreatment treatment;
		try {
			patient = getPatientFromSentence(sentence);
			treatmentDayField = sentence.getField(FieldCharacteristic.Leistungstag.getNumber());
			if (treatmentDayField == null) {
				throw new DateTimeException("TreatmentDayField is missing.");
			}
			if(treatmentDayField.getData().equals("05042018"))
			    logger.debug("field found");
			treatmentDay = Validator.createDateFromField(treatmentDayField);
			bdtCase = patient.getCaseByDate(treatmentDay);
			if (bdtCase == null) {
				bdtCase = new BDTCase(treatmentDay, new BDTValue<>(treatmentDayField.getData(), treatmentDayField));
				patient.addCase(bdtCase);
			}
			bdtCase.setType(BDTCase.CaseTypes.aerztlicheBehandlung);
			treatment = bdtCase.getTreatmentsByDate(treatmentDay);
		} catch (DateTimeException | NumberFormatException dtife) {
			logger.error("An error occurred while trying to add a new BDTCase. Couldn't access date or or case. " + dtife.getMessage() + " Skipping the sentence");
			return;
		}
        for (Field field : sentence.getFields()) {
            if (!field.isValid())
                continue;
            try {
                FieldCharacteristic fieldCharacteristic = FieldTable.getCharateristic(Integer.parseInt(field.getNumber()));
                switch (fieldCharacteristic) {
                case QuartalDerAbrechnung:
                    bdtCase.setQuartal(new BDTValue<>(field.getData(), field));
                    break;
                case Geschaeftsstelle:
                    bdtCase.setGeschaeftsstelle(new BDTValue<>(field.getData(), field));
                    break;
                case AbrechnungsartSchein:
                    // logger.debug("Abrechnungsschein Data field: " + field.getData());
                    bdtCase.setAbrechnungsArt(new BDTValue<>(field.getData(), field));
                    // logger.debug("Abrechnungsschein Data case: " + bdtCase.getAbrechnungsArt());
                    break;
                case Gebuehrenordnung:
                    bdtCase.setGebuehrenOrdnung(new BDTValue<>(field.getData(), field));
                    break;
                case Leistungstag:
                    treatmentDay = Validator.createDateFromField(field);
                    treatment = bdtCase.getTreatmentsByDate(treatmentDay);
                    if (treatment == null) {
                        bdtCase.addTreatment(Validator.createDateFromField(field), new BDTValue<>(field.getData(), field));
                        treatment = bdtCase.getTreatmentsByDate(treatmentDay);
                    } else
                        treatment.getDateAsReferenceable().addReference(field);
                    break;
                case GNR_GNR_Ident:
                    treatment.addGNR(new BDTGNR(field.getData(), field));
                    break;
                case Abrechnungsdiagnose:
                    treatment.addDiagnose(new BDTDiagnose(field.getData(), field, false, true));
                    break;
                case ICD_Schluessel:
                    treatment.addDiagnose(new BDTDiagnose(field.getData(), field, true, false));
                    break;
                default:
                    break;
                }
            } catch (Exception e) {
                logger.error("An exception (" + e.getClass() + ") occurred while trying to add data from field " + field.getNumber() + " to a case");
            }
        }
    }

    /**
     * reads each field from an Arztpraxis sentence and adds to the BDTArztpraxis class
     * 
     * @param sentence the sentence to read from
     */
    private void addPraxisdaten(Sentence sentence) {
        arztpraxis = new BDTArztpraxis();
        for (Field field : sentence.getFields()) {
            if (field.isValid()) {
                FieldCharacteristic fieldCharacteristic = FieldTable.getCharateristic(Integer.parseInt(field.getNumber()));
                if (fieldCharacteristic == null) {
                    logger.error("An error occurred while adding praxis data to the vpr. Wasn't able to find a field characteristic for a valid field");
                    continue;
                }
                switch (fieldCharacteristic) {
                case KBV_Pruefnummer:
                    arztpraxis.setKBV_Puefnummer(new BDTValue<>(field.getData(), field));
                    break;
                case Softwareverantwortlicher:
                    arztpraxis.setSoftwareverantworlicher(new BDTValue<>(field.getData(), field));
                    break;
                case Software:
                    arztpraxis.setSoftware(new BDTValue<>(field.getData(), field));
                    break;
                case Hardware:
                    arztpraxis.setHardware(new BDTValue<>(field.getData(), field));
                    break;
                case Arztnummer:
                    arztpraxis.setArztNummer(new BDTValue<>(Integer.parseInt(field.getData()), field));
                    break;
                case Praxistyp:
                    arztpraxis.setTyp(new BDTValue<>(Integer.parseInt(field.getData()), field));
                    break;
                case Arztname:
                    arztpraxis.addArzt(new BDTArzt(new BDTValue<>(field.getData(), field)));
                    break;
                case ArztgruppeVerbal:
                    if (arztpraxis.getAerzte() != null && arztpraxis.getAerzte().size() > 0) {
                        arztpraxis.getAerzte().getLast().setArztgruppe(new BDTValue<>(field.getData(), field));
                    }
                    break;
                case Strasse_Praxis:
                    arztpraxis.setStrasse(new BDTValue<>(field.getData(), field));
                    break;
                case PLZOrt:
                    arztpraxis.setOrtPLZ(new BDTValue<>(field.getData(), field));
                    break;
                case Telefonnummer:
                    arztpraxis.setTelefon(new BDTValue<>(field.getData(), field));
                    break;
                case Telefaxnummer:
                    arztpraxis.setTelefax(new BDTValue<>(field.getData(), field));
                    break;
                case Modemnummer:
                    arztpraxis.setModemnummer(new BDTValue<>(field.getData(), field));
                    break;
                case AnzahlAerzte:
                    arztpraxis.setAnzahlAerzte(new BDTValue<>(Integer.parseInt(field.getData()), field));
                    break;
                default:
                    // logger.debug("not supported fieldcharacteristic");
                }
            }
        }
    }

    /**
     * reads each field from an Behandlungsdaten sentence and adds it to a treatment in a case from a patient
     * 
     * @param sentence the sentence to read from
     */
    private void addBehandlungsdaten(Sentence sentence) {
        BDTPatient patient = null;
        Field treatmentDayField = null;
        LocalDate treatmentDay;
        BDTCase bdtCase = null;
        BDTTreatment treatment = null;
        BDTArbeitsunfaehigkeit arbeitsunfaehigkeit = null;
        BDTKrankenhausEinweisung krankenhausEinweisung = null;
        BDTTest test = null;
        BDTBefundArt befundart = null;
        try {
            patient = getPatientFromSentence(sentence);
            treatmentDayField = sentence.getField(FieldCharacteristic.TagDerSpeicherungVonBehandlungsdaten.getNumber());
            if (treatmentDayField != null) {
                treatmentDay = Validator.createDateFromField(treatmentDayField);
                bdtCase = patient.getCaseByDate(treatmentDay);
                if (bdtCase == null) {
                    bdtCase = new BDTCase(treatmentDay, new BDTValue<>(treatmentDayField.getData(), treatmentDayField));
                    patient.addCase(bdtCase);
                }
                treatment = bdtCase.getTreatmentsByDate(treatmentDay); 
            } else {
            	logger.error("An error occurred while trying to add a new BDTCase. Couldn't access date or or case. TreatmentDayField is missing. Skipping the sentence");
            }
        } catch (Exception e) {
            logger.error("An exception occurred while trying to accessing bdt treatment" + e.getMessage());
            logger.error(e.getClass());
            if (e instanceof IllegalArgumentException)
                return;
        }
        for (Field field : sentence.getFields()) {
            if (field.isValid()) {
                try {
                    FieldCharacteristic fieldCharacteristic = FieldTable.getCharateristic(Integer.parseInt(field.getNumber()));
                    switch (fieldCharacteristic) {
                    case TagDerSpeicherungVonBehandlungsdaten:
                        treatmentDay = Validator.createDateFromField(field);
                        treatment = bdtCase.getTreatmentsByDate(treatmentDay);
                        if (treatment == null) {// if null, treatment does not exist create new treatment
                            bdtCase = patient.getCaseByDate(treatmentDay);
                            if (bdtCase == null) {
                                bdtCase = new BDTCase(treatmentDay, new BDTValue<>(field.getData(), field));
                                patient.addCase(bdtCase);
                            } else
                                bdtCase.addTreatment(treatmentDay, new BDTValue<>(field.getData(), field));
                            treatment = bdtCase.getTreatmentsByDate(treatmentDay);
                        } else// treatment exists add reference
                            treatment.getDateAsReferenceable().addReference(field);
                        break;
                    case AktuelleDiagnose:
                        treatment.addDiagnose(new BDTDiagnose(field.getData(), field, false, false));
                        break;
                    case MedikamentVerordnetAufRezept:
                        treatment.addDrugsOnRecipe(new BDTValue<>(field.getData(), field));
                        break;
                    case AusserhalbRezeptVerordnetesMedikament:
                        treatment.addDrugsWithoutRecipe(new BDTValue<>(field.getData(), field));
                        break;
                    case Aerztemuster:
                        treatment.addMedDocSample(new BDTValue<>(field.getData(), field));
                        break;
                    case Befund:
                        // treatment.addBefund(new BDTValue<>(field.getData(),field));
                        treatment.addBefund(new BDTBefund(field.getData(), field, BDTBefund.BefundType.normal));
                        break;
                    case Befund_Behandlungsdaten:
                        // treatment.addBefund(new BDTValue<>(field.getData(),field));
                        treatment.addBefund(new BDTBefund(field.getData(), field, BDTBefund.BefundType.normal));
                        break;
                    case Fremdbefund:
                        // treatment.addfremdBefund(new BDTValue<>(field.getData(),field));
                        treatment.addBefund(new BDTBefund(field.getData(), field, BDTBefund.BefundType.fremdBefund));
                        break;
                    case Laborbefund:
                        // treatment.addLaborBefund(new BDTValue<>(field.getData(),field));
                        treatment.addBefund(new BDTBefund(field.getData(), field, BDTBefund.BefundType.laborBefund));
                        break;
                    case Roentgenbefund:
                        // treatment.addRoentgenBefund(new BDTValue<>(field.getData(),field));
                        treatment.addBefund(new BDTBefund(field.getData(), field, BDTBefund.BefundType.roentgenBefund));
                        break;
                    case Therapie:
                        treatment.addTherapie(new BDTValue<>(field.getData(), field));
                        break;
                    case PhysikalischeTherapie:
                        treatment.addPhysikTherapie(new BDTValue<>(field.getData(), field));
                        break;
                    case UeberweisungInhalt:
                        treatment.addUeberweisungs(new BDTValue<>(field.getData(), field));
                        break;
                    case AUDauer:
                        arbeitsunfaehigkeit = new BDTArbeitsunfaehigkeit(field.getData(), field);
                        treatment.addArbeitsUnfaehigkeit(arbeitsunfaehigkeit);
                        break;
                    case AUWegen:
                        arbeitsunfaehigkeit.auWegen.add(new BDTValue<>(field.getData(), field));
                        break;
                    case Krankenhauseinweisung_Krankenhaus:
                        krankenhausEinweisung = new BDTKrankenhausEinweisung(field.getData(), field);
                        treatment.addKrankenhausEinweisung(krankenhausEinweisung);
                        break;
                    case KrankenhauseinweisungWegen:
                        if (krankenhausEinweisung == null) {
                            krankenhausEinweisung = new BDTKrankenhausEinweisung(null, null);
                            treatment.addKrankenhausEinweisung(krankenhausEinweisung);
                        }
                        krankenhausEinweisung.einweisungWegen.add(new BDTValue<>(field.getData(), field));
                        break;
                    case Befundart:
                        befundart = new BDTBefundArt(field.getData(), field);
                        treatment.addBDTBefund(befundart);
                        break;
                    case Test_Ident:
                        test = new BDTTest(field.getData(), field);
                        befundart.tests.add(test);
                        break;
					case Testbezeichnung:
						if (test != null) {
							test.testBezeichnung = new BDTValue<>(field.getData(), field);
						} else {
							logger.error("An error occurred while trying to add a new BDTTest (Field " + field.getNumber() + ") Test-Ident missing");
						}
						break;
					case Ergebnis_Wert:
						if (test != null) {
							test.ergebnis = new BDTValue<>(field.getData(), field);
						} else {
							logger.error("An error occurred while trying to add a new BDTTest (Field " + field.getNumber() + ") Test-Ident missing");
						}
						break;
					case Einheit:
						if (test != null) {
							test.einheit = new BDTValue<>(field.getData(), field);
						} else {
							logger.error("An error occurred while trying to add a new BDTTest (Field " + field.getNumber() + ") Test-Ident missing");
						}
						break;
                    default:
                        // logger.debug("not supported fieldcharacteristic");
                    }
                } catch (Exception e) {
                    logger.error("An Exception occurred while trying to add treatment data from field " + field.getNumber() + ". Exception type: + " + e.getClass());
                }
            }
        }

    }

    /**
     * reads each field from an Patientenstamm sentence and adds it to the patient data
     * 
     * @param sentence the sentence to read from
     */
    private void addPatientenstanmm(Sentence sentence) {
        BDTPatient patient = null;
        for (Field field : sentence.getFields()) {
            FieldCharacteristic fieldCharacteristic = null;
            if (field.isValid()) {
                try {
                    fieldCharacteristic = FieldTable.getCharateristic(Integer.parseInt(field.getNumber()));
                    switch (fieldCharacteristic) {
                    case Patientennummer_Patientenkennung:
                        patient = getPatientFromSentence(sentence);
                        break;
                    case NameDesPatienten:
                        patient.setNachName(new BDTValue<>(field.getData(), field));
                        break;
                    case VornameDesPatienten:
                        patient.setVorName(new BDTValue<>(field.getData(), field));
                        break;
                    case GeburtsdatumDesPatienten:
                        patient.setGeburtsDatum(new BDTValue<>(field.getData(), field));
                        break;
                    case GeschlechtDesPatienten:
                        patient.setGeschlecht(new BDTValue<>(field.getData(), field));
                        break;
                    case WohnortDesPatienten:
                        patient.getAdresse().setOrt(new BDTValue<>(field.getData(), field));
                        break;
                    case StrasseDesPatienten:
                        patient.getAdresse().setStrasse(new BDTValue<>(field.getData(), field));
                        break;
                    case XdtPLZDesPatienten: // to support TURBOMED
                        patient.getAdresse().setZipCode(new BDTValue<>(field.getData(), field));
                        break;
                    case XdtOrtDesPatienten: // to support TURBOMED
                        patient.getAdresse().setOrt(new BDTValue<>(field.getData(), field));
                        break;    
                    case DauerdiagnosenAbDatum:
                        patient.getDauerDiagnosen().add(new BDTDauerdiagnose(field.getData(), field));
                        break;
                    case Dauerdiagnosen:
                        patient.getDauerDiagnosen().getLast().diagnosen.add(new BDTValue<>(field.getData(), field));
                        break;
                    case DauermedikamenteAbDatum:
                        patient.getDauerMedikamentation().add(new BDTDauermedikamentation(field.getData(), field));
                        break;
                    case Dauermedikamente:
                        patient.getDauerMedikamentation().getLast().medikamente.add(new BDTValue<>(field.getData(), field));
                        break;
                    case Allergien:
                        patient.addAllergie(new BDTValue<>(field.getData(), field));
                        break;
                    default:
                        // logger.debug("not supported fieldcharacteristic");
                    }
                } catch (Exception e) {
                    logger.error("an exception occured while trying to add patient data from field " + field.getData());
                    if (fieldCharacteristic != null && fieldCharacteristic.equals(FieldCharacteristic.Patientennummer_Patientenkennung))
                        return;
                }
            }
        }

    }

    /**
     * getter for the BDTArztpraxis
     */
    public BDTArztpraxis getArztpraxis() {
        return arztpraxis;
    }

    /**
     * Methode to get the BDTPatient from the sentence by extracting the patientnumber and look it up in the hashmap
     * 
     * @param sentence the sentence from witch the
     * @throws IllegalArgumentException if the the sentence is null, or the field stored in the sentence with patientnumber is invalid or null
     * @return the BDTPatienten as non null element
     */
    private BDTPatient getPatientFromSentence(Sentence sentence) {
        if (sentence == null)
            throw new IllegalArgumentException("Sentence must not be null");
        BDTPatient patient;
        Field patientnumber = sentence.getField(FieldCharacteristic.Patientennummer_Patientenkennung.getNumber());
        String strPatientno = patientnumber.getData();
        strPatientno = strPatientno.replaceAll("temp_", ""); // walkaround to enable bdt-viewer
        strPatientno = strPatientno.replaceAll("db_", "");  // to parse pseudonymised bdt files
        patientnumber.setData(strPatientno);
        if (patientnumber == null || !patientnumber.isValid()) { // is null if no field found
            logger.error("Error parsing patientnumber " + patientnumber);
            throw new IllegalArgumentException("cant access patientnumber");
        }
        Long patientno = null;
        try {
            patientno = Long.parseLong(patientnumber.getData());
        } catch (NumberFormatException nfe) {
            logger.error("Error parsing patientnumber " + patientnumber + " " + nfe.getLocalizedMessage());
        }
        patient = patientHashMap.get(patientno);
        if (patient == null) { // or throw exception
            patient = new BDTPatient(new BDTValue<>(patientnumber.getData(), patientnumber));
            patientHashMap.put(Long.parseLong(patientnumber.getData()), patient);
        } else {
            patient.addPatientNumberReference(patientnumber);
        }

        return patient;
    }

    public LocalDate getStartDateFilter() {
        return filterStartDate;
    }

    public void setStartDateFilter(LocalDate startDateFilter) {
        this.filterStartDate = startDateFilter;
    }

    public LocalDate getEndDateFilter() {
        return filterEndDate;
    }

    public void setEndDateFilter(LocalDate endDateFilter) {
        this.filterEndDate = endDateFilter;
    }

    public BDTMetadata getMetadata() {
        return metadata;
    }

    public void activateViewerMode() {
        viewerMode = true;
    }
}
