package de.uni_goettingen.med.mi.radar_software;

import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTMetadata;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.bdtparser.core.BDTMarshaller;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTArztpraxis;
import de.uni_goettingen.med.mi.radar_software.model.InformedConsent;
import de.uni_goettingen.med.mi.radar_software.model.InformedConsentWrapper;
import de.uni_goettingen.med.mi.radar_software.view.MainLayoutController;
import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;

/**
 * Prototypical implementation of a BDT-file processing software as part of the RADAR project.
 * RADAR - Routine Anonymized Data for Advanced Health Services Research
 * Project description: https://gepris.dfg.de/gepris/projekt/315084924
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class MainApp extends Application {

    private static final Logger log = LogManager.getLogger(MainApp.class);
    public BDTMetadata BDTMetaData;
    private ObservableList<InformedConsent> lstInformedConsent = FXCollections.observableArrayList();
    private String informedConsentFilePath = "informedConsent.xml";
    private Stage primaryStage;
    private BDTArztpraxis bdtData;
    private BDTMarshaller bdtMarshaller;
    private LocalDateTime startTime;
    private String USBPath;
	private boolean isPseudonymisationDone;

	public static void main(String[] args) throws Exception {
        launch(args);
    }

    @Override
    public void start(Stage stage) throws Exception {
        log.info("Starting RADAR application");
        this.primaryStage = stage;
        initRootLayout();
    }

    @Override
    public void stop() {
        saveInformedConsentDataToFile(new File(informedConsentFilePath));
        log.info("Stopping RADAR application");
    }

    /**
     * Initializes the main layout.
     */
    public void initRootLayout() {
        String fxmlFile = "/fxml/0_MainLayout.fxml";
        try {
            // Load main layout from fxml file.
            FXMLLoader loader = new FXMLLoader();
            Parent rootNode = (Parent) loader.load(getClass().getResourceAsStream(fxmlFile));

            // Show the scene containing the main layout.
            Scene scene = new Scene(rootNode, 1000, 600);
            primaryStage.setScene(scene);
            // fix window size
            primaryStage.setResizable(false);

            // Give the controller access to the main app.
            MainLayoutController controller = loader.getController();
            controller.setMainApp(this);
            controller.setHostServices(getHostServices());
            controller.initializeFirstView();

            primaryStage.show();
        } catch (IOException ioe) {
            log.error("Error loading fxmlFile " + fxmlFile + " " + ioe.getMessage());
        }

        // Load previously saved consents
        File file = getInformedConsentFile();
        if (file != null) {
            loadInformedConsentDataFromFile(file);
        }
    }

    /**
     * Loads the file with the saved informed consents from the stored path.
     *
     * @return the file object with the saved informed consents
     */
    private File getInformedConsentFile() {
        return new File(informedConsentFilePath);
    }

    /**
     * Loads the xml-file with the saved informed consents and unmarshalles it into
     * lstInformedConsent list.
     *
     * @param file object with the saved informed consents
     */
    private void loadInformedConsentDataFromFile(File file) {
        try {
            JAXBContext context = JAXBContext.newInstance(InformedConsentWrapper.class);
            Unmarshaller unmarshaller = context.createUnmarshaller();
            InformedConsentWrapper wrapper = (InformedConsentWrapper) unmarshaller.unmarshal(file);
            lstInformedConsent.clear();
            if (wrapper.getInformedConsent() != null) {
                lstInformedConsent.addAll(wrapper.getInformedConsent());
            }
        } catch (JAXBException jaxbe) {
            log.error("Error loading consent data from file: " + file.getAbsolutePath() + " " + jaxbe.getMessage());
        }
    }

    /**
     * Marshalles the lstInformedConsent list and saves it into an xml-file.
     *
     * @param file object to saved the informed consents
     */
    public void saveInformedConsentDataToFile(File file) {
        try {
            JAXBContext context = JAXBContext.newInstance(InformedConsentWrapper.class);
            Marshaller marshaller = context.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
            InformedConsentWrapper wrapper = new InformedConsentWrapper();
            wrapper.setInformedConsent(lstInformedConsent);
            marshaller.marshal(wrapper, file);
        } catch (JAXBException jaxbe) {
            log.error("Error storing consent data to file: " + file.getAbsolutePath() + " " + jaxbe.getMessage());
        }
    }

    /**
     * Access to the list of informed consents from the informedConsent (save, edit, delete consents)
     * and pseudonymization controller (filter for consented patients).
     *
     * @return the list with informed consents
     */
    public ObservableList<InformedConsent> getInformedConsentData() {
        return lstInformedConsent;
    }

    /**
     * Access to the primary stage to allow popup windows modal access.
     *
     * @return the primary stage
     */
    public Stage getPrimaryStage() {
        return primaryStage;
    }

    /**
     * Sets the processed and validated virtual bdt-data from
     * virtual patient record creation task.
     *
     * @param bdtData the bdt-data
     */
    public void setBdtData(BDTArztpraxis bdtData) {
        this.bdtData = bdtData;
    }

    /**
     * Get the bdt-data for anonymization or pseudonymization.
     *
     * @return the bdt-data object
     */
    public BDTArztpraxis getBdtData() {
        return bdtData;
    }

    /**
     * Sets the bdtMarshaller from
     * virtual patient record creation task.
     *
     * @param bdtMarshaller the bdtMarshaller object
     */
    public void setBdtMarshaller(BDTMarshaller bdtMarshaller) {
        this.bdtMarshaller = bdtMarshaller;
    }

    /**
     * Get the bdtMarshaller to get access to the
     * read data structure.
     *
     * @return the bdtMarshaller object
     */
    public BDTMarshaller getBdtMarshaller() {
        return bdtMarshaller;
    }

    public BDTMetadata getBDTMetaData() {
        return BDTMetaData;
    }

    public void setBDTMetaData(BDTMetadata BDTMetaData) {
        this.BDTMetaData = BDTMetaData;
    }

    public LocalDateTime getStartTime() {
        return startTime;
    }

    public void setStartTime(LocalDateTime startTime) {
        this.startTime = startTime;
    }

	public String getUSBPath() {
		if (USBPath == null)
			throw new NullPointerException("USB Path is null!");
		return USBPath;
	}

	public void setUSBPath(String absolutePath) {
		this.USBPath = absolutePath;
	}

	public void setPseudonymisationDone(boolean isPseudonymisationDone) {
		this.isPseudonymisationDone = isPseudonymisationDone;
	}
	
    public boolean isPseudonymisationDone() {
		return isPseudonymisationDone;
	}
	
}