package de.uni_goettingen.med.mi.radar_software.gwdg;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.SftpException;

/**
 * Helper class for the data upload process
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */

public class TransferHelper {

	private static final Logger logger = LogManager.getLogger(TransferHelper.class);
	private String errorText = "";
	private String ftp_serverAddress;
	private String ftp_password;
	private String ftp_user;
	private int ftp_port;
	private String filePath;

	/**
	 * Method to test if the GWDG server is reachable.
	 * @return an error code
	 */
	public String testConnection() {
		logger.debug("testConnection " + ftp_serverAddress);
		try {
			InetAddress gwdgFtpServer = InetAddress.getByName(ftp_serverAddress);
			if (!gwdgFtpServer.isReachable(10000)) {
				errorText += "Could not connect to " + ftp_serverAddress;
			}
		} catch (UnknownHostException uhe) {
			errorText += uhe.getLocalizedMessage();
			logger.error(uhe.getMessage());
		} catch (IOException ioe) {
			logger.error(ioe.getMessage());
		}
		return errorText;
	}

	/**
	 * Method to test if it is possible to login onto the GWDG Server.
	 * @return an error code
	 */
	public String testLogin() {
		JSch jsch = new JSch();
		Session session = null;
		try {
			session = jsch.getSession(ftp_user, ftp_serverAddress, ftp_port);
			session.setPassword(ftp_password);
			session.setConfig("StrictHostKeyChecking", "no");
			logger.debug("Establishing Connection...");
			session.connect();
			logger.debug("Connection established.");
		} catch (JSchException jsche) {
			logger.error(jsche.getMessage());
			errorText += jsche.getMessage();
		} finally {
			session.disconnect();
			logger.debug("login test successful, closing connection.");
		}
		return errorText;
	}

	/**
	 * Method to transfer the anonymized data to the GWDG server.
	 * @return if the transfer was successful
	 */
	public boolean startTransfer() {
		boolean transferSuccessful = false;
		if (!this.filePath.isEmpty()) {
			logger.debug("Starting file upload " + this.filePath);
			File bdtUploadFile = new File(this.filePath);
			logger.debug("upload file size: " + bdtUploadFile.getTotalSpace());
			JSch jsch = new JSch();
			Session session = null;
			try {
				session = jsch.getSession(ftp_user, ftp_serverAddress, ftp_port);
				session.setPassword(ftp_password);
				session.setConfig("StrictHostKeyChecking", "no");
				logger.debug("Establishing Connection...");
				session.connect();
				logger.debug("Connection established.");
				logger.debug("Creating SFTP Channel.");
				ChannelSftp sftpChannel = null;
				sftpChannel = (ChannelSftp) session.openChannel("sftp");
				sftpChannel.connect();
				logger.debug("SFTP Channel created.");
				sftpChannel.cd("/files");
				sftpChannel.put(new FileInputStream(bdtUploadFile), bdtUploadFile.getName(), ChannelSftp.OVERWRITE);
				logger.debug("file uploaded");
				sftpChannel.exit();
				transferSuccessful = true;
			} catch (JSchException | FileNotFoundException | SftpException jsche) {
				logger.error(jsche.getMessage());
				errorText += jsche.getMessage();
			} finally {
				session.disconnect();
			}
		}
		return transferSuccessful;
	}

	public void setFtpServerAddress(String ftp_serverAddress) {
		this.ftp_serverAddress = ftp_serverAddress;
	}

	public void setFtpPort(int ftp_port) {
		this.ftp_port = ftp_port;
	}

	public void setFtp_user(String ftp_user) {
		this.ftp_user = ftp_user;
	}

	public void setFtp_password(String ftp_password) {
		this.ftp_password = ftp_password;
	}

	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}

}
