package de.uni_goettingen.med.mi.radar_software.model;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;

/**
 * Representation of a informed consent. Manages five consent options and holds a key to store
 * in the xml-file. The key is made up of the the localSystemPatientId, firstLetterFirstName and
 * firstLetterLastName so that it does not contain any identifying data. The reason for this is
 * that at no time identifying data should be stored on the USB stick.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class InformedConsent {

    private final StringProperty informedConsentKey;
    private final StringProperty localSystemPatientId;
    private final StringProperty firstLetterFirstName;
    private final StringProperty firstLetterLastName;
    private final BooleanProperty consentErhebungVerarbeitungNutzung;
    private final BooleanProperty consentErneuteKontaktaufnahme;
    private final BooleanProperty consentGeltungsdauer;
    private final BooleanProperty consentWiderrufsrecht;
    private final BooleanProperty tabletWorkflow;

    /**
     * Enum for the different section of the consent
     */
    public enum InformedConsentSection {
        Erhebung_Verarbeitung_Nutzung, Erneute_Kontaktaufnahme, Geltungsdauer, Widerrufsrecht
    }

    /**
     * Default Constructor for the informed consent
     * By default all consent is set to false.
     */
    public InformedConsent() {
        this(null, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE);
    }

    /**
     * Constructor for the informed consent
     * @param informedConsentKey the key used to link the consent to the patient
     * @param consent flag if the patient agreed to Erhebung_Verarbeitung_Nutzung
     * @param consent flag if the patient agreed to Erneute_Kontaktaufnahme
     * @param consent flag if the patient agreed to Geltungsdauer
     * @param consent flag if the patient agreed to Widerrufsrecht
     */
    public InformedConsent(String informedConsentKey, Boolean isConsentErhebung_Verarbeitung_Nutzung, Boolean isConsentErneute_Kontaktaufnahme, Boolean isConsentGeltungsdauer, Boolean isConsentWiderrufsrecht, Boolean isTabletWorkflow) {
        this.informedConsentKey = new SimpleStringProperty(informedConsentKey);
        this.localSystemPatientId = new SimpleStringProperty();
        this.firstLetterFirstName = new SimpleStringProperty();
        this.firstLetterLastName = new SimpleStringProperty();
        this.consentErhebungVerarbeitungNutzung = new SimpleBooleanProperty(isConsentErhebung_Verarbeitung_Nutzung);
        this.consentErneuteKontaktaufnahme = new SimpleBooleanProperty(isConsentErneute_Kontaktaufnahme);
        this.consentGeltungsdauer = new SimpleBooleanProperty(isConsentGeltungsdauer);
        this.consentWiderrufsrecht = new SimpleBooleanProperty(isConsentWiderrufsrecht);
        this.tabletWorkflow = new SimpleBooleanProperty(isTabletWorkflow);
    }

    /**
     * Method to add a Patient to the informed consent list
     * @param localSystemPatientId the patient id
     * @param firstName the patients first name
     * @param lastName  the patients last name
     */
    public void addInformedConsent(String localSystemPatientId, String firstName, String lastName) {
        this.localSystemPatientId.set(localSystemPatientId);
        this.firstLetterFirstName.set(getFirstLetter(firstName));
        this.firstLetterLastName.set(getFirstLetter(lastName));
        generateInformedConsentKey();
    }

    /**
     * private method to generate the key
     */
    private void generateInformedConsentKey() {
        StringBuilder sbInformedConsentKey = new StringBuilder();
        sbInformedConsentKey.append(firstLetterFirstName.get());
        sbInformedConsentKey.append(firstLetterLastName.get());
        sbInformedConsentKey.append(localSystemPatientId.get());
        this.informedConsentKey.set(sbInformedConsentKey.toString());
    }

    /**
     * Method to split up  the informed consent key in the parts of first Letter of the first name,
     * the first letter of the last name and the patient id
     * @param informedConsentKey
     */
    private void divideInformedConsentKey(String informedConsentKey) {
        final int beginIndexFirstName = 0;
        final int beginIndexLastName = 1;
        final int beginIndexLocalSystemPatientId = 2;
        if (informedConsentKey != null && !informedConsentKey.isEmpty()) {
            int endIndex = informedConsentKey.length();
            this.firstLetterFirstName.set(informedConsentKey.substring(beginIndexFirstName, beginIndexLastName));
            this.firstLetterLastName.set(informedConsentKey.substring(beginIndexLastName, beginIndexLocalSystemPatientId));
            this.localSystemPatientId.set(informedConsentKey.substring(beginIndexLocalSystemPatientId, endIndex));
        }
    }

    /**
     * Method to get the first character of a text as upper case
     * @param text the text  must not be null and empty
     * @return the first letter as uppercase
     */
    private String getFirstLetter(String text) {
        if (text != null && !text.isEmpty()) {
            return text.substring(0, 1).toUpperCase();
        }
        return "";
    }

    /**
     * Getter for for the Key Property
     * @return the key property
     */
    public StringProperty informedConsentKeyProperty() {
        return informedConsentKey;
    }

    /**
     * Getter for the Key
     * @return the key
     */
    public String getInformedConsentKey() {
        return informedConsentKey.get();
    }

    /**
     * Setter for the Key
     * @param informedConsentKey the key
     */
    public void setInformedConsentKey(String informedConsentKey) {
        this.informedConsentKey.set(informedConsentKey);
        divideInformedConsentKey(informedConsentKey);
    }

    /**
     * Getter for the local Patient ID property
     * @return the id property
     */
    public StringProperty getLocalSystemPatientIdProperty() {
        return localSystemPatientId;
    }

    /**
     * The getter for the local system patient  id
     * @return the patient id
     */
    public String getLocalSystemPatientId() {
        return localSystemPatientId.get();
    }

    /**
     * Getter for the first letter of the first name
     * @return the first letter of first name as upper case
     */
    public String getFirstLetterFirstName() {
        return firstLetterFirstName.get();
    }
   
    /**
     * The getter for the first letter of the last name
     * @return the first letter of the last name as upper case
     */
    public String getFirstLetterLastName() {
        return firstLetterLastName.get();
    }
    
    /**
     * Getter for the BooleanProperty if the patient has agreed to Erhebung_Verarbeitung_Nutzung
     * @return the BooleanProperty
     */
    public BooleanProperty consentErhebungVerarbeitungNutzungProperty() {
        return consentErhebungVerarbeitungNutzung;
    }
    
    /**
     * Getter for the flag if the patient has agreed to Erhebung_Verarbeitung_Nutzung
     * @return the flag
     */
    public boolean isConsentErhebungVerarbeitungNutzung() {
        return consentErhebungVerarbeitungNutzung.get();
    }
    
    /**
     * Setter if the patient agreed to Erhebung_Verarbeitung_Nutzung
     * @param isConsentErhebungVerarbeitungNutzung
     */
    public void setConsentErhebungVerarbeitungNutzung(boolean isConsentErhebungVerarbeitungNutzung) {
        this.consentErhebungVerarbeitungNutzung.set(isConsentErhebungVerarbeitungNutzung);
    }
    
    /**
     * Getter for the flag as String (ja for true, nein for false) if the patient has agreed to Erhebung_Verarbeitung_Nutzung
     * @return the string
     */
    public String getTextConsentErhebungVerarbeitungNutzung() {
        String result = "";
        if (consentErhebungVerarbeitungNutzung.getValue()) {
            result = "ja";
        } else {
            result = "nein";
        }
        return result;
    }
   
    /**
     * Getter for the BooleanProperty if the patient has agreed to Erneute_Kontaktaufnahme
     * @return the BooleanProperty
     */
    public BooleanProperty consentErneuteKontaktaufnahmeProperty() {
        return consentErneuteKontaktaufnahme;
    }
    
    /**
     * Getter for the flag if the patient has agreed to Erneute_Kontaktaufnahme
     * @return the flag
     */
    public boolean isConsentErneuteKontaktaufnahme() {
        return consentErneuteKontaktaufnahme.get();
    }
    
    /**
     * Setter if the patient agreed to Erneute_Kontaktaufnahme
     * @param isConsentErneute_Kontaktaufnahme
     */
    public void setConsentErneuteKontaktaufnahme(boolean isConsentErneuteKontaktaufnahme) {
        this.consentErneuteKontaktaufnahme.set(isConsentErneuteKontaktaufnahme);
    }
    
    /**
     * Getter for the flag as String (ja for true, nein for false) if the patient has agreed to Erneute_Kontaktaufnahme
     * @return the string
     */
    public String getTextConsentErneuteKontaktaufnahme() {
        String result = "";
        if (consentErneuteKontaktaufnahme.getValue()) {
            result = "ja";
        } else {
            result = "nein";
        }
        return result;
    }
        
    /**
     * Getter for the BooleanProperty if the patient has agreed to Geltungsdauer
     * @return the BooleanProperty
     */
    public BooleanProperty consentGeltungsdauerProperty() {
        return consentGeltungsdauer;
    }
    
    /**
     * Getter for the flag if the patient has agreed to Geltungsdauer
     * @return the flag
     */
    public boolean isConsentGeltungsdauer() {
        return consentGeltungsdauer.get();
    }
    
    /**
     * Setter if the patient agreed to Geltungsdauer
     * @param isConsentGeltungsdauer
     */
    public void setConsentGeltungsdauer(boolean isConsentGeltungsdauer) {
        this.consentGeltungsdauer.set(isConsentGeltungsdauer);
    }
    
    /**
     * Getter for the flag as String (ja for true, nein for false) if the patient has agreed to Geltungsdauer
     * @return the string
     */
    public String getTextConsentGeltungsdauer() {
        String result = "";
        if (consentGeltungsdauer.getValue()) {
            result = "ja";
        } else {
            result = "nein";
        }
        return result;
    }
    
    /**
     * Getter for the BooleanProperty if the patient has agreed to Widerrufsrecht
     * @return the BooleanProperty
     */
    public BooleanProperty consentWiderrufsrechtProperty() {
        return consentWiderrufsrecht;
    }
    
    /**
     * Getter for the flag if the patient has agreed to Widerrufsrecht
     * @return the flag
     */
    public boolean isConsentWiderrufsrecht() {
        return consentWiderrufsrecht.get();
    }
    
    /**
     * Setter if the patient agreed to Widerrufsrecht
     * @param isConsentWiderrufsrecht
     */
    public void setConsentWiderrufsrecht(boolean isConsentWiderrufsrecht) {
        this.consentWiderrufsrecht.set(isConsentWiderrufsrecht);
    }
    
    /**
     * Getter for the flag as String (ja for true, nein for false) if the patient has agreed to Widerrufsrecht
     * @return the string
     */
    public String getTextConsentWiderrufsrecht() {
        String result = "";
        if (consentWiderrufsrecht.getValue()) {
            result = "ja";
        } else {
            result = "nein";
        }
        return result;
    }

    /**
     * Getter for the flag tabletWorkflow
     * @return the flag
     */
    public boolean isTabletWorkflow() {
        return tabletWorkflow.get();
    }
    
    /**
     * Setter if TabletWorkflow
     * @param isTabletWorkflow
     */
    public void setTabletWorkflow(boolean isTabletWorkflow) {
        this.tabletWorkflow.set(isTabletWorkflow);
    }
        
}