package de.uni_goettingen.med.mi.radar_software.ths;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Base64;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLProtocolException;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.radar_software.MainApp;

/**
 * Helper class for the pseudonymisation process
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class THSHelper {

    private static final Logger logger = LogManager.getLogger(THSHelper.class);

    private String ths_url;
    private String ths_apikey;
    private String ssl_keystore;
    private String ssl_password;
    private SSLContext ssl_context;
    private String errorText;
    private String basicAuthName;
    private String basicAuthPassword;

    public void setURL(String url) {
        this.ths_url = url;
    }

    public void setApikey(String apikey) {
        this.ths_apikey = apikey;
    }

    public void setKeystore(String keystore) {
        this.ssl_keystore = keystore;
    }

    public void setCertPassword(String password) {
        this.ssl_password = password;
    }

    public void setBasicAuthName(String basicAuthName) {
        this.basicAuthName = basicAuthName;
    }

    public void setBasicAuthPassword(String basicAuthPassword) {
        this.basicAuthPassword = basicAuthPassword;
    }

    /**
     * init methid that is loading the credentials for the anonymization service
     */
    public void init() {
        errorText = "";
        InputStream keystore = MainApp.class.getResourceAsStream(ssl_keystore);
        if (keystore != null) {
            init(keystore);
        } else {
            errorText += "Fehler beim Einlesen des THS-Zertifikates";
        }
    }

    /**
     * Method to test the connection to the anonymization service (Treuhandstelle)
     * @param uri the uri to the service
     * @return an error code
     */
    public String testConnection(String uri) {
        logger.debug("testConnection " + uri);
        if (ssl_context != null) {
            HttpsURLConnection request;
            try {
                request = (HttpsURLConnection) new URL(uri).openConnection();
                request.setSSLSocketFactory(ssl_context.getSocketFactory());
                request.setRequestProperty("charset","utf-8");
                request.connect();
                InputStreamReader in = new InputStreamReader(request.getInputStream());
                BufferedReader br = null;
                StringBuilder sb = new StringBuilder();
                String line;
                try {
                    br = new BufferedReader(in);
                    while ((line = br.readLine()) != null) {
                        sb.append(line);
                    }
                } catch (IOException ioe) {
                    logger.error(ioe.getMessage());
                } finally {
                    if (br != null) {
                        try {
                            br.close();
                        } catch (IOException ioe) {
                            logger.error(ioe.getMessage());
                        }
                    }
                }
                if (!sb.toString().contains("You have nothing more to do.")) {
                    errorText += " Fehler bei der Authenfizierung mit der Treuhandstelle.";
                }
            } catch (IOException ioe) {
                logger.error("Could not connect to " + ioe.getMessage());
                errorText += "Could not connect to " + ioe.getMessage();
            }
        }
        return errorText;
    }

    /**
     * Initializes the the connection to the Keystore
     * @param in
     */
    public void init(InputStream in) {
        try {
            SSLContext custom = SSLContext.getInstance("TLSv1.2");
            KeyStore store = KeyStore.getInstance(KeyStore.getDefaultType());
            KeyManagerFactory key = KeyManagerFactory.getInstance("SunX509");

            TrustManager[] trustAllCerts = new TrustManager[] { new X509TrustManager() {
                public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                    return null;
                }

                public void checkClientTrusted(X509Certificate[] certs, String authType) {
                }

                public void checkServerTrusted(X509Certificate[] certs, String authType) {
                }

            } };

            store.load(in, ssl_password.toCharArray());
            key.init(store, ssl_password.toCharArray());
            custom.init(key.getKeyManagers(), trustAllCerts, new java.security.SecureRandom());

            ssl_context = custom;
        } catch (NoSuchAlgorithmException | KeyStoreException | IOException | CertificateException | UnrecoverableKeyException | KeyManagementException e) {
            logger.error("Could not create custom SSLContext. Fallback to default. " + e.getMessage());
            errorText += "Fehler beim Einlesen des Zertifikates";
        }
    }

    public HttpsURLConnection getConnection() throws IOException {
        return getConnection(this.ths_url);
    }

    public HttpsURLConnection getConnection(String uri) throws IOException {
        uri = uri.replaceAll("https://test.radar", "https://basic.test.radar"); // redirect
        logger.debug("getConnection(" + uri + ")");
        HttpsURLConnection request = getConnectionEx(uri, false);
        // this is a legacy implementation for hosts that dont support sni
        try {
            request.connect();
        } catch (SSLProtocolException ssle) {
            if (ssle.getMessage().endsWith("unrecognized_name")) {
                logger.warn("unrecognized name in tls handshake. disabling sni.");
                request = getConnectionEx(uri, true);
                request.connect();
            } else
                throw ssle;
        }

        return request;
    }

    private HttpsURLConnection getConnectionEx(String uri, boolean noverify) throws IOException {
        HttpsURLConnection request = (HttpsURLConnection) new URL(uri).openConnection();
        if (ssl_context != null)
            request.setSSLSocketFactory(ssl_context.getSocketFactory());

        // Disable hostname verification by replacing the default HostnameVerifier
        // Might chance in future, as I suspect wrong implementation
        // Does _PERSIST_ by whatsoever reasons
        if (noverify)
            request.setHostnameVerifier((s, sslSession) -> true);

        request.setRequestProperty("charset","utf-8");
        request.setConnectTimeout(10000);
        request.setReadTimeout(15000);
        request.setDoOutput(true);
        request.setRequestMethod("POST");
        request.setRequestProperty("apiKey", ths_apikey);
        String authString = basicAuthName + ":" + basicAuthPassword;
        byte[] authEncBytes = Base64.getEncoder().encode(authString.getBytes());
        String authStringEnc = new String(authEncBytes);
        request.setRequestProperty("Authorization", "Basic " + authStringEnc);
        request.setRequestProperty("Content-Type", "application/json");

        return request;
    }

}