package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import javax.net.ssl.HttpsURLConnection;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

import de.uni_goettingen.med.mi.radar_software.model.InformedConsent.InformedConsentSection;
import de.uni_goettingen.med.mi.radar_software.ths.THSHelper;

/**
 * THS RequestAddConsent
 * 
 * Code friendly adopted from DZHK colleagues (author Rottmann)
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class RequestAddConsent {

	public enum RequestProcessType {
		addConsent
	}
	
    public enum RequestProcessDomain {
        RADAR
    }

	RequestProcessType processType;
	RequestProcessDomain processDomain;
	private String consentTemplate;
	private String consentVersion;
	private String psnName;

	String tokenId;
	String url;
	List<THSPatient> thsPatients;

	public RequestAddConsent(GetTokenResponse tokenResponse) {
		this.tokenId = tokenResponse.getTokenId();
		this.url = tokenResponse.getUrl();
	}

	public String getTokenId() {
		return this.tokenId;
	}

	public String getUrl() {
		return this.url;
	}

	public void setConsentTemplate(String consentTemplate) {
		this.consentTemplate = consentTemplate;
	}

	public void setConsentVersion(String consentVersion) {
		this.consentVersion = consentVersion;
	}

	public void setPsnDomain(RequestProcessDomain processDomain) {
		this.processDomain = processDomain;
	}

	public void setPsnName(String psnName) {
		this.psnName = psnName;
	}

	public void setSourceType(RequestProcessType processType) {
		this.processType = processType;
	}

	public void setPatients(List<THSPatient> thsPatients) {
		this.thsPatients = thsPatients;
	}

	/**
	 *
	 * @return JSONObject following the structure defines in THS interface specification
	 */
	@SuppressWarnings("unchecked")
	private JSONObject toJSON() {
        JSONObject json = new JSONObject();
        JSONArray patients = new JSONArray();

        if (thsPatients != null) {
            for (THSPatient thsPatient : thsPatients) {

                JSONObject pat = new JSONObject();

                // patient
                pat.put("index", Integer.toString(thsPatient.getIndex()));
                JSONObject patient = new JSONObject();
                JSONArray identifiers = new JSONArray();
                JSONObject identifier = new JSONObject();
                identifiers.add(identifier);
                identifier.put("domain", this.processDomain.toString());
                identifier.put("name", this.psnName);
                identifier.put("id", thsPatient.getIdentifier_id());
                patient.put("identifier", identifiers);
                patient.put("firstName", thsPatient.getFirstName());
                patient.put("lastName", thsPatient.getLastName());
                patient.put("birthdate", thsPatient.getBirthdate());
                patient.put("gender", thsPatient.getGender());
                patient.put("epix_value1", thsPatient.getTreatingPractice());
                pat.put("patient", patient);

                // contacts
                JSONArray contacts = new JSONArray();
                JSONObject contact = new JSONObject();
                contacts.add(contact);
                contact.put("city", thsPatient.getCity());
                contact.put("street", thsPatient.getStreet());
                contact.put("zipCode", thsPatient.getZipCode());
                pat.put("contacts", contacts);

                // consents
                JSONArray consents = new JSONArray();
                JSONObject consent = new JSONObject();
                consents.add(consent);
                consent.put("template", this.consentTemplate);
                consent.put("version", this.consentVersion);
                consent.put("processType", this.processType.toString());
                JSONArray modules = new JSONArray();
                HashMap<InformedConsentSection, Boolean> hmConsents = thsPatient.getConsentModule();
                for (Entry<InformedConsentSection, Boolean> consentEntry : hmConsents.entrySet()) {
                    JSONObject module = new JSONObject();
                    modules.add(module);
                    module.put("name", getTHSConsentModuleName(consentEntry.getKey()));
                    if (consentEntry.getValue().equals(Boolean.TRUE)) {
                        module.put("status", "accepted");
                    } else {
                        module.put("status", "declined");
                    }
                    consent.put("modules", modules);
                }
                pat.put("consents", consents);

                patients.add(pat);

            }
        }
		json.put("patients", patients);
		return json;
	}

	private String getTHSConsentModuleName(InformedConsentSection informedConsentSection) {
		String result;
		switch (informedConsentSection) {
		case Erhebung_Verarbeitung_Nutzung:
			result = "Erhebung, Verarbeitung und wissenschaftliche Nutzung";
			break;
		case Erneute_Kontaktaufnahme:
			result = "Erneute Kontaktaufnahme";
			break;
		case Geltungsdauer:
			result = "Geltungsdauer";
			break;
		case Widerrufsrecht:
			result = "Widerrufsrecht";
			break;
		default:
			result = "";
		}
		return result;
	}

	public RequestAddConsentResponse request(THSHelper helper) throws IOException, ParseException {
		HttpsURLConnection con = helper.getConnection(this.url);
		OutputStreamWriter post = new OutputStreamWriter(con.getOutputStream(), "UTF-8");

		this.toJSON().writeJSONString(post);
		post.flush();
		post.close();

		return (new RequestAddConsentResponse(con));
	}

	@Override
	public String toString() {
		return this.toJSON().toJSONString();
	}

}
