package de.uni_goettingen.med.mi.radar_software.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * util class to summarize to validation log file
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class LogAnalyser {
    public static final Logger log = LogManager.getLogger(LogAnalyser.class);

    protected enum ErrorType {
        Datatyps, unknownNumbers, invalidLengths, violatedRules, invalidSentenceStructure, wronglyFormatted
    }

    private File logFile;

    public LogAnalyser(File logFile) {
        this.logFile = logFile;
    }

    /**
     * Reads the given log file and parse it. Creates a list of Hashmaps which includes the error and their number of appearance.
     *
     * @return the list
     */
    private List<ErrorMap> createMaps() {
        List<ErrorMap> maps = new ArrayList<>(5);
        ErrorMap<String, Integer> violatedFieldLength = new ErrorMap<>("Fields with violated length", ErrorType.invalidLengths);
        ErrorMap<String, Integer> unknownFieldNumber = new ErrorMap<>("Fields with unknown fieldnumber", ErrorType.unknownNumbers);
        ErrorMap<String, Integer> violatedDataType = new ErrorMap<>("Field that violated their datatyp", ErrorType.Datatyps);
        ErrorMap<RuleErrorEntry, Integer> violatedRules = new ErrorMap<>("Fields that violated rules", ErrorType.violatedRules);
        ErrorMap<String, Integer> wrongFormat = new ErrorMap<>("Fields that were wrongly formatted", ErrorType.wronglyFormatted);
        ErrorMap<SentenceStructureErrorEntry, Integer> violatedSentenceStructure = new ErrorMap<>("Fields that violated the sentence structure", ErrorType.invalidSentenceStructure);
        maps.add(violatedDataType);
        maps.add(unknownFieldNumber);
        maps.add(violatedFieldLength);
        maps.add(violatedRules);
        maps.add(wrongFormat);
        maps.add(violatedSentenceStructure);

        try {
            BufferedReader reader = new BufferedReader(new FileReader(logFile));
            String line = reader.readLine();
            while (line != null) {
                Map<String, Integer> currentMap = new HashMap();
                String fieldNumber = getFieldNumber(line);

                if (line.contains("Rule")) {
                    addRuleEntry(violatedRules, line, fieldNumber);
                } else if (line.contains("expected one from")) {// illegal sentence structure (not checking rules
                    addSentenceStructurEntry(violatedSentenceStructure, line);
                } else {

                    if (line.contains("Field length") || line.contains("BDT line length")) {
                        currentMap = violatedFieldLength;
                    }
                    if (line.contains("was wrongly formatted")) {
                        currentMap = wrongFormat;
                    } else if (line.contains("with unknown number")) {
                        currentMap = unknownFieldNumber;
                    } else if (line.contains("Datatyp")) {
                        currentMap = violatedDataType;
                    }
                    // add field to map or increase counter
                    if (currentMap.containsKey(fieldNumber)) {

                        currentMap.replace(fieldNumber, currentMap.get(fieldNumber) + 1);
                    } else {
                        currentMap.put(fieldNumber, 1);
                    }

                }

                line = reader.readLine();
            }
            reader.close();
        } catch (IOException ioe) {
            log.error("An error occurred while parsing the log file");
            return null;
        }
        return maps;
    }

    /**
     * Helper function to a SenteceStructurEntry to Map, which holds all the sentence structure validation. 
     * If the entry is already in the map, the counter is increased otherwise it is added.
     *
     * @param violatedSentenceStructure the map of the sentence structure violations
     * @param line the current line from the log file
     */
    private void addSentenceStructurEntry(Map<SentenceStructureErrorEntry, Integer> violatedSentenceStructure, String line) {
        String fieldNumbers;
        int start = line.indexOf("Fieldnumber") + 12;// offset of 12: 11 characters in Fieldnumber and 1 whitespace
        int end = line.indexOf(": Sentence structure");
        fieldNumbers = line.substring(start, end);

        String sentenceNumber = getSentenceNumber(line);
        List<String> expectedFields = new ArrayList<>();
        extractExcpectedFields(expectedFields, line);
        SentenceStructureErrorEntry entry = new SentenceStructureErrorEntry(fieldNumbers, sentenceNumber, expectedFields);
        if (violatedSentenceStructure.containsKey(entry)) {
            int incValue = violatedSentenceStructure.get(entry) + 1;
            violatedSentenceStructure.replace(entry, incValue);
        } else {
            violatedSentenceStructure.put(entry, 1);
        }

    }

    /**
     * Extract the fields that would be expected to be next in the sentence from the given line. The extracted fields are stored in
     * the given List.
     *
     * @param expectedFields the list to store the fields
     * @param line the given lines
     */
    private void extractExcpectedFields(List<String> expectedFields, String line) {
        int start = line.indexOf("fields:") + 8;
        StringTokenizer tokenizer = new StringTokenizer(line.substring(start), ", ");
        while (tokenizer.hasMoreTokens()) {
            expectedFields.add(tokenizer.nextToken());
        }

    }

    /**
     * Extract and returns the sentence number as a String from the given line.
     *
     * @param line the given line
     * @return the sentence number
     */
    private String getSentenceNumber(String line) {
        int start = line.indexOf("Sentence structure") + 19;// offset of 19: 18 characters in sentence structure and 1 whitespace
        int end = start + 4;
        return line.substring(start, end);
    }

    /**
     * adds or updates (increasing the counter, if already included) an RuleErrorEntry in the map
     *
     * @param violatedRules the Map of violated rules
     * @param line the actual report line
     * @param fieldNumber the fieldnumber extracted from the report line.
     */
    private void addRuleEntry(Map<RuleErrorEntry, Integer> violatedRules, String line, String fieldNumber) {
        int start = line.indexOf("Rule") + 5;// offset of 5: 4 characters in rule and 1 whitespace
        int end = start + 3;
        // String ruleString = line.substring(start, end).replaceAll("[^\\d.]", ""); //remove all non numerical character; can appear if rule <100
        int rule = Integer.parseInt(line.substring(start).split(" ")[0]);
        RuleErrorEntry entry = new RuleErrorEntry(fieldNumber, rule);
        if (violatedRules.containsKey(entry)) {
            int incValue = violatedRules.get(entry) + 1;
            violatedRules.replace(entry, incValue);
        } else {
            violatedRules.put(entry, 1);
        }

    }

    /**
     * Extracts the fieldnumber from the given line.
     *
     * @param line the given line
     * @return the fieldnumber
     */
    private String getFieldNumber(String line) {
        int start = line.indexOf("Fieldnumber") + 12;// offset of 12: 11 characters in Fieldnumber and 1 whitespace
        int end = start + 4;
        return line.substring(start, end);
    }

    /**
     * Overrides the to String function. Should be called to generate and get the Summary of the validation log.
     *
     * @return the summary as string
     */
    @Override
    public String toString() {
        List<ErrorMap> maps = createMaps();
        StringBuilder results = new StringBuilder();
        for (ErrorMap map : maps) {
            appendErrors(results, map);
            results.append("\n");
        }
        return results.toString();
    }

    /**
     * helper function to append the errors from the maps with a matching message to the given string builder
     *
     * @param builder the string builder
     * @param map the map the errors should be extracted from
     */
    private void appendErrors(StringBuilder builder, ErrorMap map) {
        builder.append(map.Description).append("\n");
        Iterator iterator = map.entrySet().iterator();
        switch (map.getErrorType()) {
        case Datatyps:
            if (map.isEmpty()) {
                builder.append("No Data typ errors found!\n");
            } else {
                while (iterator.hasNext()) {
                    Map.Entry pair = (Map.Entry) iterator.next();
                    builder.append("Field ").append(pair.getKey()).append(": ").append(pair.getValue()).append(" times.\n");
                }
            }
            break;
        case violatedRules:
            if (map.isEmpty()) {
                builder.append("No rules were violated!\n");
            } else {
                while (iterator.hasNext()) {
                    Map.Entry pair = (Map.Entry) iterator.next();
                    RuleErrorEntry entry = (RuleErrorEntry) pair.getKey();
                    builder.append("Field ").append(entry.getFieldNumber()).append(", Rule ").append(entry.getRuleNumber()).append(": ").append(pair.getValue()).append(" times.\n");
                }
            }
            break;
        case invalidLengths:
            if (map.isEmpty()) {
                builder.append("No field lengths were violated!\n");
            } else {
                while (iterator.hasNext()) {
                    Map.Entry entry = (Map.Entry) iterator.next();
                    builder.append("Field ").append(entry.getKey()).append(": ").append(entry.getValue()).append(" times\n");
                }
            }
            break;
        case unknownNumbers:
            if (map.isEmpty()) {
                builder.append("No unknown Fields!\n");
            } else {
                while (iterator.hasNext()) {
                    Map.Entry entry = (Map.Entry) iterator.next();
                    builder.append("Field ").append(entry.getKey()).append(": ").append(entry.getValue()).append(" times\n");
                }
            }
            break;
        case invalidSentenceStructure:
            if (map.isEmpty()) {
                builder.append("No sentence violations found!\n");
            } else {
                while (iterator.hasNext()) {
                    Map.Entry entry = (Map.Entry) iterator.next();
                    SentenceStructureErrorEntry errorEntry = (SentenceStructureErrorEntry) entry.getKey();
                    builder.append("In sentence ").append(errorEntry.sentenceNumber).append(" expected one of the following fields: ");
                    for (String fieldNumber : errorEntry.expectedFieldNumbers) {
                        builder.append(fieldNumber).append(", ");
                    }
                    builder.deleteCharAt(builder.length() - 1).append(" but was ").append("Field ").append(errorEntry.fieldNumber).append(": ").append(entry.getValue()).append(" times\n");
                }
            }
            break;
        case wronglyFormatted:
            if (map.isEmpty()) {
                builder.append("No wrongly formatted fields were found!\n");
            } else {
                while (iterator.hasNext()) {
                    Map.Entry entry = (Map.Entry) iterator.next();
                    builder.append("Field ").append(entry.getKey()).append(": ").append(entry.getValue()).append(" times\n");
                }
            }
        }

    }
}
