package de.uni_goettingen.med.mi.radar_software.view;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.radar_software.MainApp;
import de.uni_goettingen.med.mi.radar_software.model.InformedConsent;
import javafx.application.HostServices;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.Label;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.layout.AnchorPane;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.FileChooser;

/**
 * The controller for the view of the administration of the informed consents.
 * Consents can be created, changed and deleted here.
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class InformedConsentController implements WizardController {

    private static final Logger log = LogManager.getLogger(InformedConsentController.class);
    private MainApp mainApp;

    @FXML
    private TableView<InformedConsent> informedConsentTable;

    @FXML
    private TableColumn<InformedConsent, String> informedConsentKeyColumn;

    @FXML
    private Label localSystemPatientIdLabel;

    @FXML
    private Label firstLetterFirstNameLabel;

    @FXML
    private Label consentSection1Label;

    @FXML
    private Label consentSection2Label;

    @FXML
    private Label consentSection3Label;

    @FXML
    private Label consentSection4Label;

    @FXML
    private Label firstLetterLastNameLabel;

    @FXML
    private void initialize() {
        log.debug("initialized");
        informedConsentKeyColumn.setCellValueFactory(cellData -> cellData.getValue().informedConsentKeyProperty());
        showInformedConsentDetails(null);
        informedConsentTable.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> showInformedConsentDetails(newValue));
    }

    @Override
    public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;
        log.debug("got " + this.mainApp);
        informedConsentTable.setItems(mainApp.getInformedConsentData());
        log.debug("loaded " + informedConsentTable.getItems().size() + " informed consents from file (previous session)");
    }

    @Override
    public void setHostServices(HostServices hostServices) {
    }

    /**
     * Method to add a consent for a new patient
     */
    @FXML
    private void newInformedConsent() {
        InformedConsent tempInformedConsent = new InformedConsent();
        boolean okClicked = showInformedConsentEditDialog(tempInformedConsent);
        if (okClicked) {
            mainApp.getInformedConsentData().add(tempInformedConsent);
            log.debug("added a new informed consent");
        }
    }

    /**
     * Method to edit the consent.
     */
    @FXML
    private void editInformedConsent() {
        InformedConsent selectedInformedConsent = informedConsentTable.getSelectionModel().getSelectedItem();
        if (selectedInformedConsent != null) {
            boolean okClicked = showInformedConsentEditDialog(selectedInformedConsent);
            if (okClicked) {
                showInformedConsentDetails(selectedInformedConsent);
                log.debug("edited an informed consent");
            }
        } else {
            Alert alert = new Alert(AlertType.WARNING);
            alert.initOwner(mainApp.getPrimaryStage());
            alert.setTitle("Warnung");
            alert.setHeaderText("Keine Einwilligung ausgewählt!");
            alert.setContentText("Bitte eine Einwilligung aus der Liste auswählen.");
            alert.showAndWait();
        }
    }

    /**
     * Method to delet a given consent.
     */
    @FXML
    private void deleteInformedConsent() {
        int selectedIndex = informedConsentTable.getSelectionModel().getSelectedIndex();
        if (selectedIndex >= 0) {
            informedConsentTable.getItems().remove(selectedIndex);
            log.debug("removed an informed consent");
        } else {
            Alert alert = new Alert(AlertType.WARNING);
            alert.initOwner(mainApp.getPrimaryStage());
            alert.setTitle("Warnung");
            alert.setHeaderText("Keine Einwilligung ausgewählt!");
            alert.setContentText("Bitte eine Einwilligung aus der Liste auswählen.");
            alert.showAndWait();
        }
    }
    
    /**
     * Method to load tablet consent file.
     */
    @FXML
    private void tabletBasedWorkflow() {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("Tablet-basierte Einwilligungen einlesen");
        fileChooser.getExtensionFilters().addAll(new FileChooser.ExtensionFilter("CSV Files", "*.csv"), new FileChooser.ExtensionFilter("Text Files", "*.txt"));
        File selectedFile = fileChooser.showOpenDialog(null);
        if (selectedFile != null) {
            readTabletFile(selectedFile.getAbsolutePath());
        }
    }
    
    private void readTabletFile(String filePath) {
        String COMMA_DELIMITER = ";";
        List<List<String>> records = new ArrayList<>();
        try (BufferedReader br = new BufferedReader(new FileReader(filePath))) {
            String line;
            int lineNumber = 0;
            while ((line = br.readLine()) != null) {
                lineNumber++;
                String[] values = line.split(COMMA_DELIMITER);
                if (lineNumber >= 3) // = ignore header
                    records.add(Arrays.asList(values));
            }
        } catch (IOException ioe) {
            log.error("Error loading tablet consent file " + filePath + " " + ioe.getMessage());
        }        
        for (List<String> record : records) {
            InformedConsent tempInformedConsent = new InformedConsent();
            tempInformedConsent.addInformedConsent(record.get(0), record.get(1), record.get(2));
            tempInformedConsent.setTabletWorkflow(true);
            tempInformedConsent.setConsentErhebungVerarbeitungNutzung(true);
            mainApp.getInformedConsentData().add(tempInformedConsent);
        }
        log.debug("loaded " + records.size() + " informed consents from tablet file");
    }

    /**
     * Show the detailed information about a consent.
     * @param informedConsent
     */
    private void showInformedConsentDetails(InformedConsent informedConsent) {
        if (informedConsent != null) {
            localSystemPatientIdLabel.setText(informedConsent.getLocalSystemPatientId());
            firstLetterFirstNameLabel.setText(informedConsent.getFirstLetterFirstName());
            firstLetterLastNameLabel.setText(informedConsent.getFirstLetterLastName());
            if (informedConsent.isTabletWorkflow()) {
                consentSection1Label.setText("über Tablet");
                consentSection2Label.setText("über Tablet");
                consentSection3Label.setText("über Tablet");
                consentSection4Label.setText("über Tablet");
            } else {
                consentSection1Label.setText(informedConsent.getTextConsentErhebungVerarbeitungNutzung());
                consentSection2Label.setText(informedConsent.getTextConsentErneuteKontaktaufnahme());
                consentSection3Label.setText(informedConsent.getTextConsentGeltungsdauer());
                consentSection4Label.setText(informedConsent.getTextConsentWiderrufsrecht());
            }
        } else {
            localSystemPatientIdLabel.setText("");
            firstLetterFirstNameLabel.setText("");
            firstLetterLastNameLabel.setText("");
            consentSection1Label.setText("");
            consentSection2Label.setText("");
            consentSection3Label.setText("");
            consentSection4Label.setText("");
        }
    }

    /**
     * Display the dialog to edit a consent.
     * @param informedConsent the consent that should be edited
     * @return
     */
    private boolean showInformedConsentEditDialog(InformedConsent informedConsent) {
        String fxmlFile = "/fxml/InformedConsentEditDialog.fxml";
        try {
            FXMLLoader loader = new FXMLLoader();
            loader.setLocation(MainApp.class.getResource(fxmlFile));
            AnchorPane page = (AnchorPane) loader.load();

            Stage dialogStage = new Stage();
            dialogStage.setTitle("Einwilligungserklärung");
            dialogStage.initModality(Modality.WINDOW_MODAL);
            dialogStage.initOwner(mainApp.getPrimaryStage());
            Scene scene = new Scene(page);
            dialogStage.setScene(scene);

            InformedConsentEditDialogController controller = loader.getController();
            controller.setDialogStage(dialogStage);
            controller.setInformedConsent(informedConsent);

            dialogStage.showAndWait();

            return controller.isOkClicked();
        } catch (IOException ioe) {
            log.error("Error loading fxmlFile " + fxmlFile + " " + ioe.getMessage());
            return false;
        }
    }

}