package de.uni_goettingen.med.mi.radar_software.view;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Properties;
import java.util.ResourceBundle;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.radar_software.MainApp;
import javafx.application.HostServices;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.geometry.Point2D;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Line;
import javafx.scene.text.Font;
import javafx.stage.Stage;

/**
 * The controller for the main layout.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class MainLayoutController implements Initializable {

    private static final Logger log = LogManager.getLogger(MainLayoutController.class);

    @FXML
    private AnchorPane wizardPane;

    @FXML
    private ImageView logo;

    @FXML
    private Label labelInfo;
    
    @FXML
    private Label labelBDTViewer;

    @FXML
    private Circle circle1;
    @FXML
    private Circle circle2;
    @FXML
    private Circle circle3;
    @FXML
    private Circle circle4;
    @FXML
    private Circle circle5;
    @FXML
    private Circle circle6;

    @FXML
    private Line line1;
    @FXML
    private Line line2;
    @FXML
    private Line line3;
    @FXML
    private Line line4;
    @FXML
    private Line line5;

    @FXML
    private Label label1;
    @FXML
    private Label label2;
    @FXML
    private Label label3;
    @FXML
    private Label label4;
    @FXML
    private Label label5;
    @FXML
    private Label label6;
    @FXML
    private Button btnBack;
    @FXML
    private Button btnNext;

    private ArrayList<String> lstWizardView;
    private ListIterator<String> iteratorLstWizardView;
    private int currentWizardViewIndex = 0;

    private MainApp mainApp;
    private FXMLLoader fxmlLoader;
    private HostServices hostServices;
    private static Properties fontProperties = loadFontProperties();
    private static Properties radarProperties = loadRadarProperties();
    private Font font;

	@Override
	public void initialize(URL location, ResourceBundle resources) {
		log.debug("initialized");
		loadFont();
		initializeIconLabels();
		Image img = new Image("/images/LogoRadar_engl.jpg");
		logo.setImage(img);
		initializeWizardViewList();
	}

	/**
	 * Loading the radar properties from file.
	 */
	protected static Properties loadRadarProperties() {
		Properties radarProperties = new Properties();
		InputStream radar = MainApp.class.getResourceAsStream("/RADAR.properties");
		try {
			radarProperties.load(radar);
		} catch (IOException ioe) {
			log.error(ioe.getMessage());
		}
		return radarProperties;
	}

	/**
	 * Loading the font variables from file.
	 */
	protected static Properties loadFontProperties() {
		fontProperties = new Properties();
		InputStream font = MainApp.class.getResourceAsStream("/fonts/fontawesome.properties");
		try {
			fontProperties.load(font);
		} catch (IOException ioe) {
			log.error(ioe.getMessage());
		}
		return fontProperties;
	}
	
    /**
     * Loading and assigning the awesome-webfont to the labels and buttons
     */
    private void loadFont() {
        InputStream fontInputStream;
        fontInputStream = MainApp.class.getResourceAsStream("/fonts/fontawesome-webfont.ttf");
        font = Font.loadFont(fontInputStream, 20);
    }

    /**
     * Method to open the viewe mode.
     */
    @FXML
    private void openViewer() {
        String fxmlFile = "/fxml/7_Viewer.fxml";
        try {
            Stage primaryStage = mainApp.getPrimaryStage();
            FXMLLoader loader = new FXMLLoader();
            Parent parent = (Parent) loader.load(getClass().getResourceAsStream(fxmlFile));
            loader.setResources(ResourceBundle.getBundle("fonts/fontawesome"));
            Scene scene = new Scene(parent, 1000, 600);
            primaryStage.setScene(scene);
            ViewerController controller = loader.getController();
            controller.setMainApp(this.mainApp);
            primaryStage.show();
        } catch (IOException ioe) {
            log.error("could not create viewer scene" + ioe.getMessage());
        }
    }

    /**
     * Method to set and display the tool tips.
     */
	@FXML
    private void showLabelInfoTooltip() {
	    Stage primaryStage = mainApp.getPrimaryStage();
        Point2D p = labelInfo.localToScene(0.0, 0.0);
        final Tooltip tooltip = new Tooltip();
        tooltip.setStyle("-fx-font-size: 16");
        String radar_version = radarProperties.getProperty("radar_version_number");
        tooltip.setText("Institut für Medizinsiche Informatik\nRADAR Software Prototyp\nVersion: " + radar_version + "\n");
	    labelInfo.setTooltip(tooltip);
	    tooltip.setAutoHide(true);
	    tooltip.show(primaryStage, p.getX()
	        + labelInfo.getScene().getX() + labelInfo.getScene().getWindow().getX(), p.getY()
	        + labelInfo.getScene().getY() + labelInfo.getScene().getWindow().getY());
	}
	
    public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;
        log.debug("got " + this.mainApp);
    }

    /**
     * Inits the icon labels
     */
    private void initializeIconLabels() {
        labelInfo.setFont(font);
        labelInfo.setText(fontProperties.getProperty("info"));
        labelInfo.setTextFill(Color.DARKBLUE);
        labelBDTViewer.setFont(font);
        labelBDTViewer.setText(fontProperties.getProperty("bdtviewer"));
        labelBDTViewer.setTextFill(Color.DARKBLUE);
    }

    /**
     * Inits the list of wizzard views.
     */
    private void initializeWizardViewList() {
        lstWizardView = new ArrayList<>();
        lstWizardView.add("/fxml/1_Welcome.fxml");
        lstWizardView.add("/fxml/2_InformedConsent.fxml");
        lstWizardView.add("/fxml/3_Parser.fxml");
        lstWizardView.add("/fxml/4_Pseudonymization.fxml");
        lstWizardView.add("/fxml/5_Summary.fxml");
        lstWizardView.add("/fxml/6_PackingAndShipping.fxml");
        iteratorLstWizardView = lstWizardView.listIterator();
    }

    /**
     * Inits the first view
     */
    public void initializeFirstView() {
        try {
            fxmlLoader = new FXMLLoader();
            fxmlLoader.setResources(ResourceBundle.getBundle("fonts/fontawesome"));
            fxmlLoader.load(getClass().getResourceAsStream(lstWizardView.get(currentWizardViewIndex)));
            setFocusToView(1);
            wizardPane.getChildren().add(fxmlLoader.getRoot());
            WizardController controller = fxmlLoader.getController();
            controller.setMainApp(this.mainApp);
            controller.setHostServices(hostServices);
            activateButton(0);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    /**
     * Method that highlights the current view/step in the process.
     * @param step
     */
    private void setFocusToView(int step) {
        switch (step) {
        case 1:
            circle1.setFill(Color.web("#0271c5"));
            label1.setTextFill((Color.web("#0271c5")));
            line1.setStroke(Color.web("#cccccd"));
            circle2.setFill(Color.web("#cccccd"));
            label2.setTextFill(Color.BLACK);
            break;
        case 2:
            circle1.setFill(Color.web("#cde5f4"));
            label1.setTextFill((Color.web("#000000")));
            line1.setStroke(Color.web("#cde5f4"));
            circle2.setFill(Color.web("#0271c5"));
            label2.setTextFill((Color.web("#0271c5")));
            line2.setStroke(Color.web("#cccccd"));
            circle3.setFill(Color.web("#cccccd"));
            label3.setTextFill(Color.BLACK);
            break;
        case 3:
            circle2.setFill(Color.web("#cde5f4"));
            label2.setTextFill((Color.web("#000000")));
            line2.setStroke(Color.web("#cde5f4"));
            circle3.setFill(Color.web("#0271c5"));
            label3.setTextFill((Color.web("#0271c5")));
            line3.setStroke(Color.web("#cccccd"));
            circle4.setFill(Color.web("#cccccd"));
            label4.setTextFill(Color.BLACK);
            break;
        case 4:
            circle3.setFill(Color.web("#cde5f4"));
            label3.setTextFill((Color.web("#000000")));
            line3.setStroke(Color.web("#cde5f4"));
            circle4.setFill(Color.web("#0271c5"));
            label4.setTextFill((Color.web("#0271c5")));
            line4.setStroke(Color.web("#cccccd"));
            circle5.setFill(Color.web("#cccccd"));
            label5.setTextFill(Color.BLACK);
            break;
        case 5:
            circle4.setFill(Color.web("#cde5f4"));
            label4.setTextFill((Color.web("#000000")));
            line4.setStroke(Color.web("#cde5f4"));
            circle5.setFill(Color.web("#0271c5"));
            label5.setTextFill((Color.web("#0271c5")));
            line5.setStroke(Color.web("#cccccd"));
            circle6.setFill(Color.web("#cccccd"));
            label6.setTextFill(Color.BLACK);
            break;
        case 6:
            circle5.setFill(Color.web("#cde5f4"));
            label5.setTextFill((Color.web("#000000")));
            line5.setStroke(Color.web("#cde5f4"));
            circle6.setFill(Color.web("#0271c5"));
            label6.setTextFill((Color.web("#0271c5")));
            break;
        default:
            break;

        }
    }

    /**
     * Method to load next view, if the next button was clicked.
     */
    @FXML
    private void clickNext() {
        if (currentWizardViewIndex < lstWizardView.size() - 1) {
            setFocusToView(currentWizardViewIndex + 2);
            iteratorLstWizardView.next();
            currentWizardViewIndex = iteratorLstWizardView.nextIndex();
            try {
                wizardPane.getChildren().remove(fxmlLoader.getRoot());
                fxmlLoader = new FXMLLoader();
                fxmlLoader.setResources(ResourceBundle.getBundle("fonts/fontawesome"));
                fxmlLoader.load(getClass().getResourceAsStream(lstWizardView.get(currentWizardViewIndex)));
                wizardPane.getChildren().add(fxmlLoader.getRoot());
                WizardController controller = fxmlLoader.getController();
                controller.setMainApp(this.mainApp);
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
        activateButton(currentWizardViewIndex);
    }

    /**
     * Method to load the previous view, if the back button was clicked.
     */
    @FXML
    private void clickBack() {
        if (currentWizardViewIndex >= 0) {
            currentWizardViewIndex = iteratorLstWizardView.previousIndex() - 1;
            setFocusToView(currentWizardViewIndex + 2);
            try {
                wizardPane.getChildren().remove(fxmlLoader.getRoot());
                fxmlLoader = new FXMLLoader();
                fxmlLoader.setResources(ResourceBundle.getBundle("fonts/fontawesome"));
                fxmlLoader.load(getClass().getResourceAsStream(iteratorLstWizardView.previous()));
                wizardPane.getChildren().add(fxmlLoader.getRoot());
                WizardController controller = fxmlLoader.getController();
                controller.setMainApp(this.mainApp);
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
        currentWizardViewIndex++;
        activateButton(currentWizardViewIndex);
    }

    /**
     * (De)activate the next and back buttons depending on the current view
     * @param currentWizardViewIndex
     */
    private void activateButton(int currentWizardViewIndex) {
        if (currentWizardViewIndex == 0) {
            btnBack.setDisable(true);
            btnBack.setVisible(false);
        } else if (currentWizardViewIndex == 5) {
            btnNext.setDisable(true);
            btnNext.setVisible(false);
        } else {
            btnNext.setDisable(false);
            btnBack.setDisable(false);
            btnNext.setVisible(true);
            btnBack.setVisible(true);
        }
    }

    public HostServices getHostServices() {
        return hostServices;
    }

    public void setHostServices(HostServices hostServices) {
        this.hostServices = hostServices;
    }

}
