package de.uni_goettingen.med.mi.radar_software.view;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.bdtparser.core.BDTMarshaller;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTArztpraxis;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTFileSaver;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTMetadata;
import de.uni_goettingen.med.mi.radar_software.MainApp;
import de.uni_goettingen.med.mi.radar_software.gwdg.TransferHelper;
import de.uni_goettingen.med.mi.radar_software.util.LogAnalyser;
import javafx.application.HostServices;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonBar;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;

/**
 * The controller for the packing and shipping view.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class PackingAndShippingController implements WizardController {

	@FXML
	private Label labelIsConnectionGWDGAvailable;

	@FXML
	private Label labelIsGWDGPasswordCorrect;

	@FXML
	private Label labelGWDGErrorText;

	@FXML
	private TextField textFieldGWDGPassword;

	@FXML
	private Button buttonTransfer;

	@FXML
	private Button buttonNoTransfer;
	
	@FXML
	private Label labelPseudonymisationText;

	private static final Logger log = LogManager.getLogger(PackingAndShippingController.class);
	private MainApp mainApp;
	private BDTArztpraxis bdtData;
	private BDTMarshaller bdtMarshaller;
	private BDTMetadata metadata;
	private long writetime = 0;
    private static Properties fontProperties = loadFontProperties();
    private static Properties radarProperties = loadRadarProperties();
	private TransferHelper transferHelper;
    private Font font;

	@FXML
	private void initialize() {
		log.debug("initialized");
		loadFont();
		initTransferHelper();
		checkConnectionGWDG();
	}

	@Override
	public void setMainApp(MainApp mainApp) {
		this.mainApp = mainApp;
		log.debug("got " + this.mainApp);
		if (this.mainApp != null) {
			this.bdtData = mainApp.getBdtData();
			this.bdtMarshaller = mainApp.getBdtMarshaller();
			metadata = mainApp.getBDTMetaData();
			if (mainApp.isPseudonymisationDone()) {
				labelPseudonymisationText.setVisible(false);
			} else {
				labelPseudonymisationText.setVisible(true);
			}
		}
	}
	
    /**
     * Loading the radar properties from file.
     */
    protected static Properties loadRadarProperties() {
        Properties radarProperties = new Properties();
        InputStream radar = MainApp.class.getResourceAsStream("/RADAR.properties");
        try {
            radarProperties.load(radar);
        } catch (IOException ioe) {
            log.error(ioe.getMessage());
        }
        return radarProperties;
    }

    /**
     * Loading the font variables from file.
     */
    protected static Properties loadFontProperties() {
        fontProperties = new Properties();
        InputStream font = MainApp.class.getResourceAsStream("/fonts/fontawesome.properties");
        try {
            fontProperties.load(font);
        } catch (IOException ioe) {
            log.error(ioe.getMessage());
        }
        return fontProperties;
    }
    
    /**
     * Loading and assigning the awesome-webfont to the labels and buttons
     */
    private void loadFont() {
        InputStream fontInputStream;
        fontInputStream = MainApp.class.getResourceAsStream("/fonts/fontawesome-webfont.ttf");
        font = Font.loadFont(fontInputStream, 20);
        labelIsGWDGPasswordCorrect.setFont(font);
    }

	/**
	 * Method that inits the helper class to connect to the data collector (GWDG)
	 */
	private void initTransferHelper() {
		transferHelper = new TransferHelper();
		buttonTransfer.setDisable(true);
		buttonNoTransfer.setDisable(true);
	}

	@Override
	public void setHostServices(HostServices hostServices) {
	}

	/**
	 * Method to check if the GWDG server are reachable.
	 */
	private void checkConnectionGWDG() {
		String ftp_serverAddress = radarProperties.getProperty("ftp_server");
        labelIsConnectionGWDGAvailable.setFont(font);
		if (ftp_serverAddress.isEmpty()) {
			labelGWDGErrorText.setText("no ftp server in property file");
		} else {
			transferHelper.setFtpServerAddress(ftp_serverAddress);
			String result = transferHelper.testConnection();
			if (result.isEmpty()) {
				labelIsConnectionGWDGAvailable.setText(fontProperties.getProperty("check-circle-o"));
				labelIsConnectionGWDGAvailable.setTextFill(Color.DARKSEAGREEN);
			} else {
				labelIsConnectionGWDGAvailable.setText(fontProperties.getProperty("times-circle"));
				labelIsConnectionGWDGAvailable.setTextFill(Color.DARKRED);
				labelGWDGErrorText.setText(result);
			}
		}
	}

	/**
	 * Method to check if it is possible to login into the GWDG server.
	 */
	@FXML
	private void checkLogin() {
		labelGWDGErrorText.setText("");
        labelIsGWDGPasswordCorrect.setFont(font);
		int ftp_port = Integer.valueOf(radarProperties.getProperty("ftp_port"));
		String ftp_user = radarProperties.getProperty("ftp_user");
		String ftp_password = textFieldGWDGPassword.getText();
		if (ftp_user.isEmpty()) {
			labelGWDGErrorText.setText("no ftp user in property file");
		} else if (ftp_password.isEmpty()) {
			labelGWDGErrorText.setText("no password");
		} else {
			transferHelper.setFtp_user(ftp_user);
			transferHelper.setFtpPort(ftp_port);
			transferHelper.setFtp_password(ftp_password);
			String result = transferHelper.testLogin();
			if (result.isEmpty()) {
				labelIsGWDGPasswordCorrect.setText(fontProperties.getProperty("check-circle-o"));
				labelIsGWDGPasswordCorrect.setTextFill(Color.DARKSEAGREEN);
				buttonTransfer.setDisable(false);
				buttonNoTransfer.setDisable(false);
			} else {
				labelIsGWDGPasswordCorrect.setText(fontProperties.getProperty("times-circle"));
				labelIsGWDGPasswordCorrect.setTextFill(Color.DARKRED);
				labelGWDGErrorText.setText(result);
			}
		}
	}

	/**
	 * method to start the transfer of the bdt files to the GWDG server
	 */
	@FXML
	private void actionStartTransfer() {
	    String directoryPath = getDirectory();
	    writeExportFiles(directoryPath);
	    uploadFiles(directoryPath);
	}

	/**
	 * Method to write the pseudonized bdt files
	 * @param directoryPath the path to the directory where the bdt files should be stored
	 */
    private void writeExportFiles(String directoryPath) {
        BDTFileSaver fileSaver = null;
        writetime = System.currentTimeMillis();
        String filePath = directoryPath + "\\bdtfiles-" + writetime;
        if (bdtData != null) {
            fileSaver = new BDTFileSaver(bdtMarshaller.datapackage, filePath);
            fileSaver.save(bdtData);
            try {
                writeMetadata(mainApp.getStartTime(), LocalDateTime.now(), bdtMarshaller.getNumberOfLines());
            } catch (IOException ioe) {
                log.error(ioe.getMessage());
            }
            try {
                pack(filePath, filePath + ".zip");
            } catch (IOException ioe) {
                log.error(ioe.getMessage());
            }
        } else {
            log.debug("no bdt data to write");
        }
    }

	/**
	 * Method to upload the files on the GWDG-Server
	 * @param directoryPath the path to the directory where the bdt files are stored
	 */
	private void uploadFiles(String directoryPath) {
        String filePath = directoryPath + "\\bdtfiles-" + writetime;
        File f = new File(filePath + ".zip");
        if (f.exists() && !f.isDirectory()) {
            transferHelper.setFilePath(filePath + ".zip");
            Boolean transferSuccessful = transferHelper.startTransfer();
            String alertString = "";
            if (transferSuccessful) {
                alertString = "Datenübertragung erfolgreich!";
            } else {
                alertString = "Datenübertragung gescheitert!";
            }
            String exitBtnText = "Programm beenden.";
            ButtonType exitBtn = new ButtonType(exitBtnText, ButtonBar.ButtonData.OK_DONE);
            Alert exitAlert = new Alert(Alert.AlertType.INFORMATION, alertString, exitBtn);
            exitAlert.showAndWait().filter(response -> response.getButtonData() == ButtonBar.ButtonData.OK_DONE).ifPresent(response -> Platform.exit());
        }
    }

	/**
	 * Method to pack the bdt files as .zip
	 * @param sourceDirPath the path to the bdt files
	 * @param zipFilePath the path where the zipped file should be stored
	 * @throws IOException
	 */
	private void pack(String sourceDirPath, String zipFilePath) throws IOException {
		Path p = Files.createFile(Paths.get(zipFilePath));
		try (ZipOutputStream zs = new ZipOutputStream(Files.newOutputStream(p))) {
			Path pp = Paths.get(sourceDirPath);
			Files.walk(pp).filter(path -> !Files.isDirectory(path)).forEach(path -> {
				ZipEntry zipEntry = new ZipEntry(pp.relativize(path).toString());
				try {
					zs.putNextEntry(zipEntry);
					Files.copy(path, zs);
					zs.closeEntry();
				} catch (IOException ioe) {
					log.error(ioe);
				}
			});
		}
	}

	/**
	 * Method to close the program.
	 */
	@FXML
	private void actionCloseWindow() {
		String exitBtnText = "Ja, und beenden.";
		String backBtnText = "Nein, und zurück.";
		String alertString = "Keine Daten übertragen und Programm beenden?";
		ButtonType exitBtn = new ButtonType(exitBtnText, ButtonBar.ButtonData.OK_DONE);
		ButtonType backBtn = new ButtonType(backBtnText, ButtonBar.ButtonData.BACK_PREVIOUS);
		Alert exitAlert = new Alert(Alert.AlertType.CONFIRMATION, alertString, exitBtn, backBtn);
		exitAlert.showAndWait().filter(response -> response.getButtonData() == ButtonBar.ButtonData.OK_DONE).ifPresent(response -> Platform.exit());
	}
	
    /**
     * Hidden functionality to export bdt-files without a transfer. Useful for testing purposes.
     */
    @FXML
    private void hiddenFunctionExportToFile() {
        String directoryPath = getDirectory();
        writeExportFiles(directoryPath);
    }

	/**
	 * Method to write  the meta data file
	 * @param start the start time of the parsing process
	 * @param end the end time of the parsing process
	 * @param readLines the number of lines read while parsing the files
	 * @throws IOException
	 */
	private void writeMetadata(LocalDateTime start, LocalDateTime end, long readLines) throws IOException {
		StringBuilder metaDataBuilder = new StringBuilder();
		metaDataBuilder.append("RADAR Metadata Log\n\n");
		metaDataBuilder.append("RADAR Software Version: ");
		String radar_version = radarProperties.getProperty("radar_version_number");
		metaDataBuilder.append(radar_version);
		metaDataBuilder.append("\n");
		
		metaDataBuilder.append("Start: ");
		if (start != null)
			metaDataBuilder.append(start.toString());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Ende: ");
		if (end != null)
			metaDataBuilder.append(end.toString());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Dauer: ");
		if (computeDuration(start, end) != null)
			metaDataBuilder.append(computeDuration(start, end));
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Gelesene Zeilen: ");
		metaDataBuilder.append(readLines);
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 9103 Datum der Erstellung: ");
		if (metadata.getDatumDerErstellung() != null)
			metaDataBuilder.append(metadata.getDatumDerErstellung().getValue());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 9106 Zeichencode: ");
		if (metadata.getZeichencode() != null)
			metaDataBuilder.append(metadata.getZeichencode().getValue());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 9210 Version ADT-Satz Beschreibung: ");
		if (metadata.getVersionADT() != null)
			metaDataBuilder.append(metadata.getVersionADT().getValue());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 9213 Version BDT: ");
		if (metadata.getVersionBDT() != null)
			metaDataBuilder.append(metadata.getVersionBDT());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 9600 Archivierungsart: ");
		if (metadata.getArchivierungsart() != null)
			metaDataBuilder.append(metadata.getArchivierungsart().getValue());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 9601 Zeitraum der Speicherung: ");
		if (metadata.getZeitraumDerSpeicherung() != null)
			metaDataBuilder.append(metadata.getZeitraumDerSpeicherung().getValue());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 9602 Beginn der Übertragung: ");
		if (metadata.getBeginnDerUebertragung() != null)
			metaDataBuilder.append(metadata.getBeginnDerUebertragung().getValue());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 0101 KBV-Prüfnummer: ");
		if (bdtData.getKBV_Puefnummer() != null)
			metaDataBuilder.append(bdtData.getKBV_Puefnummer().getValue());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("\n").append("Feld 0102 Softwareverantwortlicher: ");
		if (bdtData.getSoftwareverantworlicher() != null)
			metaDataBuilder.append(bdtData.getSoftwareverantworlicher());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 0103 Software: ");
		if (bdtData.getSoftware() != null)
			metaDataBuilder.append(bdtData.getSoftware());
		metaDataBuilder.append("\n");

		metaDataBuilder.append("Feld 0104 Hardware: ");
		if (bdtData.getHardware() != null)
			metaDataBuilder.append(bdtData.getHardware());
		metaDataBuilder.append("\n");
		metaDataBuilder.append("\n");
		
		if (bdtMarshaller.isLinesStartWithQuotes())
			metaDataBuilder.append("At least some lines started with quotes.\n\n");
		log.trace(metaDataBuilder.toString());
		String path = getDirectory();
		File dir = new File(path);
		if (dir.mkdirs())
			log.debug("created the directory");
		File resultsFile = new File(path + "bdtfiles-" + writetime + "\\metadata.log");
		if (!resultsFile.exists())
			if (resultsFile.createNewFile())
				log.info("created new report file");

		BufferedWriter writer = new BufferedWriter(new FileWriter(resultsFile));
		writer.write(metaDataBuilder.toString());
		InputStream pathStream = MainApp.class.getResourceAsStream("/RADAR.properties");
		Properties properties = new Properties();
		properties.load(pathStream);
		LogAnalyser logAnalyser = new LogAnalyser(new File(properties.getProperty("logfile_path")));
		writer.write(logAnalyser.toString());
		writer.close();
		pathStream.close();
	}

	/**
	 * Method that computed the processing based on the given start and end time.
	 * @param start the start time
	 * @param end the end time
	 * @return the duration
	 */
	private String computeDuration(LocalDateTime start, LocalDateTime end) {
		if (start == null || end == null) {
			log.error("error while computing duration; start or end time is null");
			return "";
		}
		Duration dur = Duration.between(start, end).abs();
		long min = dur.getSeconds() / 60;
		long sec = dur.getSeconds() - min * 60;
		long milli = dur.toMillis() - min * 60000 - sec * 1000;
		return min + " Minuten, " + sec + " Sekunden und " + milli + " Millisekunden ";
	}

	/**
	 * Method that returns the path to the directory on the USB stick
	 * @return the path to the directory
	 */
	private String getDirectory() {
		File USBPath = new File(mainApp.getUSBPath());
		String path = "";
		if (USBPath.getAbsolutePath() != null && !USBPath.getAbsolutePath().isEmpty()) {
			path = USBPath.getAbsolutePath() + "\\RADAR-Results\\";
		} else {
			log.error("No writable filepath found.");
		}
		return path;
	}

}
