package de.uni_goettingen.med.mi.radar_software.view;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.Properties;
import java.util.concurrent.LinkedTransferQueue;

import javax.swing.filechooser.FileSystemView;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.bdtparser.core.BDTMarshaller;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datacarrier.Datacarrier;
import de.uni_goettingen.med.mi.bdtparser.core.io.reader.BdtFileReader;
import de.uni_goettingen.med.mi.bdtparser.core.validator.ValidationException;
import de.uni_goettingen.med.mi.bdtparser.core.validator.Validator;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTVPRCreator;
import de.uni_goettingen.med.mi.radar_software.MainApp;
import javafx.application.HostServices;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.concurrent.Task;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;
import javafx.scene.control.TextField;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.stage.DirectoryChooser;

/**
 * Controller for the bdt-parser view.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class ParserController implements WizardController {

    private MainApp mainApp;
    private static final String DRIVEDISPLAYNAMESECURE = "SECURE";
    private static final String DRIVEDISPLAYNAMEVERBATIM = "Verbatim";
    private static final Logger log = LogManager.getLogger(ParserController.class);
    private DirectoryChooser directoryChooser;
    private static Properties fontProperties = loadFontProperties();
    private boolean isBdtFileInFolder = false;
    private boolean isBdtPathCorrect = false;
    private boolean isBdtUsbStickAvailable = false;
    private boolean isBdtUsbStickReadable = false;
    private final LinkedTransferQueue<String> input = new LinkedTransferQueue<String>();
    private BDTMarshaller bdtMarshaller;
    private Font fontLabel;
    private Font fontButton;

    @FXML
    private Label labelIsBdtFileAvailable;

    @FXML
    private Label labelIsBdtPathCorrect;

    @FXML
    private Label labelIsBdtUsbStickAvailable;

    @FXML
    private Label labelIsBdtUsbStickReadable;

    @FXML
    private Label progressBarLabel;

    @FXML
    private ProgressBar progressBar1;

    @FXML
    private ProgressBar progressBar2;

    @FXML
    private ProgressBar progressBar3;

    @FXML
    private TextField textFieldPathUsbStick;

    @FXML
    private Button buttonDirectoryChooser;

    @FXML
    private void initialize() {
        log.debug("initialized");
        loadFont();
        textFieldPathUsbStick.textProperty().addListener((observable, oldValue, newValue) -> {
            checkBdtPath();
            updateStatusIcons();
        });
        checkDrives();
        updateStatusIcons();
    }
    
    /**
     * Loading the radar properties from file.
     */
    protected static Properties loadRadarProperties() {
        Properties radarProperties = new Properties();
        InputStream radar = MainApp.class.getResourceAsStream("/RADAR.properties");
        try {
            radarProperties.load(radar);
        } catch (IOException ioe) {
            log.error(ioe.getMessage());
        }
        return radarProperties;
    }

    /**
     * Loading the font variables from file.
     */
    protected static Properties loadFontProperties() {
        fontProperties = new Properties();
        InputStream font = MainApp.class.getResourceAsStream("/fonts/fontawesome.properties");
        try {
            fontProperties.load(font);
        } catch (IOException ioe) {
            log.error(ioe.getMessage());
        }
        return fontProperties;
    }

    @Override
    public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;
        log.debug("got " + this.mainApp);
    }

    @Override
    public void setHostServices(HostServices hostServices) {
    }
    
    /**
     * Loading and assigning the awesome-webfont to the labels and buttons
     */
    private void loadFont() {
        InputStream fontInputStream;
        fontInputStream = MainApp.class.getResourceAsStream("/fonts/fontawesome-webfont.ttf");
        fontLabel = Font.loadFont(fontInputStream, 20.0);
        fontInputStream = MainApp.class.getResourceAsStream("/fonts/fontawesome-webfont.ttf");
        fontButton = Font.loadFont(fontInputStream, 14.0);
        buttonDirectoryChooser.setFont(fontButton);
    }

    /**
     * Open the directoryChooser, triggered by the folder button.
     */
    @FXML
    public void actionOpenDirectoryChooser() {
        directoryChooser = new DirectoryChooser();
        directoryChooser.setTitle("Pfadauswahl BDT-Ordner");
        File selectedFile = directoryChooser.showDialog(null);
        if (selectedFile != null) {
            textFieldPathUsbStick.setText(selectedFile.getAbsolutePath());
        }
    }

    /**
     * Starting the BDT-Parser, triggered by start button.
     */
    @FXML
    public void actionStartBdtParser() {
        if (isBdtFileInFolder) {
            mainApp.setUSBPath(new File(textFieldPathUsbStick.getText()).getAbsolutePath());
            mainApp.setStartTime(LocalDateTime.now());
            Task<Void> BDTInputTask = new Task<Void>() {
                @Override
                public Void call() throws Exception {
                    BdtFileReader bdtFileReader = new BdtFileReader(input, textFieldPathUsbStick.getText());
                    Thread bdtFileReaderThread = new Thread(bdtFileReader);
                    progressBar1.progressProperty().bind(bdtFileReader.processProperty);
                    bdtFileReaderThread.start();
                    bdtMarshaller = new BDTMarshaller(bdtFileReaderThread, bdtFileReader);
                    Thread marshallerThread = new Thread(bdtMarshaller);
                    progressBar2.progressProperty().bind(bdtMarshaller.processProperty);
                    marshallerThread.start();
                    try {// wait until marshaller is done
                        marshallerThread.join();
                        /*
                         * is a blocking call, so it will block the FX Application thread until the
                         * background thread completes. This will prevent the UI from being repainted,
                         * or from responding to user input.
                         * The easiest way to do what you want is to use a Task:
                         */
                    } catch (InterruptedException ie) {
                        ie.printStackTrace();
                    }
                    return null;
                }
            };
            BDTInputTask.setOnSucceeded(e -> {
                validationTask();
                log.debug("read all files and marshalled them");
            });
            BDTInputTask.setOnFailed(e -> {
                log.error("read files and marshall failed");
            });
            new Thread(BDTInputTask).start();
        }
    }

    /**
     * Task to validate the parsed bdt files
     */
    private void validationTask() {
        log.debug("Starting Validation");
        Task<Void> validationTask = new Task<Void>() {
            @Override
            public Void call() throws Exception {
                Validator validator = new Validator(progressBarLabel);
                DoubleProperty processProperty = new SimpleDoubleProperty();
                progressBar3.progressProperty().bind(processProperty);
                int i = 0;
                int bdtDatapackageSize = bdtMarshaller.datapackage.size();
                for (Datacarrier datacarrier : bdtMarshaller.datapackage.getDatacarriers()) {
                    try {
                        validator.validate(datacarrier, i == 0, i == bdtDatapackageSize - 1);
                    } catch (ValidationException ve) {
                        log.error(ve.getMessage());
                    } finally {
                        i++;
                        processProperty.set((double) i / (double) bdtDatapackageSize);
                    }
                }
                return null;
            }
        };
        validationTask.setOnSucceeded(e -> {
            vprCreationTask();
            log.debug("validated all data");
        });
        validationTask.setOnFailed(e -> {
            log.error("validated failed");
        });
        new Thread(validationTask).start();
    }

    private void vprCreationTask() {
        BDTVPRCreator bdtVprCreator = new BDTVPRCreator();
        mainApp.setBdtData(bdtVprCreator.createPatients(bdtMarshaller.datapackage));
        mainApp.setBdtMarshaller(bdtMarshaller);
        mainApp.setBDTMetaData(bdtVprCreator.getMetadata());
        log.debug("VPR creation done");
    }

    /**
     * Check if the entered bdtpath is a folder and the folder exists. If the folder exists check
     * the contained files if there is at least one file of the type bdt or xdt.
     */
    private void checkBdtPath() {
        File bdtFolder = new File(textFieldPathUsbStick.getText());
        isBdtPathCorrect = false;
        isBdtFileInFolder = false;
        if (bdtFolder.exists() && bdtFolder.isDirectory()) {
            isBdtPathCorrect = true;
            String fileName;
            File[] listOfFiles = bdtFolder.listFiles();
            if (listOfFiles != null) {
                for (int i = 0; i < listOfFiles.length; i++) {
                    if (listOfFiles[i].isFile()) {
                        fileName = listOfFiles[i].getName();
                        if (fileName.toUpperCase().contains(".BDT") || fileName.toUpperCase().contains(".XDT")) {
                            isBdtFileInFolder = true;
                            break;
                        }
                    }
                }
            }
        }
    }

    /**
     * Checking the system drives to detect the verbatim secure usb drive and check if it is
     * readable, so the drive is decrypted. Take the drive letter of the detected usb drive to set
     * the bdt path.
     */
    private void checkDrives() {
        log.debug("checking for usb drives...");
        FileSystemView fsv = FileSystemView.getFileSystemView();
        File[] f = File.listRoots();
        for (int i = 0; i < f.length; i++) {
            String driveName = f[i].toString();
            String displayName = fsv.getSystemDisplayName(f[i]);
            boolean isReadable = f[i].canRead();
            if (displayName != null && displayName.contains(DRIVEDISPLAYNAMEVERBATIM)) {
                isBdtUsbStickAvailable = true;
            } else if (displayName != null && displayName.contains(DRIVEDISPLAYNAMESECURE)) {
                if (isReadable) {
                    isBdtUsbStickReadable = true;
                    if (driveName != null) {
                        textFieldPathUsbStick.setText(driveName);
                    }
                }
            }
        }
        log.debug("isBdtUsbStickAvailable = " + isBdtUsbStickAvailable);
        log.debug("isBdtUsbStickReadable = " + isBdtUsbStickReadable);
        log.debug("set default path to = " + textFieldPathUsbStick.getText());
    }

    /**
     * Changes the icon and the color of the iconlabel according to the previous checks represented
     * by the variables isBdtUsbStickAvailable, ect.
     */
    private void updateStatusIcons() {
        // Icon isBdtUsbStickAvailable
        labelIsBdtUsbStickAvailable.setFont(fontLabel);
        if (isBdtUsbStickAvailable) {
            labelIsBdtUsbStickAvailable.setText(fontProperties.getProperty("check-circle-o"));
            labelIsBdtUsbStickAvailable.setTextFill(Color.DARKSEAGREEN);
        } else {
            labelIsBdtUsbStickAvailable.setText(fontProperties.getProperty("times-circle"));
            labelIsBdtUsbStickAvailable.setTextFill(Color.DARKRED);
        }
        // Icon isBdtUsbStickReadable
        labelIsBdtUsbStickReadable.setFont(fontLabel);
        if (isBdtUsbStickReadable) {
            labelIsBdtUsbStickReadable.setText(fontProperties.getProperty("check-circle-o"));
            labelIsBdtUsbStickReadable.setTextFill(Color.DARKSEAGREEN);
        } else {
            labelIsBdtUsbStickReadable.setText(fontProperties.getProperty("times-circle"));
            labelIsBdtUsbStickReadable.setTextFill(Color.DARKRED);
        }
        // Icon isBdtPathCorrect
        labelIsBdtPathCorrect.setFont(fontLabel);
        if (isBdtPathCorrect) {
            labelIsBdtPathCorrect.setText(fontProperties.getProperty("check-circle-o"));
            labelIsBdtPathCorrect.setTextFill(Color.DARKSEAGREEN);
        } else {
            labelIsBdtPathCorrect.setText(fontProperties.getProperty("times-circle"));
            labelIsBdtPathCorrect.setTextFill(Color.DARKRED);
        }
        // Icon isBdtFileInFolder
        labelIsBdtFileAvailable.setFont(fontLabel);
        if (isBdtFileInFolder) {
            labelIsBdtFileAvailable.setText(fontProperties.getProperty("check-circle-o"));
            labelIsBdtFileAvailable.setTextFill(Color.DARKSEAGREEN);
        } else {
            labelIsBdtFileAvailable.setText(fontProperties.getProperty("times-circle"));
            labelIsBdtFileAvailable.setTextFill(Color.DARKRED);
        }
    }

}
