package de.uni_goettingen.med.mi.radar_software.view;

import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.json.simple.parser.ParseException;

import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTAdresse;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTArztpraxis;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTPatient;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTValue;
import de.uni_goettingen.med.mi.radar_software.MainApp;
import de.uni_goettingen.med.mi.radar_software.model.InformedConsent;
import de.uni_goettingen.med.mi.radar_software.model.Patient;
import de.uni_goettingen.med.mi.radar_software.ths.THSHelper;
import de.uni_goettingen.med.mi.radar_software.ths.message.GetSessionRequest;
import de.uni_goettingen.med.mi.radar_software.ths.message.GetSessionResponse;
import de.uni_goettingen.med.mi.radar_software.ths.message.GetTokenRequest;
import de.uni_goettingen.med.mi.radar_software.ths.message.GetTokenRequest.TokenReason;
import de.uni_goettingen.med.mi.radar_software.ths.message.GetTokenRequest.TokenTargetType;
import de.uni_goettingen.med.mi.radar_software.ths.message.GetTokenResponse;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestAddConsent;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestAddConsentResponse;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestAddPatient;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestAddPatientResponse;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestPSNRequest;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestPSNRequest.RequestProcessDomain;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestPSNResponse;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestUpdateIDAT;
import de.uni_goettingen.med.mi.radar_software.ths.message.RequestUpdateIDATResponse;
import de.uni_goettingen.med.mi.radar_software.ths.message.THSPatient;
import javafx.application.HostServices;
import javafx.beans.property.BooleanProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.Tooltip;
import javafx.scene.control.cell.CheckBoxTableCell;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;

/**
 * The controller for the pseudonymization view.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class PseudonymizationController implements WizardController {

    private static final Logger log = LogManager.getLogger(PseudonymizationController.class);
    private MainApp mainApp;
    private ObservableList<Patient> lstPatient = FXCollections.observableArrayList();
    private static Properties fontProperties = loadFontProperties();
    private static Properties radarProperties = loadRadarProperties();
    private THSHelper thsHelper;
    private String arztnummer;
    private Font font;

    @FXML
    private Label labelIsConnectionTHSAvailable;

    @FXML
    private Label labelIsTHSPasswordCorrect;

    @FXML
    private Label labelTimeSpanFilter;

    @FXML
    private Label labelTHSErrorText;

    @FXML
    private TextField textFieldTHSPassword;

    @FXML
    private Button buttonPseudonym;

    @FXML
    private TableView<Patient> patientTable;

    @FXML
    private TableColumn<Patient, Boolean> consentColumn;

    @FXML
    private TableColumn<Patient, String> pseudonymColumn;

    @FXML
    private TableColumn<Patient, String> localSystemPatientIdColumn;

    @FXML
    private TableColumn<Patient, String> firstNameColumn;

    @FXML
    private TableColumn<Patient, String> lastNameColumn;

    @FXML
    private TableColumn<Patient, String> birthDateColumn;

    @FXML
    private TableColumn<Patient, String> genderColumn;

    @FXML
    private TableColumn<Patient, String> cityColumn;

    @FXML
    private TableColumn<Patient, String> zipCodeColumn;

    @FXML
    private TableColumn<Patient, String> streetColumn;

    @FXML
    private void initialize() {
        log.debug("initialized");
        loadFont();
        patientTable.setStyle("-fx-font: normal 13px 'sansserif' ");
        initialiseTimeSpanLabel();
        initTHSHelper();
        checkTHSConnection();
    }
    
    /**
     * Loading the radar properties from file.
     */
    protected static Properties loadRadarProperties() {
        Properties radarProperties = new Properties();
        InputStream radar = MainApp.class.getResourceAsStream("/RADAR.properties");
        try {
            radarProperties.load(radar);
        } catch (IOException ioe) {
            log.error(ioe.getMessage());
        }
        return radarProperties;
    }

    /**
     * Loading the font variables from file.
     */
    protected static Properties loadFontProperties() {
        fontProperties = new Properties();
        InputStream font = MainApp.class.getResourceAsStream("/fonts/fontawesome.properties");
        try {
            fontProperties.load(font);
        } catch (IOException ioe) {
            log.error(ioe.getMessage());
        }
        return fontProperties;
    }

    /**
     * Method that is called when the pseudonymisation button is clicked. Its request a pseudonym for each patient in the list.
     */
    @FXML
    public void actionGetPseudonyms() {
        
        String thsBasicAuthPassword = textFieldTHSPassword.getText();
        
        if (!thsBasicAuthPassword.isEmpty() && thsBasicAuthPassword.length() > 0) {

            if (thsHelper != null) {
                thsHelper.setBasicAuthPassword(thsBasicAuthPassword);
                labelIsTHSPasswordCorrect.setFont(font);
                labelIsTHSPasswordCorrect.setText(fontProperties.getProperty("check-circle-o"));
                labelIsTHSPasswordCorrect.setTextFill(Color.DARKSEAGREEN);

                try {
//                	log.debug("---");
//                	log.debug("PSN BSNR Temp");
//                	log.debug("---");
                	
                    // Session
                    GetSessionRequest gsRequest1_BSNR = new GetSessionRequest(radarProperties.getProperty("basicAuthName"), radarProperties.getProperty("radar_instanz_no"));
//                    log.debug("gsRequest1_BSNR");
//                    log.debug(gsRequest1_BSNR);
                    GetSessionResponse gsResponse1_BSNR = gsRequest1_BSNR.request(thsHelper);
//                    log.debug("gsResponse1_BSNR");
//                    log.debug(gsResponse1_BSNR);

                    // Token
                    GetTokenRequest gtRequest1_BSNR = new GetTokenRequest(gsResponse1_BSNR);
                    gtRequest1_BSNR.setType(GetTokenRequest.TokenType.requestPSN);
                    gtRequest1_BSNR.setMethod(GetTokenRequest.TokenMethod.getOrCreate);
                    gtRequest1_BSNR.setTargetType(GetTokenRequest.TokenTargetType.bsnr_temp);
                    gtRequest1_BSNR.setStudyId(radarProperties.getProperty("study_id"));
                    gtRequest1_BSNR.setStudyName(radarProperties.getProperty("study_name"));
                    gtRequest1_BSNR.setEvent(GetTokenRequest.TokenEvent.requestPSNBsnrTemp);
                    gtRequest1_BSNR.setReason(GetTokenRequest.TokenReason.generateTempFromBsnrForDb);
//                    log.debug("gtRequest1_BSNR");
//                    log.debug(gtRequest1_BSNR);
                    GetTokenResponse gtResponse1_BSNR = gtRequest1_BSNR.request(thsHelper);
//                    log.debug("gtResponse1_BSNR");
//                    log.debug(gtResponse1_BSNR);

                    // PSN BSNR Temp
                    RequestPSNRequest rpsnRequest1_BSNR = new RequestPSNRequest(gtResponse1_BSNR);
                    rpsnRequest1_BSNR.setBsnr(this.arztnummer);
                    rpsnRequest1_BSNR.setPsnDomain(RequestProcessDomain.bsnr_orig);
                    rpsnRequest1_BSNR.setPsnName("PSN");
//                    log.debug("rpsnRequest1_BSNR");
//                    log.debug(rpsnRequest1_BSNR);
                    RequestPSNResponse rpsnResponse1_BSNR = rpsnRequest1_BSNR.request(thsHelper);
//                    log.debug("rpsnResponse1_BSNR");
//                    log.debug(rpsnResponse1_BSNR);
                    String bsnrPsn = rpsnResponse1_BSNR.getBsnrPsn();
//                    log.debug("BSNR TMP PSN: " + bsnrPsn);
                    updateArztnummer(bsnrPsn);

                    List<THSPatient> thsPatients = buildTHSPatients();
                    HashMap<String, String> hmAisPsn;
                    
                    if (isTabletWorkflow()) {
                        
                        log.debug("*** TABLET WORKFLOW ***");
                        
//                        log.debug("---");
//                        log.debug("updateIDAT");
//                        log.debug("---");
                        
                        // Session
                        GetSessionRequest gsRequest2_updateIDAT = new GetSessionRequest(radarProperties.getProperty("basicAuthName"), radarProperties.getProperty("radar_instanz_no"));
//                        log.debug("gsRequest2_updateIDAT");
//                        log.debug(gsRequest2_updateIDAT);
                        GetSessionResponse gsResponse2_updateIDAT = gsRequest2_updateIDAT.request(thsHelper);
//                        log.debug("gsResponse2_updateIDAT");
//                        log.debug(gsResponse2_updateIDAT);
                        
                        // Token
                        GetTokenRequest gtRequest2_updateIDAT = new GetTokenRequest(gsResponse2_updateIDAT);
                        gtRequest2_updateIDAT.setType(GetTokenRequest.TokenType.updateIDAT);
                        gtRequest2_updateIDAT.setEvent(GetTokenRequest.TokenEvent.updateIDATForTabletPatient);
                        gtRequest2_updateIDAT.setTargetIdType(GetTokenRequest.TokenTargetIdType.ais);
                        gtRequest2_updateIDAT.setStudyId(radarProperties.getProperty("study_id"));
                        gtRequest2_updateIDAT.setStudyName(radarProperties.getProperty("study_name"));
                        gtRequest2_updateIDAT.setLocation_id(this.arztnummer);
                        gtRequest2_updateIDAT.setLocation_name(this.arztnummer);
//                        log.debug("gtRequest2_updateIDAT");
//                        log.debug(gtRequest2_updateIDAT);
                        GetTokenResponse gtResponse2_updateIDAT = gtRequest2_updateIDAT.request(thsHelper);
//                        log.debug("gtResponse2_updateIDAT");
//                        log.debug(gtResponse2_updateIDAT);
                        
                        // updateIDAT
                        RequestUpdateIDAT rpsnRequest2_updateIDAT = new RequestUpdateIDAT(gtResponse2_updateIDAT);
                        rpsnRequest2_updateIDAT.setPatients(thsPatients);
                        rpsnRequest2_updateIDAT.setPsnDomain(de.uni_goettingen.med.mi.radar_software.ths.message.RequestUpdateIDAT.RequestProcessDomain.RADAR);
                        rpsnRequest2_updateIDAT.setPsnName("AIS");
//                        log.debug("rpsnRequest2_updateIDAT");
//                        log.debug(rpsnRequest2_updateIDAT);
                        RequestUpdateIDATResponse rpsnResponse2_updateIDAT = rpsnRequest2_updateIDAT.request(thsHelper);
//                        log.debug("rpsnResponse2_updateIDAT");
//                        log.debug(rpsnResponse2_updateIDAT);
                        
                        if (!rpsnResponse2_updateIDAT.containsErrors()) {
                            
//                            log.debug("---");
//                            log.debug("AIS PSN");
//                            log.debug("---");

                            // Session
                            GetSessionRequest gsRequest3_AISPSN = new GetSessionRequest(radarProperties.getProperty("basicAuthName"),
                                    radarProperties.getProperty("radar_instanz_no"));
//                            log.debug("gsRequest3_AISPSN");
//                            log.debug(gsRequest3_AISPSN);
                            GetSessionResponse gsResponse3_AISPSN = gsRequest3_AISPSN.request(thsHelper);
//                            log.debug("gsResponse3_AISPSN");
//                            log.debug(gsResponse3_AISPSN);

                            // Token
                            GetTokenRequest gtRequest3_AISPSN = new GetTokenRequest(gsResponse3_AISPSN);
                            gtRequest3_AISPSN.setType(GetTokenRequest.TokenType.requestPSN);
                            gtRequest3_AISPSN.setMethod(GetTokenRequest.TokenMethod.getOrCreate);
                            gtRequest3_AISPSN.setEvent(GetTokenRequest.TokenEvent.requestAisPsnByPatId);
                            gtRequest3_AISPSN.setReason(TokenReason.generateAisPsnByPatId);
                            gtRequest3_AISPSN.setTargetIdType(GetTokenRequest.TokenTargetIdType.ais);
                            gtRequest3_AISPSN.setStudyId(radarProperties.getProperty("study_id"));
                            gtRequest3_AISPSN.setStudyName(radarProperties.getProperty("study_name"));
//                            log.debug("gtRequest3_AISPSN");
//                            log.debug(gtRequest3_AISPSN);
                            GetTokenResponse gtResponse3_AISPSN = gtRequest3_AISPSN.request(thsHelper);
//                            log.debug("gtResponse3_AISPSN");
//                            log.debug(gtResponse3_AISPSN);

                            // Request AIS PSN
                            RequestPSNRequest rpsnRequest3_AISPSN = new RequestPSNRequest(gtResponse3_AISPSN);
                            rpsnRequest3_AISPSN.setPatients(thsPatients);
                            rpsnRequest3_AISPSN.setPsnDomain(RequestProcessDomain.RADAR);
                            rpsnRequest3_AISPSN.setPsnName("AIS");
//                            log.debug("rpsnRequest3_AISPSN");
//                            log.debug(rpsnRequest3_AISPSN);
                            RequestPSNResponse rpsnResponse3_AISPSN = rpsnRequest3_AISPSN.request(thsHelper);
//                            log.debug("rpsnResponse3_AISPSN");
//                            log.debug(rpsnResponse3_AISPSN);

                            // Output results
                            hmAisPsn = rpsnResponse3_AISPSN.getPatientIdentifiers();
//                            log.debug("AIS PSN: " + rpsnResponse3_AISPSN.getPatientIdentifiers());

                        } else {
                            hmAisPsn = new HashMap<String, String>();
                            log.debug("Kein Consent für diese Person bei der Treuhandstelle vorhanden!");
                            log.debug(rpsnResponse2_updateIDAT.getErrorCodes().toString());
                        }
                        
                    } else {
                        
//                        log.debug("---");
//                        log.debug("AIS PSN");
//                        log.debug("---");
                        
                        // Session
                        GetSessionRequest gsRequest2_AISPSN = new GetSessionRequest(radarProperties.getProperty("basicAuthName"), radarProperties.getProperty("radar_instanz_no"));
//                        log.debug("gsRequest2_AISPSN");
//                        log.debug(gsRequest2_AISPSN);
                        GetSessionResponse gsResponse2_AISPSN = gsRequest2_AISPSN.request(thsHelper);
//                        log.debug("gsResponse2_AISPSN");
//                        log.debug(gsResponse2_AISPSN);

                        // Token
                        GetTokenRequest gtRequest2_AISPSN = new GetTokenRequest(gsResponse2_AISPSN);
                        gtRequest2_AISPSN.setType(GetTokenRequest.TokenType.addPatient);
                        gtRequest2_AISPSN.setEvent(GetTokenRequest.TokenEvent.addPatientAIS);
                        gtRequest2_AISPSN.setTargetIdType(GetTokenRequest.TokenTargetIdType.ais);
                        gtRequest2_AISPSN.setStudyId(radarProperties.getProperty("study_id"));
                        gtRequest2_AISPSN.setStudyName(radarProperties.getProperty("study_name"));
                        gtRequest2_AISPSN.setLocation_id(this.arztnummer);
                        gtRequest2_AISPSN.setLocation_name(this.arztnummer);
//                        log.debug("gtRequest2_AISPSN");
//                        log.debug(gtRequest2_AISPSN);
                        GetTokenResponse gtResponse2_AISPSN = gtRequest2_AISPSN.request(thsHelper);
//                        log.debug("gtResponse2_AISPSN");
//                        log.debug(gtResponse2_AISPSN);

                        // addPatient
                        RequestAddPatient rpsnRequest2_AISPSN = new RequestAddPatient(gtResponse2_AISPSN);
                        rpsnRequest2_AISPSN.setPatients(thsPatients);
                        rpsnRequest2_AISPSN.setPsnDomain(de.uni_goettingen.med.mi.radar_software.ths.message.RequestAddPatient.RequestProcessDomain.RADAR);
                        rpsnRequest2_AISPSN.setPsnName("AIS");
//                        log.debug("rpsnRequest2_AISPSN");
//                        log.debug(rpsnRequest2_AISPSN);
                        RequestAddPatientResponse rpsnResponse2_AISPSN = rpsnRequest2_AISPSN.request(thsHelper);
                        //log.debug("rpsnResponse2_AISPSN");
                        //log.debug(rpsnResponse2_AISPSN);
                        
                        if (rpsnResponse2_AISPSN.containsErrors()) {
                            log.debug("Treuhandstelle meldet unsicheren Match für diesen Patienten!");
                        }
                                                
                        // Output results
                        hmAisPsn = rpsnResponse2_AISPSN.getPsnList();
//                        log.debug("AIS PSN: " + rpsnResponse2_AISPSN.getPsnList());
                       
//                        log.debug("---");
//                        log.debug("addConsentByPatient");
//                        log.debug("---");
                        
                        // Session
                        GetSessionRequest gsRequest3_addConsentByPatient = new GetSessionRequest(radarProperties.getProperty("basicAuthName"), radarProperties.getProperty("radar_instanz_no"));
//                        log.debug("gsRequest3_addConsentByPatient");
//                        log.debug(gsRequest3_addConsentByPatient);
                        GetSessionResponse gsResponse3_addConsentByPatient = gsRequest3_addConsentByPatient.request(thsHelper);
//                        log.debug("gsResponse3_addConsentByPatient");
//                        log.debug(gsResponse3_addConsentByPatient);

                        // Token
                        GetTokenRequest gtRequest3_addConsentByPatient = new GetTokenRequest(gsResponse3_addConsentByPatient);
                        gtRequest3_addConsentByPatient.setType(GetTokenRequest.TokenType.addConsentByPatient);
                        gtRequest3_addConsentByPatient.setEvent(GetTokenRequest.TokenEvent.addConsentAIS);
                        gtRequest3_addConsentByPatient.setStudyId(radarProperties.getProperty("study_id"));
                        gtRequest3_addConsentByPatient.setStudyName(radarProperties.getProperty("study_name"));
                        gtRequest3_addConsentByPatient.setLocation_id(this.arztnummer);
                        gtRequest3_addConsentByPatient.setLocation_name(this.arztnummer);
//                        log.debug("gtRequest3_addConsentByPatient");
//                        log.debug(gtRequest3_addConsentByPatient);
                        GetTokenResponse gtResponse3_addConsentByPatient = gtRequest3_addConsentByPatient.request(thsHelper);
//                        log.debug("gtResponse3_addConsentByPatient");
//                        log.debug(gtResponse3_addConsentByPatient);

                        // addConsentByPatient
                        RequestAddConsent rpsnRequest3_addConsentByPatient = new RequestAddConsent(gtResponse3_addConsentByPatient);
                        rpsnRequest3_addConsentByPatient.setPatients(thsPatients);
                        rpsnRequest3_addConsentByPatient.setPsnDomain(de.uni_goettingen.med.mi.radar_software.ths.message.RequestAddConsent.RequestProcessDomain.RADAR);
                        rpsnRequest3_addConsentByPatient.setPsnName("AIS");  
                        rpsnRequest3_addConsentByPatient.setConsentTemplate(radarProperties.getProperty("consentTemplate"));
                        rpsnRequest3_addConsentByPatient.setConsentVersion(radarProperties.getProperty("consentVersion"));
                        rpsnRequest3_addConsentByPatient.setSourceType(RequestAddConsent.RequestProcessType.addConsent);
//                        log.debug("rpsnRequest3_addConsentByPatient");
//                        log.debug(rpsnRequest3_addConsentByPatient);
                        RequestAddConsentResponse rpsnResponse3_addConsentByPatient = rpsnRequest3_addConsentByPatient.request(thsHelper);
//                        log.debug("rpsnResponse3_addConsentByPatient");
//                        log.debug(rpsnResponse3_addConsentByPatient);

                    }

//                    log.debug("---");
//                    log.debug("AIS Temp PSN");
//                    log.debug("---");

                    // Session
                    GetSessionRequest gsRequest4_TMP_AISPSN = new GetSessionRequest(radarProperties.getProperty("basicAuthName"), radarProperties.getProperty("radar_instanz_no"));
//                    log.debug("gsRequest4_TMP_AISPSN");
//                    log.debug(gsRequest4_TMP_AISPSN);
                    GetSessionResponse gsResponse4_TMP_AISPSN = gsRequest4_TMP_AISPSN.request(thsHelper);
//                    log.debug("gsResponse4_TMP_AISPSN");
//                    log.debug(gsResponse4_TMP_AISPSN);

                    // Token
                    GetTokenRequest gtRequest4_TMP_AISPSN = new GetTokenRequest(gsResponse4_TMP_AISPSN);
                    gtRequest4_TMP_AISPSN.setType(GetTokenRequest.TokenType.requestPSN);
                    gtRequest4_TMP_AISPSN.setMethod(GetTokenRequest.TokenMethod.getOrCreate);
                    gtRequest4_TMP_AISPSN.setEvent(GetTokenRequest.TokenEvent.requestPSNTemp);
                    gtRequest4_TMP_AISPSN.setTargetType(TokenTargetType.temp);
                    gtRequest4_TMP_AISPSN.setReason(TokenReason.generateTempFromAisForDb);
                    gtRequest4_TMP_AISPSN.setStudyId(radarProperties.getProperty("study_id"));
                    gtRequest4_TMP_AISPSN.setStudyName(radarProperties.getProperty("study_name"));                  
//                    log.debug("gtRequest4_TMP_AISPSN");
//                    log.debug(gtRequest4_TMP_AISPSN);
                    GetTokenResponse gtResponse4_TMP_AISPSN = gtRequest4_TMP_AISPSN.request(thsHelper);
//                    log.debug("gtResponse4_TMP_AISPSN");
//                    log.debug(gtResponse4_TMP_AISPSN);

                    // AIS Temp PSN
                    RequestPSNRequest rpsnRequest4_TMP_AISPSN = new RequestPSNRequest(gtResponse4_TMP_AISPSN);

                    rpsnRequest4_TMP_AISPSN.setAisPsn(hmAisPsn);
                    rpsnRequest4_TMP_AISPSN.setPsnDomain(RequestProcessDomain.ais);
                    rpsnRequest4_TMP_AISPSN.setPsnName("PSN");
//                    log.debug("rpsnRequest4_TMP_AISPSN");
//                    log.debug(rpsnRequest4_TMP_AISPSN);
                    RequestPSNResponse rpsnResponse4_TMP_AISPSN = rpsnRequest4_TMP_AISPSN.request(thsHelper);
//                    log.debug("rpsnResponse4_TMP_AISPSN");
//                    log.debug(rpsnResponse4_TMP_AISPSN);

                    // Output results
                    HashMap<String, String> hmTmpAisPsn = rpsnResponse4_TMP_AISPSN.getPatientIdentifiers();
					if (hmAisPsn != null && hmTmpAisPsn != null) {
//						log.debug("AIS TMP PSN: " + rpsnResponse4_TMP_AISPSN.getPatientIdentifiers());
						for (THSPatient thsPatient : thsPatients) {
							String pseudonymAis = hmAisPsn.get(String.valueOf(thsPatient.getIndex()));
							String pseudonymTmpAis = hmTmpAisPsn.get(String.valueOf(thsPatient.getIndex()));
							updatePseudonym(thsPatient.getIdentifier_id(), pseudonymAis, pseudonymTmpAis);
						}
					}

                } catch (IOException | ParseException e) {
                    log.error(e.getMessage());
                    labelIsTHSPasswordCorrect.setFont(font);
                    labelIsTHSPasswordCorrect.setText(fontProperties.getProperty("times-circle"));
                    labelIsTHSPasswordCorrect.setTextFill(Color.DARKRED);
                }
            }
        }
    }

    /**
     * updates the arztnummer. Converts the given String to a number and uses it as a new Arztnummer.
     * @param bsnrPsn the new Arztnummer. Should be parsable as Integer
     */
    private void updateArztnummer(String bsnrPsn) {
        if (bsnrPsn != null && !bsnrPsn.isEmpty()) {
            try {
                int arztNummerPsn = Integer.parseInt(bsnrPsn);
                BDTArztpraxis bdtData = mainApp.getBdtData();
                if (bdtData != null) {
                    BDTValue<Integer> arztNummer = bdtData.getArztNummer();
                    arztNummer.setValue(arztNummerPsn);
                }
            } catch (NumberFormatException nfe) {
                log.error("bsnrPsn " + bsnrPsn + "could not converted into number.");
            }
        }
    }

    /**
     * Updates the old patient identifier to the given pseudonym
     * @param identifier_id the old identifier
     * @param pseudonymAis the ais pseudonym
     * @param pseudonymTmpAis temporary ais pseudonym
     */
    private void updatePseudonym(String identifier_id, String pseudonymAis, String pseudonymTmpAis) {
        for (Patient patient : lstPatient) {
            String localSystemPatientId = patient.localSystemPatientIdProperty().get();
            if (localSystemPatientId.equals(identifier_id)) {
                patient.setPseudonym(pseudonymAis);
                patientTable.refresh();
                // Update in BDT set
                BDTArztpraxis bdtData = mainApp.getBdtData();
                if (bdtData != null) {
                    HashMap<Long, BDTPatient> bdtPatients = bdtData.getPatients();
                    for (BDTPatient bdtPatient : bdtPatients.values()) {
                        BDTValue<String> patNr = bdtPatient.getPatientNummer();
                        if (patNr != null) {
                            if (identifier_id.equals(patNr.getValue())) {
                                patNr.setValue(pseudonymTmpAis);
                                mainApp.setPseudonymisationDone(true);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Method to create a patient list with all the data required by the trustee authority using patients list and the InformedConsens data.
     * @return the list
     */
    private List<THSPatient> buildTHSPatients() {
        List<THSPatient> thsPatients = new ArrayList<THSPatient>();
        int indexCount = 0;
        for (Patient patient : lstPatient) {
            if (patient.consentProperty().get()) {
                String localSystemPatientId = patient.localSystemPatientIdProperty().get();
                THSPatient thsPatient = new THSPatient(localSystemPatientId);
                thsPatient.setIndex(++indexCount);
                // patient
                thsPatient.setFirstName(patient.firstNameProperty().get());
                thsPatient.setLastName(patient.lastNameProperty().get());
                thsPatient.setBirthdate(patient.birthDateProperty().get());
                thsPatient.setGender(patient.genderProperty().get());
                thsPatient.setTreatingPractice(patient.treatingPracticeProperty().get());
                // contacts
                thsPatient.setCity(patient.cityProperty().get());
                thsPatient.setStreet(patient.streetProperty().get());
                thsPatient.setZipCode(patient.zipCodeProperty().get());
                // consents
                ObservableList<InformedConsent> lstInformedConsent = mainApp.getInformedConsentData();
                for (InformedConsent ic : lstInformedConsent) {
                    String icLocalSystemPatientId = ic.getLocalSystemPatientId();
                    if (localSystemPatientId.equals(icLocalSystemPatientId)) {
                        thsPatient.addConsentModule(InformedConsent.InformedConsentSection.Erhebung_Verarbeitung_Nutzung, ic.isConsentErhebungVerarbeitungNutzung());
                        thsPatient.addConsentModule(InformedConsent.InformedConsentSection.Erneute_Kontaktaufnahme, ic.isConsentErneuteKontaktaufnahme());
                        thsPatient.addConsentModule(InformedConsent.InformedConsentSection.Geltungsdauer, ic.isConsentGeltungsdauer());
                        thsPatient.addConsentModule(InformedConsent.InformedConsentSection.Widerrufsrecht, ic.isConsentWiderrufsrecht());
                    }
                }
                thsPatients.add(thsPatient);
            }
        }

        return thsPatients;
    }

    /**
     * Method that inits the helper class to connect to the trustee authority (Treuhandstelle [THS])
     */
    private void initTHSHelper() {
        thsHelper = new THSHelper();
        thsHelper.setURL(radarProperties.getProperty("ths_url"));
        thsHelper.setApikey(radarProperties.getProperty("apikey"));
        thsHelper.setKeystore("/certs/" + radarProperties.getProperty("certFileName"));
        thsHelper.setCertPassword(radarProperties.getProperty("certPassword"));
        thsHelper.setBasicAuthName(radarProperties.getProperty("basicAuthName"));
        thsHelper.init();
    }

    /**
     * Method to check if the trustee authority (Treuhandstelle [THS]) is reachable.
     */
    private void checkTHSConnection() {
        labelIsConnectionTHSAvailable.setFont(font);
        String result = thsHelper.testConnection(radarProperties.getProperty("ths_testconnection_url"));
        if (result.isEmpty()) {
            labelIsConnectionTHSAvailable.setText(fontProperties.getProperty("check-circle-o"));
            labelIsConnectionTHSAvailable.setTextFill(Color.DARKSEAGREEN);
        } else {
            labelIsConnectionTHSAvailable.setText(fontProperties.getProperty("times-circle"));
            labelIsConnectionTHSAvailable.setTextFill(Color.DARKRED);
            labelTHSErrorText.setText(result);
            Tooltip tooltip = new Tooltip();
            tooltip.setText("Keine Verbindung möglich!");
            buttonPseudonym.setTooltip(tooltip);
            buttonPseudonym.setDisable(true);
        }
    }

    /**
     * Loading and assigning the awesome-webfont to the labels and buttons
     */
    private void loadFont() {
        InputStream fontInputStream;
        fontInputStream = MainApp.class.getResourceAsStream("/fonts/fontawesome-webfont.ttf");
        font = Font.loadFont(fontInputStream, 20);
        labelIsTHSPasswordCorrect.setFont(font);
    }

    @Override
    public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;
        log.debug("got " + this.mainApp);

        if (this.mainApp != null) {
            BDTArztpraxis bdtData = mainApp.getBdtData();
            if (bdtData != null) {
                if (bdtData.getArztNummer() != null) {
                    Integer arztNummerInteger = bdtData.getArztNummer().getValue();
                    if (arztNummerInteger != null) {
                        // THS expects fixed length of nine digits, so fill up with zeros if shorter
                        this.arztnummer = String.format("%09d", arztNummerInteger);
                    }
                }
                HashMap<Long, BDTPatient> patients = bdtData.getPatients();
                for (BDTPatient patient : patients.values()) {
                    String patNr = "";
                    if (patient.getPatientNummer() != null)
                        patNr = patient.getPatientNummer().getValue();
                    String vorname = "";
                    if (patient.getVorName() != null)
                        vorname = patient.getVorName().getValue();
                    String nachname = "";
                    if (patient.getNachName() != null)
                        nachname = patient.getNachName().getValue();
                    String geburtsDatum = "";
                    LocalDate geburtsDatumDate;
                    if (patient.getGeburtsDatum() != null) {
                        geburtsDatumDate = patient.getGeburtsDatumDate();
                        DateTimeFormatter formatter = DateTimeFormatter.ISO_LOCAL_DATE;
                        geburtsDatum = geburtsDatumDate.format(formatter);
                    }
                    String geschlecht = "";
                    if (patient.getGeschlecht() != null) {
                        geschlecht = patient.getGeschlecht().getValue();
                        if (geschlecht.equals("1")) {
                            geschlecht = "M";
                        } else if (geschlecht.equals("2")) {
                            geschlecht = "F";
                        } else {
                            geschlecht = "U";
                        }
                    }
                    String wohnort = "";
                    String plz = "";
                    String strasse = "";
                    if (patient.getAdresse() != null) {
                        BDTAdresse adresse = patient.getAdresse();
                        if (adresse.getOrt() != null) {
                            wohnort = adresse.getCity();
                            plz = adresse.getZipCode();
                        }
                        if (adresse.getStrasse() != null) {
                            strasse = adresse.getStreet();
                        }
                    }

                    Patient newPatient = new Patient(patNr);
                    newPatient.setFirstName(vorname);
                    newPatient.setLastName(nachname);
                    newPatient.setBirthDate(geburtsDatum);
                    newPatient.setGender(geschlecht);
                    newPatient.setCity(wohnort);
                    newPatient.setZipCode(plz);
                    newPatient.setStreet(strasse);
                    newPatient.setTreatingPractice(this.arztnummer);
                    newPatient.setPseudonym("");
                    newPatient.setConsent(checkForConsent(patNr, vorname, nachname));

                    lstPatient.add(newPatient);
                }
                patientTable.setItems(lstPatient);

                consentColumn.setCellFactory(column -> new CheckBoxTableCell<>());
                consentColumn.setCellValueFactory(cellData -> {
                    BooleanProperty property = cellData.getValue().consentProperty();
                    return property;
                });
                pseudonymColumn.setCellValueFactory(cellData -> cellData.getValue().pseudonymProperty());
                localSystemPatientIdColumn.setCellValueFactory(cellData -> cellData.getValue().localSystemPatientIdProperty());
                firstNameColumn.setCellValueFactory(cellData -> cellData.getValue().firstNameProperty());
                lastNameColumn.setCellValueFactory(cellData -> cellData.getValue().lastNameProperty());
                birthDateColumn.setCellValueFactory(cellData -> cellData.getValue().birthDateProperty());
                genderColumn.setCellValueFactory(cellData -> cellData.getValue().genderProperty());
                cityColumn.setCellValueFactory(cellData -> cellData.getValue().cityProperty());
                zipCodeColumn.setCellValueFactory(cellData -> cellData.getValue().zipCodeProperty());
                streetColumn.setCellValueFactory(cellData -> cellData.getValue().streetProperty());
            }
        }

    }
    
    /**
     * Method to check fast if its the tablet workflow
     * @return true if it is the tablet workflow
     */
    public boolean isTabletWorkflow() {
        ObservableList<InformedConsent> lstInformedConsent = mainApp.getInformedConsentData();
        for (InformedConsent ic : lstInformedConsent) {
            // if already one is tablet workflow return true
            if (ic.isTabletWorkflow()) {
                return true;
            }
        }
        return false;
    }

    @Override
    public void setHostServices(HostServices hostServices) {
    }

    private Boolean checkForConsent(String patNr, String firstName, String lastName) {
        ObservableList<InformedConsent> lstInformedConsent = mainApp.getInformedConsentData();
        if (!patNr.isEmpty() && !firstName.isEmpty() && !lastName.isEmpty()) {
            for (InformedConsent ic : lstInformedConsent) {
                String localSystemPatientId = ic.getLocalSystemPatientId();
                String firstLetterFirstName = ic.getFirstLetterFirstName();
                String firstLetterLastName = ic.getFirstLetterLastName();
                if (localSystemPatientId.equals(patNr) && firstLetterFirstName.equals(firstName.substring(0, 1)) && firstLetterLastName.equals(lastName.substring(0, 1))) {
                    if (ic.isConsentErhebungVerarbeitungNutzung()) {
                        return Boolean.TRUE;
                    }
                }
            }
        }
        return Boolean.FALSE;
    }

    /**
     * Method to initialise the lable with the table headline with the start and end date used to filter the patients
     */
    private void initialiseTimeSpanLabel() {
        String startDate = radarProperties.getProperty("radar_filter_from");
        String endDate = radarProperties.getProperty("radar_filter_to");
        labelTimeSpanFilter.setText("Patienten aus dem Zeitraum vom " + startDate + " bis " + endDate);
    }

}