package de.uni_goettingen.med.mi.radar_software.view;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.time.LocalDateTime;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.concurrent.LinkedTransferQueue;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.bdtparser.core.BDTMarshaller;
import de.uni_goettingen.med.mi.bdtparser.core.io.reader.BdtFileReader;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTFileSaverViewer;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTVPRCreator;
import de.uni_goettingen.med.mi.radar_software.MainApp;
import javafx.application.Platform;
import javafx.concurrent.Task;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.geometry.Point2D;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.stage.DirectoryChooser;
import javafx.stage.Stage;


/**
 * The controller for the viewer mode view.
 * The viewer allows to parse a already pseudonized bdt-files and to display their content, which is not possible with
 * the default bdet-parser mode.
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class ViewerController implements Initializable {

    private static final Logger log = LogManager.getLogger(ViewerController.class);

    @FXML
    private AnchorPane viewerPane;

    @FXML
    private ImageView logo;

    @FXML
    private Label labelInfo;

    @FXML
    private TextField textFieldPathBDTFiles;

    @FXML
    private Button buttonDirectoryChooser;

    @FXML
    private Label labelIsBdtPathCorrect;

    @FXML
    private CheckBox checkBoxFilter;

    private final LinkedTransferQueue<String> input = new LinkedTransferQueue<String>();
    private BDTMarshaller bdtMarshaller;
    private boolean isBdtFileInFolder = false;
    private SummaryController summaryController;
    private MainApp mainApp;
    private FXMLLoader fxmlLoader;
    private static Properties fontProperties = loadFontProperties();
    private static Properties radarProperties = loadRadarProperties();
    private Font fontLabel;
    private Font fontButton;

    public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;
        initializeSummaryView();
    }

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        log.debug("initializing Viewer Mode");
        loadFont();
        initializeInfoLabel();
        initializeOpenFolderButton();
        initializeBDTPathLable();
        Image img = new Image("/images/LogoRadar_engl.jpg");
        logo.setImage(img);
    }

    /**
     * Loading the radar properties from file.
     */
    protected static Properties loadRadarProperties() {
        Properties radarProperties = new Properties();
        InputStream radar = MainApp.class.getResourceAsStream("/RADAR.properties");
        try {
            radarProperties.load(radar);
        } catch (IOException ioe) {
            log.error(ioe.getMessage());
        }
        return radarProperties;
    }

    /**
     * Loading the font variables from file.
     */
    protected static Properties loadFontProperties() {
        fontProperties = new Properties();
        InputStream font = MainApp.class.getResourceAsStream("/fonts/fontawesome.properties");
        try {
            fontProperties.load(font);
        } catch (IOException ioe) {
            log.error(ioe.getMessage());
        }
        return fontProperties;
    }

    /**
     * Loading and assigning the awesome-webfont to the labels and buttons
     */
    private void loadFont() {
        InputStream fontInputStream;
        fontInputStream = MainApp.class.getResourceAsStream("/fonts/fontawesome-webfont.ttf");
        fontLabel = Font.loadFont(fontInputStream, 20.0);
        fontInputStream = MainApp.class.getResourceAsStream("/fonts/fontawesome-webfont.ttf");
        fontButton = Font.loadFont(fontInputStream, 14.0);
    }

    /**
     * Check if the entered bdtpath is a folder and the folder exists. If the folder exists check the contained files if there is at least one file of the type bdt or xdt.
     */
    private void checkBdtPath() {
        File bdtFolder = new File(textFieldPathBDTFiles.getText());
        isBdtFileInFolder = false;
        if (bdtFolder.exists() && bdtFolder.isDirectory()) {
            String fileName;
            File[] listOfFiles = bdtFolder.listFiles();
            if (listOfFiles != null) {
                for (int i = 0; i < listOfFiles.length; i++) {
                    if (listOfFiles[i].isFile()) {
                        fileName = listOfFiles[i].getName();
                        if (fileName.toUpperCase().contains(".BDT") || fileName.toUpperCase().contains(".XDT")) {
                            isBdtFileInFolder = true;
                            return;
                        }
                    }
                }
            }
        }
    }

    /**
     * Method to start the parsing process and display the results.
     */
    public void triggerBDTView() {
        checkBdtPath();
        updateStatusIcon();
        actionStartBdtParser();
    }

    /**
     * method to update the displayed status icons and tooltips  depending if the given path is correct and the folder holds bdt-files
     */
    private void updateStatusIcon() {
        Tooltip tooltip = new Tooltip();
        if (isBdtFileInFolder) {
            labelIsBdtPathCorrect.setText(fontProperties.getProperty("check-circle-o"));
            labelIsBdtPathCorrect.setTextFill(Color.DARKSEAGREEN);
            tooltip.setText("OK!");
        } else {
            labelIsBdtPathCorrect.setText(fontProperties.getProperty("times-circle"));
            labelIsBdtPathCorrect.setTextFill(Color.DARKRED);
            tooltip.setText("Im angegebenen Pfad wurden keine BDT-Dateien gefunden!");
        }
        labelIsBdtPathCorrect.setTooltip(tooltip);
    }

    /**
     * Starting the BDT-Parser, triggered by start button.
     */
    @FXML
    public void actionStartBdtParser() {
        if (isBdtFileInFolder) {
            mainApp.setUSBPath(new File(textFieldPathBDTFiles.getText()).getAbsolutePath());
            mainApp.setStartTime(LocalDateTime.now());
            Task<Void> BDTInputTask = new Task<Void>() {
                @Override
                public Void call() throws Exception {
                    BdtFileReader bdtFileReader = new BdtFileReader(input, textFieldPathBDTFiles.getText());
                    Thread bdtFileReaderThread = new Thread(bdtFileReader);
                    bdtFileReaderThread.start();
                    bdtMarshaller = new BDTMarshaller(bdtFileReaderThread, bdtFileReader);
                    Thread marshallerThread = new Thread(bdtMarshaller);
                    marshallerThread.start();
                    try {// wait until marshaller is done
                        marshallerThread.join();
                        /*
                         * is a blocking call, so it will block the FX Application thread until the background thread completes. This will prevent the UI from being repainted, or
                         * from responding to user input. The easiest way to do what you want is to use a Task:
                         */
                    } catch (InterruptedException ie) {
                        ie.printStackTrace();
                    }
                    return null;
                }
            };
            BDTInputTask.setOnSucceeded(e -> {
                vprCreationTask();
                summaryController.triggerDrawTree(mainApp);
                log.debug("read all files and marshalled them");
            });
            BDTInputTask.setOnFailed(e -> {
                log.error("read files and marshall failed");
            });
            new Thread(BDTInputTask).start();
        }
    }

    /**
     * Method to open the direcotory chooser if the given button is clicked
     */
    public void actionOpenDirectoryChooser() {
        DirectoryChooser directoryChooser = new DirectoryChooser();
        directoryChooser.setTitle("Pfadauswahl BDT-Ordner");
        File selectedFile = directoryChooser.showDialog(null);
        if (selectedFile != null) {
            textFieldPathBDTFiles.setText(selectedFile.getAbsolutePath());
            triggerBDTView();
        }
    }

    /**
     * Method to start the creation of the virtual patient records
     */
    private void vprCreationTask() {
        BDTVPRCreator bdtVprCreator = new BDTVPRCreator();
        if (!checkBoxFilter.isSelected()) {
            bdtVprCreator.activateViewerMode();
        }
        mainApp.setBdtData(bdtVprCreator.createPatients(bdtMarshaller.datapackage));
        mainApp.setBdtMarshaller(bdtMarshaller);
        mainApp.setBDTMetaData(bdtVprCreator.getMetadata());
        log.debug("VPR creation done");
    }

    /**
     * init function for the info label
     */
    private void initializeInfoLabel() {
        labelInfo.setFont(fontLabel);
        labelInfo.setText(fontProperties.getProperty("info"));
        labelInfo.setTextFill(Color.DARKBLUE);
    }

    /**
     * init function for the open folder button (add the open folder icon)
     */
    private void initializeOpenFolderButton() {
        buttonDirectoryChooser.setFont(fontButton);
        buttonDirectoryChooser.setText(fontProperties.getProperty("folder-open-o"));
    }
    /**
     * init function for the BDT-Path label
     */
    private void initializeBDTPathLable(){
        labelIsBdtPathCorrect.setFont(fontLabel);
        labelIsBdtPathCorrect.setText(fontProperties.getProperty("times-circle"));
    }

    /**
     * Method to set and display the content of the tooltips.
     */
    @FXML
    private void showLabelInfoTooltip() {
        Stage primaryStage = mainApp.getPrimaryStage();
        Point2D p = labelInfo.localToScene(0.0, 0.0);
        final Tooltip tooltip = new Tooltip();
        tooltip.setStyle("-fx-font-size: 16");
        String radar_version = radarProperties.getProperty("radar_version_number");
        tooltip.setText("Institut für Medizinsiche Informatik\nRADAR Software Prototyp\nVersion: " + radar_version + "\n");
        labelInfo.setTooltip(tooltip);
        tooltip.setAutoHide(true);
        tooltip.show(primaryStage, p.getX()
            + labelInfo.getScene().getX() + labelInfo.getScene().getWindow().getX(), p.getY()
            + labelInfo.getScene().getY() + labelInfo.getScene().getWindow().getY());
    }

    /**
     * Method to initialize the embedded summary view.
     */
    public void initializeSummaryView() {
        try {
            fxmlLoader = new FXMLLoader();
            fxmlLoader.setResources(ResourceBundle.getBundle("fonts/fontawesome"));
            fxmlLoader.load(getClass().getResourceAsStream("/fxml/5_Summary.fxml"));
            viewerPane.getChildren().add(fxmlLoader.getRoot());
            summaryController = fxmlLoader.getController();
            summaryController.activateViewerMode();
            summaryController.setMainApp(this.mainApp);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    /**
     * Method to save the current displayed data to a bdt-file.
     */
    public void save() {
            String directoryPath = System.getProperty("user.dir");
            long writeTime = System.currentTimeMillis();
            String filePath = directoryPath + "/bdtfiles-" + writeTime;
            if (mainApp.getBdtData() != null) {
                new BDTFileSaverViewer(bdtMarshaller.datapackage, filePath).save(mainApp.getBdtData());
                log.info("Saved current bdt data to file");
            } else {
                log.debug("no bdt data to write");
            }
    }

    /**
     * close the programm if the exit button was clicked
     */
    public void clickExit() {
        Platform.exit();
    }
    
}